/*
 * CreateGaborFilter1D.c
 *
 *  Function creates the m'th filter in a 1D multiband Gabor filter bank.  
 *  A pointer to the new filter is returned through the function name.  
 *
 *  The filter design equations are found on pp. 96 - 100 of book VIII.
 *
 *  jph 9/23/94
 *
 */

#ifndef CREATEGABORFILTER1D_C
#define CREATEGABORFILTER1D_C
#include "teager2D.h"
#include "Esa1D.h"
#include <stdio.h>

FilterPtr CreateGaborFilter1D(G,m)

  filterbank *G;      /* multiband Gabor filter bank */
  int         m;      /* index in the bank of filter to create */

{
  filter *f;             /* pointer to the new filter */
  double  twopi;         /* two time pi */

 /*
  * Allocate the filter on the heap
  */
  if ((f = (filter *)malloc(sizeof(filter))) == NULL) {
    printf("\n\nCreateGaborFilter: free store exhausted.\n");
    exit(-1);
  }

 /*
  * Useful factors
  */
  twopi = (double)2.0 * M_PI;

 /*
  * Assigned filter parameters
  */
  f->index = m;
  f->Eta   = G->Eta;
  f->OBW   = G->OBW;
  f->gamma = G->gamma;
  f->kappa = G->kappa;

 /*
  * Computed filter parameters
  */
  f->Ray = 0;
  f->IndexOnRay = m;
  f->Theta      = G->Theta0;
  f->r0 = G->r0 * pow((double)G->R, (double)f->IndexOnRay);
  f->u0 = f->r0;
  f->v0 = (double)0.0;

  f->sigma = pow(-log((double)f->Eta) ,(double)0.5) 
             / (twopi * f->r0 * pow(f->gamma, (double)0.5));
  f->gMax  = (float)pow((double)((double)2.0 * M_PI * f->sigma * f->sigma),
                                 (double)-0.25);
  f->GMax  = (float)(pow((double)2.0,(double)0.5)/(double)f->gMax);

  f->gArgGaussCoef = (double)-0.25 / pow(f->sigma, (double)2.0);
  f->GArgGaussCoef = M_PI * M_PI / f->gArgGaussCoef;

  return(f);
}
#endif 
