/*
 * GMakeReal1D.c
 *
 *  Function returns a float array equal to the Real part of the spectrum of
 *  the specified gabor filter.  The float array is allocated on the heap.
 *
 *  jph 9/24/94
 *
 */

#ifndef GMAKEREAL1D_C
#define GMAKEREAL1D_C
#include "teager2D.h"
#include "Esa1D.h"
#include <stdio.h>

float *GMakeReal1D(f,size)

  FilterPtr   f;      /* filter whose center freq is to be used */
  int         size;   /* image row (col) dimension */

{
  float      *GReal;     /* array of filter Real values */
  int         i;         /* loop counter */
  int         so2;       /* size over 2 */
  int         k;         /* indexes the array */
  double      u;         /* freq in cycles per pixel */
  double      fincr;     /* increment in freq for one sample */

 /*
  * Allocate the filter on the heap
  */
  if ((GReal = (float *)malloc(size*sizeof(float))) == NULL) {
    printf("\n\nGMakeReal1D: free store exhausted.\n");
    exit(-1);
  }

 /*
  * Useful values 
  */
  so2 = size >> 1;
  fincr = (double)1.0 / (double) size;

 /*
  * Make the data
  */
  for (k=0, i=-so2; i < so2; i++) {
    u = (double)i * fincr;
    GReal[k++] = G_EvalReal1D(f,u);
  }

  return(GReal);

}
#endif
