/*
 * MkHardy1D.c
 *
 *  Function turns a real signal into it's associated "analytic" function by
 *    taking the fft of the real signal
 *    zeroing the negative frequency samples and the +/- N/2 sample
 *    taking the inverse fft
 *
 *  SEE MODIFICATION NOTE BELOW.
 *
 *  No memory allocation is performed.  All four input float arrays must be
 *  malloc'd by the calling routine.
 *
 *  jph 9/24/94
 *
 *
 *-----------------------
 *
 *  MOD 12/19/2000:
 *
 *  The analytic spectrum is being calculated wrong -- not consistent with
 *  Cizek or with my dissertation.  Old version zeros all negative frequency
 *  samples and the +/- N/2 frequency sample.  This is incorrect.
 *
 *  Should be: +/- N/2 samples:       leave alone
 *             1,2,...,N/2-1:         zero
 *             N/2+1, N/2+2,...,N-1:  double
 *
 *  See book XII, pp. 94-97.
 *
 *
 */

#ifndef MAKEHARDY1D_C
#define MAKEHARDY1D_C
#include "teager2D.h"
#include "Esa1D.h"
#include <stdio.h>

void MakeHardy1D(xReal,xImag,XReal,XImag,size)

  float *xReal;       /* real part of signal */
  float *xImag;       /* imaginary part of signal */
  float *XReal;       /* real part of spectrum */
  float *XImag;       /* imaginary part of spectrum */
  int    size;        /* length of signal */

{
  int i;                        /* loop counter */
  int so2;                      /* size over 2 */

 /*
  * Get the spectrum of the real signal
  */  
  for (i=0; i<size; i++) {
    xImag[i] = (float)0.0;
  }
  fft1d(xReal,xImag,XReal,XImag,size,1);
    
 /*
  * Zero the negative frequency samples.
  */
  so2 = size >> 1;
  for (i=1; i<so2; i++) {
    XReal[i] = XImag[i] = (float)0.0;
    XReal[i+so2] *= (float)2.0;
    XImag[i+so2] *= (float)2.0;
  }

 /*
  * Get the Hardy image by inverse fft
  */
  fft1d(xReal,xImag,XReal,XImag,size,-1);

  return;

}
#endif 
