/*
 * MkPostFilt1D.c
 *
 *   Function returns a pointer to a Gaussian postfilter sequence.  The filter 
 *   has zero phase, which means that the main lobe is split across the two 
 *   ends of the sequence instead of being centered.
 *   See book XI p. 119.
 *
 *   The Post Filters must have unit AREA, not unit L2 norm.
 *
 *  jph 4/12/96
 *
 */
#ifndef MKPOSTFILT1D_C
#define MKPOSTFILT1D_C
#include <stdio.h>
#include "teager2D.h"

pFloatImage MkPostFilt1D(sigma,kappa,size)

  double      sigma;  /* channel filter space constant */
  float       kappa;  /* postfilter space constant scaling factor */
  int         size;   /* size of signal */

{

  pFloatImage g;      /* ptr to filter image being created */
  float       gMax;   /* max val of filter in space domain */
  double      gFact;  /* phase factor for space domain */
  int         k;      /* counter */
  int         i;      /* counter */
  int         xx;     /* col argument for zerophase */
  int         yy;     /* row argument for zerophase */
  int         so2;    /* size / 2 */

 /*
  * Allocate the floating image and its constituents
  */
  so2 = size >> 1;
  if ((g = (pFloatImage)malloc(sizeof(FloatImage))) == NULL) {
    printf("\nMkPostFilt: Free Store Exhausted.\n\n");
    exit(-1);
  }
  g->size = g->cols = size;
  g->square = 0;
  g->rows = 1;

  if ((g->Real = (float *)malloc(size * sizeof(float))) == NULL) {
    printf("\nMkPostFilt1D: Free Store Exhausted.\n\n");
    exit(-1);
  }
  if ((g->Imag = (float *)malloc(size * sizeof(float))) == NULL) {
    printf("\nMkPostFilt1D: Free Store Exhausted.\n\n");
    exit(-1);
  }
  if ((g->FFTReal = (float *)malloc(size * sizeof(float))) == NULL) {
    printf("\nMkPostFilt1D: Free Store Exhausted.\n\n");
    exit(-1);
  }
  if ((g->FFTImag = (float *)malloc(size * sizeof(float))) == NULL) {
    printf("\nMkPostFilt1D: Free Store Exhausted.\n\n");
    exit(-1);
  }

 /*
  * Compute Factors 
  */
  gMax = (float)pow((double)4.0*(double)M_PI*(double)(kappa*kappa)*sigma*sigma, 
                           (double)-0.5);
  gFact = (double)-0.25 / ((double)(kappa*kappa) * sigma * sigma);

 /*
  * Compute filter in space domain
  */
  for (i=0; i < size; i++) {
    k = (i < so2) ? i : i - size;
    g->Real[i] = gMax * (float)exp(gFact * (double)(k*k));
    g->Imag[i] = (float)0.0;
  }

 /*
  * Compute filter in freq domain by FFT
  */
  fft1d(g->Real,g->Imag,g->FFTReal,g->FFTImag,size,1);

  return(g);
}
#endif
