/*
 * ReComputeAsm1D.c
 *
 *  Compute amplitude estimates from smoothed frequency estimates following
 *  postfiltering of the frequency estimates.
 *
 *  See Book XI, p. 119.
 *
 *  jph 4/12/96
 */

#ifndef RECOMPUTEASM1D_C
#define RECOMPUTEASM1D_C
#include <stdio.h>
#include "teager2D.h"
#include "Esa1D.h"

#define FLOAT_TOO_SMALL ((float)0.001)

void ReComputeAsm1D(t,Usm,Asm,G)

  pFloatImage   t;                  /* response image */
  float        *Usm;                /* postfiltered frequency estimates */
  float        *Asm;                /* smoothed amplitude estimates */
  FilterPtr     G;                  /* pointer to the Gabor filter */
{

  int           i;                  /* counter */
  float         FilterMag;          /* magnitude of filter at estimated u,v */

  for (i=0; i < t->size; i++) {
    FilterMag = G_EvalMag1D(G,fabs((double)Usm[i]));
    if ((FilterMag / G->GMax) > FLOAT_TOO_SMALL) { 
      Asm[i] = FNorm(t->Real[i],t->Imag[i]) / FilterMag;
    } else {                       
      Asm[i] = (float)0.0;
    } 
  }

  return;
} 
#endif
