/*
 * fBackDiff1D.c
 *
 *  Function computes the first finite backward difference of a float signal.
 *  For the first point, the difference is set equal to the difference at
 *  the second point.
 *
 *  The output array is allocated on the heap.
 *
 *  jph 10/28/98
 *
 */

#ifndef FBACKDIFF1D_C
#define FBACKDIFF1D_C
#include "teager2D.h"
#include "Esa1D.h"
#include <stdio.h>

float *fBackDiff1D(x,size)

  float      *x;      /* input float signal */
  int         size;   /* size of x array */

{
  int         i;         /* loop counter */
  float      *y;         /* output difference signal */

 /*
  * Allocate the output on the heap
  */
  if ((y = (float *)malloc(size*sizeof(float))) == NULL) {
    printf("\n\nfBackDiff1D: free store exhausted.\n");
    exit(-1);
  }

 /*
  * Compute difference at second point, replicate this value for first point.
  */
  y[0] = y[1] = x[1] - x[0];

 /*
  * Compute difference at all subsequent points.
  */
  for (i=2; i < size; i++) {
    y[i] = x[i] - x[i-1];
  }

  return(y);

}
#endif
