/*
 * gMakeImag1D.c
 *
 *  Function returns a float array equal to the Imaginary part of the filter 
 *  function of the specified gabor filter.  If zerophase=1, a zero phase 
 *  filter is returned.  If zerophase=0, a filter centered at the center of 
 *  the domain axis. The float array is allocated on the heap.
 *
 *  jph 9/24/94
 *
 */

#ifndef gMAKEIMAG1D_C
#define gMAKEIMAG1D_C
#include "teager2D.h"
#include "Esa1D.h"
#include <stdio.h>

float *gMakeImag1D(f,zerophase,size)

  FilterPtr   f;         /* filter to be evaluated */
  int         zerophase; /* 1: zero-phase filter, 0: filter centered in plane */
  int         size;      /* row (col) dimension of the image */

{

  float      *gImag;     /* array of filter Real values */
  int         row;       /* loop counter for image rows */
  int         col;       /* loop counter for image cols */
  int         i;         /* indexes the array */

 /*
  * Allocate the filter on the heap
  */
  if ((gImag = (float *)malloc(size*sizeof(float))) == NULL) {
    printf("\n\ngMakeImag1D: free store exhausted.\n");
    exit(-1);
  }
  
 /*
  * Make the data
  */
  for (i=0; i < size; i++) {
    gImag[i] = g_EvalImag1D(f,(double)i,zerophase,size);
  }

  return(gImag);

}
#endif
