/*
 * gMakeMag1D.c
 *
 *  Function returns a float array equal to the magnitude of the filter 
 *  function of the specified gabor filter.  If zerophase=1, a zero phase 
 *  filter is returned.  If zerophase=0, a centered filter is returned.  
 *  The float array is allocated on the heap.
 *
 *  jph 9/94/94
 *
 */

#ifndef gMAKEMAG1D_C
#define gMAKEMAG1D_C
#include "teager2D.h"
#include "Esa1D.h"
#include <stdio.h>

float *gMakeMag1D(f,zerophase,size)

  FilterPtr   f;         /* filter to be evaluated */
  int         zerophase; /* 1: zero-phase filter, 0: filter centered in plane */
  int         size;      /* row (col) dimension of the image */

{

  float      *gMag;      /* array of filter Real values */
  int         i;         /* indexes the array */

 /*
  * Allocate the filter on the heap
  */
  if ((gMag = (float *)malloc(size*sizeof(float))) == NULL) {
    printf("\n\ngMakeMag1D: free store exhausted.\n");
    exit(-1);
  }
  
 /*
  * Make the data
  */
  for (i=0; i < size; i++) {
      gMag[i] = g_EvalMag(f,(double)i,zerophase,size);
  }

  return(gMag);

}
#endif
