/*
 * gMakeReal1D.c
 *
 *  Function returns a float array equal to the Real part of the filter 
 *  function of the specified gabor filter.  If zerophase=1, a zero phase 
 *  filter is returned.  If zerophase=0, a centered filter is returned.  
 *  The float array is allocated on the heap.
 *
 *  jph 9/24/94
 *
 */

#ifndef gMAKEREAL1D_C
#define gMAKEREAL1D_C
#include "teager2D.h"
#include "Esa1D.h"
#include <stdio.h>

float *gMakeReal1D(f,zerophase,size)

  FilterPtr   f;         /* filter to be evaluated */
  int         zerophase; /* 1: zero-phase filter, 0: filter centered in plane */
  int         size;      /* row (col) dimension of the image */

{

  float      *gReal;     /* array of filter Real values */
  int         i;         /* indexes the array */

 /*
  * Allocate the filter on the heap
  */
  if ((gReal = (float *)malloc(size*sizeof(float))) == NULL) {
    printf("\n\ngMakeReal1D: free store exhausted.\n");
    exit(-1);
  }
  
 /*
  * Make the data
  */
  for (i=0, i=0; i < size; i++) {
    gReal[i] = g_EvalReal1D(f,(double)i,zerophase,size);
  }

  return(gReal);

}
#endif
