/*
 * g_EvalReal1D.c
 *
 *  Function image inputs x domain coordinate, a filter, and the length
 *  of the data sequence.  The Real value of the filter at the point x is 
 *  returned.  If zerophase=1, the value for a zero phase filter is returned.  
 *  If zerophase=0, the value for a centered filter is returned.
 *
 *  jph 9/24/94
 *
 */

#ifndef g_EVALREAL1D_C
#define g_EVALREAL1D_C
#include "teager2D.h"
#include "Esa1D.h"
#include <stdio.h>

float g_EvalReal1D(f,x,zerophase,size)

  FilterPtr   f;      /* filter to be evaluated */
  double      x;      /* domain coordinate at which to evaluate filter */
  int         zerophase; /* 1: zero-phase filter, 0: filter centered */
  int         size;   /* row (col) dimension of the image */

{

  int         so2;    /* size / 2 */
  float       xx;     /* domain coordinate at which to evaluate filter */
  double      twopi;  /* two time pi */


  so2   = size >> 1;
  twopi = (double)2.0 * M_PI;

  if (zerophase) {
    xx = (x < so2) ? x : x - size;
  }
  else {
    xx = x - so2;
  }
  return(f->gMax * (float)
                   (exp(f->gArgGaussCoef * pow((double)xx,(double)2.0))
                   * cos(twopi*(f->u0*(double)xx))));
}
#endif
