/*
 * ConvolveGabor.c
 *
 *  Function convolves input image x with a specified gabor filter by
 *  multiplication in the frequency domain.  The image domain and spectrum
 *  representations of Y are computed.
 *
 *  On invocation, it is assumed that the X and Y images have the same size and 
 *  have been fully allocated, and that the X image has been fully initialized. 
 *  It is also assumed that the size field of the Y image has been initialized.
 *
 *  jph 2/11/94
 */

#ifndef CONVOLVEGABOR_C
#define CONVOLVEGABOR_C
#include <stdio.h>
#include "teager2D.h"

void ConvolveGabor(Y,X,G)

  pFloatImage   Y;  /* output image */
  pFloatImage   X;  /* output image */
  FilterPtr     G;  /* pointer to the Gabor filter */
{

  double      u;       /* horizontal freq in cycles per pixel */ 
  double      v;       /* horizontal freq in cycles per pixel */ 
  double      fincr;   /* incr in freq for one row or one col in the image */
  int         row;     /* counter for image rows */
  int         col;     /* counter for image cols */
  int         k;       /* array offset index */
  int         looplim; /* terminal/initial value for loop counters */
  int         so2;     /* size over 2 */

 /*
  * Useful values
  */
  fincr = (double)1.0 / (double)X->size;
  so2   = X->size >> 1;

 /*
  * Compute the spectrum of the output
  */
  for (k=row=0; row < X->size; row++) {
    v = (double)(row-so2) * fincr;
    for (col=0; col < X->size; col++,k++) {
      u = (double)(col-so2) * fincr;  
      fCMult(&Y->FFTReal[k],    &Y->FFTImag[k], 
              X->FFTReal[k],     X->FFTImag[k],
              G_EvalReal(G,u,v), G_EvalImag(G,u,v));
    }
  }

 /*
  * Compute the image domain response by inverse fft
  */
  fft2d(Y->Real,Y->Imag,Y->FFTReal,Y->FFTImag,Y->size,-1);

  return;
} 
#endif
