/*
 * CreateDCFilt.c
 *
 *  Function creates the DC filter to go with the multiband filter bank.
 *  This filter extracts the lo-pass image component as explained in
 *  our paper in Trans IP June 1996.  See book X, p. 99 for the filter
 *  design.
 *
 *  A pointer to the new filter is returned through the function name.  
 *
 *  NOTE: 9/1/96:  This routine has been superceded by DesignDCfilt.c
 *
 *  jph 11/3/95
 *
 */

#ifndef CREATEDCFILT_C
#define CREATEDCFILT_C
#include "teager2D.h"

FilterPtr CreateDCFilter(G,DCR)

  filterbank *G;      /* multiband Gabor filter bank */
  int         DCR;    /* the eta-peak contour of the DC filter is at a radial */
                      /* frequency equal to the center frequency of the  */
                      /* DCR'th ring of filters in the filterbank. */
{
  filter *f;             /* pointer to the new filter */
  double  twopi;         /* two time pi */

 /*
  * Allocate the filter on the heap
  */
  if ((f = (filter *)malloc(sizeof(filter))) == NULL) {
    printf("\n\nCreateDCFilter: free store exhausted.\n");
    exit(-1);
  }

 /*
  * Useful factors
  */
  twopi = (double)2.0 * M_PI;

 /*
  * Assigned filter parameters
  */
  f->index = 0;
  f->Eta   = G->Eta;
  f->gamma = G->gamma;
  f->isotropicPF = G->isotropicPF;
  f->kappa = G->kappa;
  f->kappa_x = G->kappa_x;
  f->kappa_y = G->kappa_y;

 /*
  * Computed filter parameters
  */
  f->Ray        = 0;
  f->IndexOnRay = 0;
  f->Theta      = (double)0.0;
  f->r0 = (double)0.0;
  f->u0 = (double)0.0;
  f->v0 = (double)0.0;
  f->OBW = (float)0.0;
  f->sigma = pow(-log((double)f->Eta),(double)0.5) / (twopi * G->filt[DCR]->r0);
  f->gMax  = (float) ((double)1.0 / (pow(twopi, (double)0.5) * f->sigma));
  f->GMax  = (float)2.0 / f->gMax;
  f->gArgGaussCoef = (double)-0.25 / pow(f->sigma, (double)2.0);
  f->GArgGaussCoef = M_PI * M_PI / f->gArgGaussCoef;

  return(f);
}
#endif 
