/*
 * DIFFARRAY.c
 *
 *   Function subtracts two float arrays and prints the statistics of the
 *   difference image to standard out.
 *
 *  jph 9 Feb 1994
 */

#ifndef DIFFARRAY_C
#define DIFFARRAY_C
#include <stdio.h>
#include "teager2D.h"

void DIFFARRAY(x,y,size)

  float *x, *y;   /* images to be differenced */
  int    size;    /* linear dimension of the arrays */
{

  float   *z;     /* difference image */
  float    min;   /* minimum of difference image */
  float    med;   /* median of difference image */
  float    max;   /* maximum of difference image */
  float    mean;  /* mean of difference image */
  float    sigma; /* std deviation of difference image */
  int      i;     /* loop counter */

 /* 
  * allocate memory for z, return error if out of memory
  */
  z = (float *) malloc(size * sizeof(float));
  if (z == NULL) {
    printf("\nDIFFARRAY : out of memory error !\n");
    exit(-1);
  }

 /*
  * set z = x-y
  */
  for (i=0; i < size; i++) {
    *(z + i) = (*(x + i)) - (*(y + i));
  }
  
 /*
  * Compute the statistics of z
  */
  fsstat(z, size, 1, &min, &med, &max, &mean, &sigma);

 /*
  * Report
  */
  printf("\n\nstatistics of difference image:");
  printf("\n\tmin:   \t%f", min);
  printf("\n\tmed:   \t%f", med);
  printf("\n\tmean:  \t%f", mean);
  printf("\n\tmax:   \t%f", max);
  printf("\n\tsigma: \t%f\n", sigma);

 /*
  * Clean house
  */
  free(z);

  return;

} 
#endif
