/*
 * DesignDCfilt.c
 *
 *  Function creates the DC filter in a multiband Gabor filter bank.  
 *  A pointer to the new filter is returned through the function name.  
 *  
 *  The DC filter eta-peak contour and the eta-peak contours of any two
 *  adjacent filters on the innermost ring intersect at a single 
 *  frequency.
 *
 *  The DC filter octave bandwidth is not OBW !!!
 *
 *  The DC filter does have unit L2 norm.
 *
 *  jph 9/1/96
 *
 */

#ifndef DESIGNDCFILT_C
#define DESIGNDCFILT_C
#include "teager2D.h"
#include <stdio.h>

FilterPtr DesignDCfilt(G)

  filterbank *G;      /* multiband Gabor filter bank */

{
  filter *f;             /* pointer to the new filter */
  double  twopi;         /* two time pi */
  double  EtaPeakRad;    /* Eta-peak radius of DC filter IN RADIANS */

 /*
  * Allocate the filter on the heap
  */
  if ((f = (filter *)malloc(sizeof(filter))) == NULL) {
    printf("\n\nCreateGaborFilter: free store exhausted.\n");
    exit(-1);
  }

 /*
  * Useful factors
  */
  twopi = (double)2.0 * M_PI;

 /*
  * Assigned filter parameters
  */
  f->index = G->TotNumFilters;
  f->Eta   = G->Eta;
  f->OBW   = 0;
  f->gamma = G->gamma;
  f->isotropicPF = G->isotropicPF;
  f->kappa = G->kappa;
  f->kappa_x = G->kappa_x;
  f->kappa_y = G->kappa_y;

 /*
  * Computed filter parameters
  */
  f->Ray        = G->NumRays;
  f->IndexOnRay = 0;
  f->Theta      = 0;

  f->r0 = (double)0.0;
  f->u0 = (double)0.0;
  f->v0 = (double)0.0;

  EtaPeakRad = twopi * G->r0 * cos((double)0.5 * G->Lambda)
               - pow(
                      ( (double)-1.0 * log((double)G->Eta)
                          /(G->filt[0]->sigma * G->filt[0]->sigma)
                      )
                    - (
                         twopi*twopi*G->r0*G->r0 
                         * sin((double)0.5*G->Lambda) 
                            * sin((double)0.5*G->Lambda)
                      ),(double)0.5
                     );                 /* pow */

  f->sigma = (float)(pow((double)-1.0*log((double)G->Eta),(double)0.5) 
                 / EtaPeakRad);

  f->gMax  = (float) ((double)1.0 / (pow(twopi, (double)0.5) * f->sigma));
  f->GMax  = (float)2.0 / f->gMax;

  f->gArgGaussCoef = (double)-0.25 / pow(f->sigma, (double)2.0);
  f->GArgGaussCoef = M_PI * M_PI / f->gArgGaussCoef;

  return(f);
}
#endif 
