/*
 * GMakeReal.c
 *
 *  Function returns a float array equal to the Real part of the spectrum of
 *  the specified gabor filter.  The image is assumed square, with row (col) 
 *  dimension size.  The float array is allocated on the heap.
 *
 *  jph 9/10/93
 *
 */

#ifndef GMAKEREAL_C
#define GMAKEREAL_C
#include "teager2D.h"
#include <stdio.h>

float *GMakeReal(f,size)

  FilterPtr   f;      /* filter whose center freq is to be used */
  int         size;   /* image row (col) dimension */

{
  float      *GReal;     /* array of filter Real values */
  int         row;       /* loop counter for image rows */
  int         col;       /* loop counter for image cols */
  int         so2;       /* size over 2 */
  int         k;         /* indexes the array */
  double      u;         /* horizontal freq in cycles per pixel */
  double      v;         /* vertical freq in cycles per pixel */
  double      fincr;     /* increment in freq for one row or one col */

 /*
  * Allocate the filter on the heap
  */
  if ((GReal = (float *)malloc(size*size*sizeof(float))) == NULL) {
    printf("\n\nGMakeReal: free store exhausted.\n");
    exit(-1);
  }

 /*
  * Useful values 
  */
  so2 = size >> 1;
  fincr = (double)1.0 / (double) size;

 /*
  * Make the data
  */
  for (k=0, row=-so2; row < so2; row++) {
    v = (double)row * fincr;
    for (col=-so2; col < so2; col++, k++) {
      u = (double)col * fincr;
      GReal[k] = G_EvalReal(f,u,v);
    }
  }

  return(GReal);

}
#endif
