/*
 * IsAdjacent.c
 *
 *  Function inputs the numbers of two channels and the filterbank.  If
 *  the channels are adjacent to one another (8-connected tesselation),
 *  1 is returned.  Otherwise, 0 is returned.
 * 
 *  jph 8/18/94
 *
 */

#ifndef ISADJACENT_C
#define ISADJACENT_C
#include "teager2D.h"
#include <stdio.h>

int IsAdjacent(c1,c2,G,wrap)

  int         c1;     /* number of first channel */
  int         c2;     /* number of second channel */
  filterbank *G;      /* the filter bank */
  int        *wrap;   /* 1 if adjacency is accross 1st and last rays */

{
  int      i;            /* loop counter */

  *wrap = 0;

 /*
  * Trivial Case 
  */
  if (c2 == c1) return(1);

 /*
  * Special Cases: c1 is on first ray
  */
  if (G->filt[c1]->Ray == 0) {
    if (c2 == (c1 + G->NumFiltersPerRay)) return(1);
    if (c2 == (c1 + (G->NumRays-1)*G->NumFiltersPerRay)) {
      *wrap = 1;
      return(1);
    }
    if (G->filt[c1]->IndexOnRay != 0) {
      if (c2 == (c1 - 1)) return(1);
      if (c2 == (c1 - 1 + G->NumFiltersPerRay)) return(1);
      if (c2 == (c1 - 1 + (G->NumRays-1)*G->NumFiltersPerRay)) {
        *wrap = 1;
        return(1);
      }
    }
    if (G->filt[c1]->IndexOnRay != (G->NumFiltersPerRay - 1)) {
      if (c2 == (c1 + 1)) return(1);
      if (c2 == (c1 + 1 + G->NumFiltersPerRay)) return(1);
      if (c2 == (c1 + 1 + (G->NumRays-1)*G->NumFiltersPerRay)) {
        *wrap = 1;
        return(1);
      }
    }
    return(0);
  }

 /*
  * Special Cases: c1 is on last ray
  */
  if (G->filt[c1]->Ray == (G->NumRays - 1)) {
    if (c2 == (c1 - G->NumFiltersPerRay)) return(1);
    if (c2 == (c1 - (G->NumRays-1)*G->NumFiltersPerRay)) {
      *wrap = 1;
      return(1);
    }
    if (G->filt[c1]->IndexOnRay != 0) {
      if (c2 == (c1 - 1)) return(1);
      if (c2 == (c1 - 1 - G->NumFiltersPerRay)) return(1);
      if (c2 == (c1 - 1 - (G->NumRays-1)*G->NumFiltersPerRay)) {
        *wrap = 1;
        return(1);
      }
    }
    if (G->filt[c1]->IndexOnRay != (G->NumFiltersPerRay - 1)) {
      if (c2 == (c1 + 1)) return(1);
      if (c2 == (c1 + 1 - G->NumFiltersPerRay)) return(1);
      if (c2 == (c1 + 1 - (G->NumRays-1)*G->NumFiltersPerRay)) {
        *wrap = 1;
        return(1);
      }
    }
    return(0);
  } 

 /*
  * Special Cases: c1 is last on ray
  */
  if (G->filt[c1]->IndexOnRay == (G->NumFiltersPerRay - 1)) {
    if (c2 == (c1 - 1)) return(1);
    if (c2 == (c1 + G->NumFiltersPerRay)) return(1);
    if (c2 == (c1 - G->NumFiltersPerRay)) return(1);
    if (c2 == (c1 - 1 + G->NumFiltersPerRay)) return(1);
    if (c2 == (c1 - 1 - G->NumFiltersPerRay)) return(1);
    return(0);
  }

 /*
  * Special Cases: c1 is first on ray
  */
  if (G->filt[c1]->IndexOnRay == 0) {
    if (c2 == (c1 + 1)) return(1);
    if (c2 == (c1 + G->NumFiltersPerRay)) return(1);
    if (c2 == (c1 - G->NumFiltersPerRay)) return(1);
    if (c2 == (c1 + 1 + G->NumFiltersPerRay)) return(1);
    if (c2 == (c1 + 1 - G->NumFiltersPerRay)) return(1);
    return(0);
  }

 /*
  * General Case
  */
  if (c2 == (c1 + 1)) return(1);
  if (c2 == (c1 - 1)) return(1);
  if (c2 == (c1 + G->NumFiltersPerRay)) return(1);
  if (c2 == (c1 - G->NumFiltersPerRay)) return(1);
  if (c2 == (c1 - 1 + G->NumFiltersPerRay)) return(1);
  if (c2 == (c1 - 1 - G->NumFiltersPerRay)) return(1);
  if (c2 == (c1 + 1 + G->NumFiltersPerRay)) return(1);
  if (c2 == (c1 + 1 - G->NumFiltersPerRay)) return(1);

 /*
  * If you are here, c1 and c2 are not adjacent
  */
  return(0);
}
#endif
