/*
 * MakeHardy.c
 *
 *  Function turns a real image into a complex image by:
 *    taking the fft of the real image
 *    zeroing the 2nd and 3rd quadrants of the fft
 *    taking the inverse fft
 *
 *  No memory allocation is performed.  All four input float arrays must be
 *  malloc'd by the calling routine.
 *
 *  jph 9/13/93
 *
 *  Major Revision 7/11/96:  set the analytic spectrum as in dissertation.
 *
 */

#ifndef MAKEHARDY2_C
#define MAKEHARDY2_C
#include "teager2D.h"
#include <stdio.h>

void MakeHardy2(xReal,xImag,XReal,XImag,size)

  float *xReal;       /* real part of signal */
  float *xImag;       /* imaginary part of signal */
  float *XReal;       /* real part of spectrum */
  float *XImag;       /* imaginary part of spectrum */
  int    size;        /* row/col dim of image */

{
  int i;                        /* loop counter */
  int row,col;                  /* loop counters for image coordinates */
  int limit;                    /* terminal value for loop counter */
  int so2;                      /* size over 2 */

  so2 = size >> 1;
 /*
  * Get the spectrum of the real signal
  */  
  for (i=0; i<size*size; i++) {
    xImag[i] = (float)0.0;
  }
  fft2d(xReal,xImag,XReal,XImag,size,1);
    
 /*
  * First row of spectrum
  */
  XImag[0] = (float)0.0;
  for (col=1; col < so2; col++) {
    XReal[col] = XImag[col] = (float)0.0;
  }
  XImag[so2] = (float)0.0;
  for (col=so2+1; col < size; col++) {
    XReal[col] *= (float)2.0;
    XImag[col] *= (float)2.0;
  }

 /*
  * Rows 1 to so2-1
  */
  for (row=1; row < so2; row++) {
    i = row * size;
    for (col=0; col < so2+1; col++) {
      XReal[i+col] = XImag[i+col] = (float)0.0;
    }
    for (col=so2+1; col < size; col++) {
      XReal[i+col] = XImag[i+col] = (float)0.0;
    }
  }

 /*
  * Row so2
  */
  i = so2 * size;
  XImag[i] = (float)0.0;
  for (col=1; col < so2; col++) {
    XReal[i+col] = XImag[i+col] = (float)0.0;
  }
  XImag[i+so2] = (float)0.0;
  for (col=so2+1; col < size; col++) {
    XReal[i+col] *= (float)2.0;
    XImag[i+col] *= (float)2.0;
  }

 /*
  * Rows so2+1 to size-1
  */
  for (row=so2+1; row < size; row++) {
    i = row * size;
    XReal[i] *= (float)2.0;
    XImag[i] *= (float)2.0; 
    for (col=1; col < so2; col++) {
      XReal[i+col] *= (float)2.0;
      XImag[i+col] *= (float)2.0;
    }
    for (col=so2; col < size; col++) {
      XReal[i+col] *= (float)2.0;
      XImag[i+col] *= (float)2.0;
    }
  }
    
 /*
  * Get the Hardy image by inverse fft
  */
  fft2d(xReal,xImag,XReal,XImag,size,-1);

  return;

}
#endif 
