/*
 * MkCosFilt.c
 *
 *   Function returns a pointer to a real-valued "Gabor" filter (Cosine filter).
 *   The filter has zero phase, which means that the main lobe is split across 
 *   the four corners of the image instead of being centered.
 *
 *  jph 2/13/99
 *
 */
#ifndef MKCOSFILT_C
#define MKCOSFILT_C
#include <stdio.h>
#include "teager2D.h"

pFloatImage MkCosFilt(f,size)

  FilterPtr   f;      /* A true complex Gabor filter structure */
  int         size;   /* row / col dimension of image */

{

  pFloatImage g;      /* ptr to filter image being created */
  int         n;      /* size * size, the num of pixels in image */
  int         i;      /* counter for image pixels */

 /*
  * Allocate the floating image and its constituents
  */
  if ((g = (pFloatImage)malloc(sizeof(FloatImage))) == NULL) {
    printf("\nMkCosFilt: Free Store Exhausted.\n\n");
    exit(-1);
  }
  n = size * size;
  g->size = g->rows = g->cols = size;
  g->square = 1;

  g->Real = gMakeReal(f,1,size);
  if ((g->Imag = (float *)malloc(n * sizeof(float))) == NULL) {
    printf("\nMkCosFilt: Free Store Exhausted.\n\n");
    exit(-1);
  }
  for (i=0; i < n; i++) {
    g->Imag[i] = (float)0.0;
  }
  if ((g->FFTReal = (float *)malloc(n * sizeof(float))) == NULL) {
    printf("\nMkCosFilt: Free Store Exhausted.\n\n");
    exit(-1);
  }
  if ((g->FFTImag = (float *)malloc(n * sizeof(float))) == NULL) {
    printf("\nMkCosFilt: Free Store Exhausted.\n\n");
    exit(-1);
  }

 /*
  * Compute filter in freq domain by FFT
  */
  fft2d(g->Real,g->Imag,g->FFTReal,g->FFTImag,size,1);

  return(g);
}
#endif
