/*
 * PC.c
 *
 *   Function to print out a column of a 2D array in the debugger.
 *
 *   To make it available, do a degenerate call like 
 *     PC((void *)x,0,0,-1,'b',0); somewhere in the program.
 *
 *  jph 27 Jan 1994
 */

#ifndef PC_C
#define PC_C
#include <stdio.h>
#include "teager2D.h"

void PC(x,col,firstrow,lastrow,d_type,size)

  void *   x;         /* base of array, cast to (void *) */
  int      col;       /* column to output */
  int      firstrow;  /* first row to output */
  int      lastrow;   /* last row to output */
  char     d_type;    /* data type of array: b=BYTE   i=int   f=float 
                                             d=double c=char */
  int      size;      /* row/col dimension of array (must be square) */

{

  int    i;   /* loop counters */
  int    row; 

  printf("\n");
  switch(d_type) {
    case 'b':
    case 'i':
    case 'f':
    case 'd':
    case 'c':  break;

    default :  printf("PC: unknown data type %c specified.",d_type);
               return;
  }
  for (row=firstrow; row <= lastrow ; row++) {
    i = row * size + col;
    switch(d_type) {
      case 'b':  printf("\n[%d,%d]\t=\t%d",row,col,(BYTE) *( ((BYTE *)x) + i ));
                 break;
      case 'i':  printf("\n[%d,%d]\t=\t%d",row,col,(int) *( ((int *)x) + i ));
                 break;
      case 'f':  printf("\n[%d,%d]\t=\t%f",row,col,(float) *( ((float *)x) + i ));
                 break;
      case 'd':  printf("\n[%d,%d]\t=\t%lf",row,col,(double) *( ((double *)x) + i ));
                 break;
      case 'c':  printf("\n[%d,%d]\t=\t%c",row,col,(char) *( ((char *)x) + i ));
                 break;
    }
  }
  printf("\n");
  return;
}
#endif
