/*
 * PRINTA.c
 *
 *   Function to write out an array to an ascii disk file in xdbx.
 *
 *   To make it available, do a degenerate call like 
 *     PA((void *)x,0,-1,'b',"null.dat"); somewhere in the program.
 *
 *   Parameters:
 *     
 *      x      base of array, cast to (void *)
 *      first  index of first element to print out
 *      last   index of last  element to print out
 *      d_type a char that tells the data type of the elements of x.
 *          'b'  for unsigned char (BYTE) in %d format
 *          'i'  for integer
 *          'f'  for float
 *          'd'  for double
 *          'c'  for char
 *      fn     filename to write to.
 *
 *  jph 19 Jan 1994
 */

#ifndef PRINTA_C
#define PRINTA_C
#include <stdio.h>
#include <fcntl.h>
#include "teager2D.h"

void PRINTA(x,first,last,d_type,fn)

  void    *x;
  int      first;
  int      last;
  char     d_type;
  char    *fn;

{

  int    i;
  FILE  *outfile;

  if ((outfile = fopen(fn,"w")) == NULL) {
    printf("\n\nPRINTA: unable to open %s\n",fn);
    exit(-1);
  }

  switch(d_type) {
    case 'b':
    case 'i':
    case 'f':
    case 'd':
    case 'c':  break;

    default :  printf("PRINTA: unknown data type %c specified.",d_type);
               return;
  }
  for (i=first; i <= last ; i++) {
    switch(d_type) {
      case 'b':  fprintf(outfile,"%d\t%d\n",i,(BYTE) *( ((BYTE *)x) + i ));
                 break;
      case 'i':  fprintf(outfile,"%d\t%d\n",i,(int) *( ((int *)x) + i ));
                 break;
      case 'f':  fprintf(outfile,
                             "%d\t%f\n",i,(float) *( ((float *)x) + i ));
                 break;
      case 'd':  fprintf(outfile,
                          "%d\t%lf\n",i,(double) *( ((double *)x) + i ));
                 break;
      case 'c':  fprintf(outfile,"%d\t%c\n",i,(char) *( ((char *)x) + i ));
                 break;
    }
  }
  fclose(outfile);
  return;
}
#endif
