/*
 * PRINTC.c
 *
 *   Function to write out a column of a 2D array to an ascii disk file in xdbx.
 *   the file format is suitable for use with xvgr.  
 *
 *   To make it available, do a degenerate call like 
 *     PA((void *)x,0,0,-1,'b',0,"null.dat"); somewhere in the program.
 *
 *  jph 24 Jan 1994
 */

#ifndef PRINTC_C
#define PRINTC_C
#include <stdio.h>
#include <fcntl.h>
#include "teager2D.h"

void PRINTC(x,col,firstrow,lastrow,d_type,size,fn)

  void    *x;        /* data array */
  int      col;      /* column to be printed */
  int      firstrow; /* first row to print */
  int      lastrow;  /* last row to print */
  char     d_type;   /* data type of array: b=BYTE  i=int  d=double */
                     /*                     f=float c=char          */
  int      size;     /* row/col dimension of the array */
  char    *fn;       /* filename to print to */

{

  int    i;          /* loop counter */
  int    row;        /* loop counter */
  FILE  *outfile;    /* output file pointer */

  if ((outfile = fopen(fn,"w")) == NULL) {
    printf("\n\nPRINTC: unable to open %s\n",fn);
    exit(-1);
  }

  switch(d_type) {
    case 'b':
    case 'i':
    case 'f':
    case 'd':
    case 'c':  break;

    default :  printf("PRINTC: unknown data type %c specified.",d_type);
               return;
  }
  for (row=firstrow; row <= lastrow; row++) {
    i = row*size + col;
    switch(d_type) {
      case 'b':  fprintf(outfile,"%d\t%d\n",row,(BYTE) *( ((BYTE *)x) + i ));
                 break;
      case 'i':  fprintf(outfile,"%d\t%d\n",row,(int) *( ((int *)x) + i ));
                 break;
      case 'f':  fprintf(outfile,
                             "%d\t%f\n",row,(float) *( ((float *)x) + i ));
                 break;
      case 'd':  fprintf(outfile,
                          "%d\t%lf\n",row,(double) *( ((double *)x) + i ));
                 break;
      case 'c':  fprintf(outfile,"%d\t%c\n",row,(char) *( ((char *)x) + i ));
                 break;
    }
  }
  fclose(outfile);
  return;
}
#endif
