/*
 * ReComputeAsm.c
 *
 *  Compute amplitude estimates from smoothed frequency estimates following
 *  postfiltering of the frequency estimates.
 *
 *  See Book XI, p. 119.
 *
 *  jph 4/15/96
 */

#ifndef RECOMPUTEASM_C
#define RECOMPUTEASM_C
#include <stdio.h>
#include "teager2D.h"

#define FLOAT_TOO_SMALL ((float)0.00001)


void ReComputeAsm(t,Usm,Vsm,Asm,G)

  pFloatImage   t;                  /* response image */
  float        *Usm;                /* postfiltered horiz frequency estimates */
  float        *Vsm;                /* postfiltered vert  frequency estimates */
  float        *Asm;                /* smoothed amplitude estimates */
  FilterPtr     G;                  /* pointer to the Gabor filter */
{

  int           i;                  /* counter */
  int           n;                  /* num pixels in image */
  float         FilterMag;          /* magnitude of filter at estimated u,v */

  n = t->size * t->size;
  for (i=0; i < n; i++) {
    FilterMag = G_EvalMag(G,fabs((double)Usm[i]),(double)Vsm[i]);
    if ((FilterMag / G->GMax) > FLOAT_TOO_SMALL) { 
      Asm[i] = fNorm(t->Real[i],t->Imag[i]) / FilterMag;
    } else {                       
      Asm[i] = (float)0.0;
    } 
  }

  return;
} 
#endif
