/*
 * ShiftDownRows.c
 *
 *   function returns y(i,j) = x(i-delta,j) where x and y are integer
 *   images.  x is declared int x[row_dim][col_dim], and y has the
 *   same dimensions.  The output image y is allocated on the heap.
 *   The values shifted in are REFLECTED about the edge sampes of x.
 *
 *  jph 4 June 1992
 */

#ifndef SHIFTDOWNROWS_C
#define SHIFTDOWNROWS_C
#include <stdio.h>
#include "teager2D.h"

int* ShiftDownRows(x,row_dim,col_dim,delta)

  int* x;         /* input image */
  int row_dim;    /* row dimension of x */
  int col_dim;    /* col dimension of x */
  int delta;      /* number of rows to shift up */
{

  int *y;         /* pointer to output image */
  int i;          /* loop counters */
  int targ_row;   /* row currently being copied to */
  int src_row;    /* row currently being copied from */

 /*
  * detect zero dimension input
  */
  if ((row_dim==0) || (col_dim==0)) return NULL;

 /*
  * detect delta larger than image
  */
  if (delta >= row_dim) {
    printf("\nShiftDownRows : delta larger than row_dim !");
    return NULL;
  }

 /*
  * detect delta negative
  */
  if (delta < 0) {
    printf("\nShiftDownRows : delta negative !");
    return NULL;
  }

 /* 
  * allocate memory for y, return error if out of memory
  */
  y = (int *) malloc(row_dim * col_dim * sizeof(int));
  if (y == NULL) {
    printf("\nShiftDownRows.c : out of memory error !");
    return NULL;
  }

 /* 
 /*
  * set y(i,j) = x(i-delta,j).  First, shift the rows that can be
  * shifted directly.
  */
  for (src_row=(targ_row=row_dim-1)-delta;src_row>=0;targ_row--,src_row--) {
    for (i=0; i<col_dim; i++) {
      y[targ_row*col_dim + i] = x[src_row*col_dim + i];
    }
  }

 /*
  * Now do the reflection
  */
  for (src_row+=2; targ_row>=0; targ_row--,src_row++) {
    for (i=0; i<col_dim; i++) {
      *(y+ targ_row*col_dim + i) = *(x+ src_row*col_dim + i);
    }
  }
  return y;
} 
#endif
