/*
 * ShiftLeftCols.c
 *
 *   function returns y(i,j) = x(i,j+delta) where x and y are integer
 *   images.  x is declared int x[row_dim][col_dim], and y has the
 *   same dimensions.  The output image y is allocated on the heap.
 *   The values shifted in are REFLECTED about the edge sampes of x.
 *
 *  jph 4 June 1992
 */

#ifndef SHIFTLEFTCOLS_C
#define SHIFTLEFTCOLS_C
#include <stdio.h>
#include "teager2D.h"

int* ShiftLeftCols(x,row_dim,col_dim,delta)

  int* x;         /* input image */
  int row_dim;    /* row dimension of x */
  int col_dim;    /* col dimension of x */
  int delta;      /* number of rows to shift up */
{

  int *y;         /* pointer to output image */
  int i;          /* loop counters */
  int targ_col;   /* col currently being copied to */
  int src_col;    /* col currently being copied from */

 /*
  * detect zero dimension input
  */
  if ((row_dim==0) || (col_dim==0)) return NULL;

 /*
  * detect delta larger than image
  */
  if (delta >= col_dim) {
    printf("\nShiftLeftCols : delta larger than col_dim !");
    return NULL;
  }

 /*
  * detect delta negative
  */
  if (delta < 0) {
    printf("\nShiftLeftCols : delta negative !");
    return NULL;
  }

 /* 
  * allocate memory for y, return error if out of memory
  */
  y = (int *) malloc(row_dim * col_dim * sizeof(int));
  if (y == NULL) {
    printf("\nShiftLeftCols.c : out of memory error !");
    return NULL;
  }

 /*
  * set y(i,j) = x(i,j+delta).  First, shift the cols that can be
  * shifted directly.
  */
  for (targ_col=0,src_col=delta; src_col<col_dim; targ_col++,src_col++) {
    for (i=0; i<row_dim; i++) {
      y[i*col_dim + targ_col] = x[i*col_dim + src_col];
    }
  }
 /*
  * Now do the reflection
  */
  for (src_col-=2; targ_col<col_dim; targ_col++,src_col--) {
    for (i=0; i<row_dim; i++) {
      y[i*col_dim + targ_col] = x[i*col_dim + src_col];
    }
  }
  return y;
} 
#endif
