/*
 * SpecMag2disk
 *
 *  Function writes the magnitude of the spectrum of an image to a disk
 *  file.  The disk file is of type BYTE with maximum dynamic range.
 *
 *  The function written to disk is 
 *
 *       c log[1 + |F(u,v)|] 
 *
 *  where F(u,v) is the image spectrum.
 *
 *  20 Jan 1994  jph
 */

#ifndef SPECMAG2DISK_C
#define SPECMAG2DISK_C
#include <stdio.h>
#include "teager2D.h"

void SpecMag2disk(XReal,XImag,size,fn)

  float        *XReal;   /* Real Spectrum */
  float        *XImag;   /* Imaginary Spectrum */
  int          size;     /* row/col dimension of image */
  char         *fn;      /* filename to write the spectrum image to */
{

  int         N;      /* number of pixels in the image */
  int         i;      /* loop counter */
  float      *Xmag;   /* magnitude spectrum for display */
  BYTE       *b_Xmag; /* byte version of the magnitude spectrum */

 /*
  * allocate a working image
  */
  N = size * size;
  Xmag = floatalloc(size);

 /*
  * compute the magnitude spectrum for display
  */
  for (i=0; i < N; i++) {
    Xmag[i] = log((double)1.0 + (double)fNorm(XReal[i], XImag[i]));
  }

 /*
  * Expand dynamic range and write to disk
  */
  b_Xmag = f_dbx(Xmag, size, size);
  byte2disk(b_Xmag, size, size, fn);

 /*
  * Free workspace
  */
  free(Xmag);
  free(b_Xmag);

  return;
} 
#endif
