/*
 * WRITEA.c
 *
 *   Function to write out an array to a binary disk file in xdbx.
 *
 *   To make it available, do a degenerate call like 
 *     WRITEA((void *)x,0,-1,'b',"null.dat"); somewhere in the program.
 *
 *   Parameters:
 *     
 *      x      base of array, cast to (void *)
 *      first  index of first element to print out
 *      last   index of last  element to print out
 *      d_type a char that tells the data type of the elements of x.
 *          'b'  for unsigned char (BYTE) in %d format
 *          'i'  for integer
 *          'f'  for float
 *          'd'  for double
 *          'c'  for char
 *      fn     filename to write to.
 *
 *  jph 19 Jan 1994
 */

#ifndef WRITEA_C
#define WRITEA_C
#include <stdio.h>
#include <fcntl.h>
#include "teager2D.h"

void WRITEA(x,first,last,d_type,fn)

  void    *x;
  int      first;
  int      last;
  char     d_type;
  char    *fn;

{

  if (first >= last) return;
  switch(d_type) {
    case 'c':  
    case 'b':  byte2disk((unsigned char *)x,(last-first+1),1,fn);
               break;
    case 'i':  byte2disk((unsigned char *)x,(last-first+1)*sizeof(int),1,fn);
               break;
    case 'f':  f2disk((float *)x,(last-first+1),1,fn);
               break;
    case 'd':  d2disk((double *)x,(last-first+1),1,fn);
               break;
    default :  printf("WRITEA: unknown data type %c specified.",d_type);
               return;
  }
  return;
}
#endif
