/*
 * WRITEC.c
 *
 *   Function to write out a column of a 2D array to a binary disk file in xdbx.
 *
 *   To make it available, do a degenerate call like 
 *     WRITEC((void *)x,0,0,-1,'b',0,"null.dat"); somewhere in the program.
 *
 *   NOTE: this is not a very efficient operation to perform.
 *
 *  jph 24 Jan 1994
 */

#ifndef WRITEC_C
#define WRITEC_C
#include <stdio.h>
#include <fcntl.h>
#include "teager2D.h"

void WRITEC(x,col,firstrow,lastrow,d_type,size,fn)

  void    *x;         /* data array */ 
  int      col;       /* column to output */
  int      firstrow;  /* first row to print out */
  int      lastrow;   /* last row to print out */
  char     d_type;    /* data type of array: b=BYTE  i=int  d=double */
                      /*                     f=float c=char          */
  int      size;      /* row/col dimension of the array */
  char    *fn;

{

  int     row;   /* loop counter */
  int     i;     /* index */

  if (firstrow >= lastrow) return;
  for (row=firstrow; row <= lastrow; row++) {
    i = row*size + col;
    switch(d_type) {
      case 'c':  
      case 'b':  byte2disk((unsigned char *)x+i,1,1,fn);
                 break;
      case 'i':  byte2disk((unsigned char *)x+i,1,sizeof(int),fn);
                 break;
      case 'f':  f2disk((float *)x+i,1,1,fn);
                 break;
      case 'd':  d2disk((double *)x+i,1,1,fn);
                 break;
      default :  printf("WRITEA: unknown data type %c specified.",d_type);
                 return;
    }
  }
  return;
}
#endif
