/*
 * addimage.c
 *
 *   function returns the pointwise sum of two integer images:
 *
 *        z(i,j) = x(i,j) + y(i,j)
 *
 *   where x and y are declared int x[row_dim][col_dim], and z has the
 *   same dimensions.  The output image y is allocated on the heap.
 *
 *  jph 4 June 1992
 */

#ifndef ADDIMAGE_C
#define ADDIMAGE_C
#include <stdio.h>
#include "teager2D.h"

int* addimage(x,y,row_dim,col_dim)

  int *x,*y;      /* input images */
  int row_dim;    /* row dimension of x,y */
  int col_dim;    /* col dimension of x,y */
{

  int *z;         /* pointer to output image */
  int i;          /* loop counter */

 /*
  * detect zero dimension input
  */
  if ((row_dim==0) || (col_dim==0)) return NULL;

 /* 
  * allocate memory for z, return error if out of memory
  */
  z = (int *) malloc(row_dim * col_dim * sizeof(int));
  if (z == NULL) {
    printf("\naddimage.c : out of memory error !");
    return NULL;
  }

 /*
  * set z = x+y
  */
  for (i=0; i<row_dim * col_dim; i++) {
    *(z + i) = (*(x + i)) + (*(y + i));
  }
  return z;
} 
#endif
