/*
 * bMakeBankSpectrum.c
 *
 * Function creates a binary image suitable for display of the specified 
 * filter Gabor filter bank.  The quantity displayed is c log[1 + |F(u,v)|]
 * The output image is allocated on the heap.
 * 
 * the filterbank parameters are written out to the file "bank.log"
 *
 *  jph 9/11/93
 *
 */

#ifndef BMAKEBANKSPECTRUM_C
#define BMAKEBANKSPECTRUM_C
#define BYTE unsigned char
#include "teager2D.h"
#include <stdio.h>

unsigned char *bMakeBankSpectrum(size,r0,R,OBW,Eta,kappa)


  int         size;                /* row (col) dimension of the image */
  double      r0;                  /* radial center freq. of the first filter */
                                   /*   on each ray in CYCLES per pixel */
  float       R;                   /* common ratio of radii of adjcnt filtrs */
  float       OBW;                 /* Eta-peak octave bandwidth of filters */
  float       Eta;                 /* OBW is defined in terms of Eta-peak. */
  float       kappa;               /* postfilter sigma scaling factor */

{

  float      *M;         /* Composite Magnitude Image */
  float      *X;         /* magnitude spectrum of one filter in the bank */
  BYTE       *Y;         /* output binary image */
  filterbank *G;         /* Gabor filter bank */
  int         i;         /* loop counter for filters in the bank */
  int         k;         /* loop counter for image coordinates */
  int         N;         /* num pixels in image == size**2  */
  FILE *logfile;         /* ascii output file */

  N = size * size;
  logfile = fopen("bank.log","w");
  G = CreateFilterBank(r0,R,OBW,Eta,1,kappa,kappa,kappa);

  fprintf(logfile,"\nFilterbank:\n");
  fprintf(logfile,"NumRays = %d\n",G->NumRays);
  fprintf(logfile,"NumFiltersPerRay = %d\n",G->NumFiltersPerRay);
  fprintf(logfile,"TotNumFilters = %d\n",G->TotNumFilters);
  fprintf(logfile,"r0 = %lf\n",G->r0);
  fprintf(logfile,"Common Ratio = %lf\n",G->R);
  fprintf(logfile,"Theta0 = %lf\n",G->Theta0);
  fprintf(logfile,"Radial Spacing = %lf\n",G->Lambda);
  fprintf(logfile,"Eta = %f\n",G->Eta);
  fprintf(logfile,"OBW = %f\n",G->OBW);
  fprintf(logfile,"gamma = %f\n",G->gamma);

  fprintf(logfile,"\n\nFilter %d\n",0);
  fprintf(logfile,"Ray = %d\n",G->filt[0]->Ray);
  fprintf(logfile,"IndexOnRay = %d\n",G->filt[0]->IndexOnRay);
  fprintf(logfile,"Theta = %lf\n",G->filt[0]->Theta);
  fprintf(logfile,"r0 = %lf\n",G->filt[0]->r0);
  fprintf(logfile,"u0 = %lf\n",G->filt[0]->u0);
  fprintf(logfile,"v0 = %lf\n",G->filt[0]->v0);
  fprintf(logfile,"sigma = %lf\n",G->filt[0]->sigma);
  fprintf(logfile,"GMax = %f\n",G->filt[0]->GMax);

  printf("\nDoing Filter 0...");
  M = GMakeMag(G->filt[0],size);
  for (i=0; i < N; i++) {
    M[i] = (float) log((double)1.0 + (double)M[i]);
  }
  for (i=1; i < G->TotNumFilters; i++) {
    fprintf(logfile,"\n\nFilter %d\n",i);
    fprintf(logfile,"Ray = %d\n",G->filt[i]->Ray);
    fprintf(logfile,"IndexOnRay = %d\n",G->filt[i]->IndexOnRay);
    fprintf(logfile,"Theta = %lf\n",G->filt[i]->Theta);
    fprintf(logfile,"r0 = %lf\n",G->filt[i]->r0);
    fprintf(logfile,"u0 = %lf\n",G->filt[i]->u0);
    fprintf(logfile,"v0 = %lf\n",G->filt[i]->v0);
    fprintf(logfile,"sigma = %lf\n",G->filt[i]->sigma);
    fprintf(logfile,"GMax = %f\n",G->filt[i]->GMax);
    printf("\nDoing Filter %d...",i);
    X = GMakeMag(G->filt[i],size);
    for (k=0; k < N; k++) {
      X[k] = (float) log((double)1.0 + (double)X[k]);
    }
    for (k=0; k < N; k++) {
      if (X[k] > M[k]) {
        M[k] = X[k];
      }
    }
    free(X);
  }
  Y = f_dbx(M,size,size);
  free(M);
  return(Y);

}
#endif
