/*
 * bMakeFilterImage.c
 *
 * Function creates three binary images suitable for display of the specified 
 * filter.  Magnitude, Real component, and Imaginary component images are 
 * created.  The output images are allocated on the heap.
 *
 *  jph 9/11/93
 *
 */

#ifndef BMAKEFILTERIMAGE_C
#define BMAKEFILTERIMAGE_C
#define BYTE unsigned char
#include "teager2D.h"
#include <stdio.h>

void bMakeFilterImage(f,zerophase,size,Real,Imag,Mag)

  FilterPtr   f;         /* filter to be evaluated */
  int         zerophase; /* 1: zero-phase filter, 0: filter centered in plane */
  int         size;      /* row (col) dimension of the image */
  BYTE      **Real;      /* real component image */
  BYTE      **Imag;      /* imaginary component image */
  BYTE      **Mag;       /* magnitude image */

{

  float      *R;         /* float real component array */
  float      *I;         /* float imaginary component array */
  float      *M;         /* float magnitude array */

  R = gMakeReal(f,zerophase,size);
  I = gMakeImag(f,zerophase,size);
  M = gMakeMag(f,zerophase,size);

  *Real = f_dbx(R,size,size);
  *Imag = f_dbx(I,size,size);
  *Mag  = f_dbx(M,size,size);

  free(R);
  free(I);
  free(M);

}
#endif
