/*
 * b_scaleimage.c
 *
 *   function in-place computes the rounded scalar multiple of an unsigned char
 *   (byte) image, defined as:
 *
 *        x(i,j) <== (unsigned char) (c * (double) x(i,j) + 0.5)
 *
 *   where x is declared unsigned char x[row_dim][col_dim], and c is a (double) 
 *   scalar.
 *
 *  jph 15 June 1992
 */

#ifndef B_SCALEIMAGE_C
#define B_SCALEIMAGE_C
#include <stdio.h>
#include "teager2D.h"

void b_scaleimage(x,row_dim,col_dim,c)

  unsigned char* x;         /* input image */
  int row_dim;    /* row dimension of x */
  int col_dim;    /* col dimension of x */
  double c;       /* factor by which to scale */
{

  int i;        /* loop counters */

 /*
  * detect zero dimension input
  */
  if ((row_dim==0) || (col_dim==0)) return;

 /*
  * set x = c*x
  */
  for (i=0; i<row_dim * col_dim; i++) {
    x[i] = (unsigned char) (c * (double)x[i] + 0.5);
  }
  return;
} 
#endif
