/*
 * bytedisplay.c
 *
 *   function displays a byte image interactively and allows
 *   histogram modification and disk writing.
 *
 *   Revision 9 October 1992
 *     upgrade to allow display with false color in addition to grey scale
 *
 *  jph 17 June 1992
 */

#ifndef BYTEDISPLAY_C
#define BYTEDISPLAY_C
#include <stdio.h>
#include "teager2D.h"

void bytedisplay(x,row_dim,col_dim)

  unsigned char* x;     /* input image */
  int row_dim;    /* row dimension of x */
  int col_dim;    /* col dimension of x */
{

  int size;          /* row_dim = col_dim of displayed image */
  unsigned char min,max; /* min and max pixel values in x */
  int notsquare;     /* true if image not square */
  unsigned char *im; /* temp image */
  char cmd[100];     /* user command */
  int choice;        /* user menu choice */
  int i;             /* loop counter */
  int bias;          /* bias offset */
  double gain;       /* gain factor */
  int newval;        /* trial new value for pixel */
  int color;         /* 0 means grey-scale, 1 means false color */

 /*
  * detect zero dimension input
  */
  if ((row_dim==0) || (col_dim==0)) return;

 /*
  * Main loop, continue until user quits
  */
  color = 0;
  if (row_dim != col_dim) notsquare = 1;
  else notsquare = 0;
  do {

   /*
    * If not square, make square
    */
    if (notsquare) size = squarepad(x,im,row_dim,col_dim);
    else {
      im = x;
      size = row_dim;
    }

   /*
    * Display image and histogram
    */
    if (color) c_byte2monitor(im,size);
    else byte2monitor(im,size);
    histo(x,row_dim,col_dim);

   /*
    * If im was allocated, free
    */
    if (notsquare) free(im);

   /*
    * Display menu, get a valid choice
    */
    do {
      system("clear");
      printf("\n");
      printf("\nENTER YOUR CHOICE:");
      printf("\n");
      printf("\n 1.  Enter new min and max.");
      printf("\n 2.  Expand dynamic range to 8 bits.");
      printf("\n 3.  Change gain.");
      printf("\n 4.  Change bias.");
      printf("\n 5.  Write image to disk file.");
      printf("\n 6.  Display image and histogram again.");
      printf("\n 7.  Print histogram.");
      printf("\n 8.  Change grey level.");
      printf("\n 9.  Toggle false color ON or OFF.");
      printf("\n10.  Quit this menu.");
      printf("\n\n");
      scanf("%s",cmd);
      choice = atoi(cmd);
    }while((choice < 1) || (choice > 10));

   /*
    * Carry out choice
    */
    switch(choice)
    {
      case 1:  do {
                 printf("\n\nNew min ? ");
                 scanf("%s",cmd);
                 min = atoi(cmd);
               }while((min < 0) || (min > 255));

               do {
                 printf("\n\nNew max ? ");
                 scanf("%s",cmd);
                 max = atoi(cmd);
               }while((max < 1) || (max > 255));

               chop(x,row_dim,col_dim,min,max);
               break;

      case 2:  dbx(x,row_dim,col_dim);
               break;

      case 3:  printf("\n\nGain factor ? ");
               scanf("%s",cmd);
               sscanf(cmd,"%lf",&gain);
               b_scaleimage(x,row_dim,col_dim,gain);
               break;

      case 4:  printf("\n\nBias offset (integer) ? ");
               scanf("%s",cmd);
               bias = atoi(cmd);
               for (i=0; i<row_dim*col_dim; i++) {
                 newval = (int)x[i] + bias;
                 if (newval < 0) x[i] = 0;
                 else if (newval > 255) x[i] = 255;
                      else x[i] = (unsigned char) newval; 
               }
               break;

      case 5:  printf("\n\nFilename ? ");
               scanf("%s",cmd);
               byte2disk(x,row_dim,col_dim,cmd);
               break;

      case 7:  print_histo(x,row_dim,col_dim);
               break;

      case 8:  do {
                 printf("\n\nChange what grey level ? ");
                 scanf("%s",cmd);
                 min = atoi(cmd);
               }while((min < 0) || (min > 255));

               do {
                 printf("\n\nTo what ? ");
                 scanf("%s",cmd);
                 max = atoi(cmd);
               }while((max < 0) || (max > 255));
               for (i=0; i<row_dim*col_dim; i++)
                 if (x[i] == min) x[i] = max;
               break;

      case 9:  color = !color;
               break;

      default: break;
    }
  }while (choice != 10);

  return;
} 
#endif
