/*
 * d_ShiftUpRows.c
 *
 *   function returns y(i,j) = x(i+delta,j) where x and y are double
 *   images.  x is declared double x[row_dim][col_dim], and y has the
 *   same dimensions.  The output image y is allocated on the heap.
 *   The values shifted in are REFLECTED about the edge sampes of x.
 *
 *  jph 4 June 1992
 */

#ifndef D_SHIFTUPROWS_C
#define D_SHIFTUPROWS_C
#include <stdio.h>
#include "teager2D.h"

double* d_ShiftUpRows(x,row_dim,col_dim,delta)

  double* x;         /* input image */
  int row_dim;    /* row dimension of x */
  int col_dim;    /* col dimension of x */
  int delta;      /* number of rows to shift up */
{

  double *y;         /* pointer to output image */
  int i;          /* loop counters */
  int targ_row;   /* row currently being copied to */
  int src_row;    /* row currently being copied from */

 /*
  * detect zero dimension input
  */
  if ((row_dim==0) || (col_dim==0)) return NULL;

 /*
  * detect delta larger than image
  */
  if (delta >= row_dim) {
    printf("\nd_ShiftUpRows : delta larger than row_dim !");
    return NULL;
  }

 /*
  * detect delta negative
  */
  if (delta < 0) {
    printf("\nd_ShiftUpRows : delta negative !");
    return NULL;
  }

 /* 
  * allocate memory for y, return error if out of memory
  */
  y = (double *) malloc(row_dim * col_dim * sizeof(double));
  if (y == NULL) {
    printf("\nd_ShiftUpRows.c : out of memory error !");
    return NULL;
  }

 /*
  * set y(i,j) = x(i+delta,j).  First, shift the rows that can be
  * shifted directly.
  */
  for (targ_row=0,src_row=delta; src_row<row_dim; targ_row++,src_row++) {
    for (i=0; i<col_dim; i++) {
      *(y+ targ_row*col_dim + i) = *(x+ src_row*col_dim + i);
    }
  }
 /*
  * Now do the reflection
  */
  for (src_row-=2; targ_row<row_dim; targ_row++,src_row--) {
    for (i=0; i<col_dim; i++) {
      *(y+ targ_row*col_dim + i) = *(x+ src_row*col_dim + i);
    }
  }
  return y;
} 
#endif
