/*
 * d_dRow.c
 *
 *   Function returns the first central difference of integer input image x
 *   with respect to the first (row) coordinate:
 *
 *        y(i,j) = [x(i+1,j) - x(i-1,j)]/2 ~ d/di x
 *
 *   where x is declared int x[row_dim][col_dim], and y has the
 *   same dimensions.  The output image y is allocated on the heap.
 *
 *  jph 4 June 1992
 */

#ifndef D_DROW_C
#define D_DROW_C
#include <stdio.h>
#include "teager2D.h"

int* d_dRow(x,row_dim,col_dim)

  int* x;         /* input image */
  int row_dim;    /* row dimension of x */
  int col_dim;    /* col dimension of x */
{

  int *y;         /* pointer to output image */
  int *a,*b,*c;   /* temp images */
  int i,j;        /* loop counters */

 /*
  * detect zero dimension input
  */
  if ((row_dim==0) || (col_dim==0)) return NULL;

 /*
  * set y = d/di x
  */
  a = shiftimage(x,row_dim,col_dim, 1,0);
  b = shiftimage(x,row_dim,col_dim,-1,0);
  c = subtractimage(a,b,row_dim,col_dim);
  y = scaleimage(c,row_dim,col_dim,(double)0.5);
  free(a);
  free(b);
  free(c);

  return y;
} 
#endif
