/*
 * d_d_dCol.c
 *
 *   Function returns the first central difference of double input image x
 *   with respect to the second (col) coordinate:
 *
 *        y(i,j) = [x(i,j+1) - x(i,j-1)]/2 ~ d/dj x
 *
 *   where x is declared double x[row_dim][col_dim], and y has the
 *   same dimensions.  The output image y is allocated on the heap.
 *
 *  jph 10 June 1992
 */

#ifndef D_D_DCOL_C
#define D_D_DCOL_C
#include <stdio.h>
#include "teager2D.h"

double* d_d_dCol(x,row_dim,col_dim)

  double* x;         /* input image */
  int row_dim;    /* row dimension of x */
  int col_dim;    /* col dimension of x */
{

  double *y;         /* pointer to output image */
  double *a,*b,*c;   /* temp images */
  int i,j;        /* loop counters */

 /*
  * detect zero dimension input
  */
  if ((row_dim==0) || (col_dim==0)) return NULL;


 /*
  * set y = d/dj x
  */
  a = d_shiftimage(x,row_dim,col_dim,0, 1);
  b = d_shiftimage(x,row_dim,col_dim,0,-1);
  c = d_subtractimage(a,b,row_dim,col_dim);
  y = d_scaleimage(c,row_dim,col_dim,(double)0.5);
  free(a);
  free(b);
  free(c);

  return y;
} 
#endif
