/*
 * d_dbx.c
 *
 *   function converts a double image to unsigned char (byte) with
 *   maximum dynamic range.
 *
 *   the byte image is allocated on the stack.
 *
 *  jph 15 June 1992
 */

#ifndef D_DBX_C
#define D_DBX_C
#include <stdio.h>
#include "teager2D.h"

unsigned char* d_dbx(x,row_dim,col_dim)

  double* x;         /* input image */
  int row_dim;    /* row dimension of x */
  int col_dim;    /* col dimension of x */
{

  unsigned char *y;         /* pointer to output image */
  int i;             /* loop counters */
  double *xtemp;     /* temporary double image */
  double *xxtemp;    /* temporary double image */
  double min,max;    /* min and max pixel values in x */
  double factor;     /* multiplicative factor */

 /*
  * detect zero dimension input
  */
  if ((row_dim==0) || (col_dim==0)) return NULL;

 /* 
  * allocate memory for xtemp, return error if out of memory
  */
  xtemp = (double *) malloc(row_dim * col_dim * sizeof(double));
  if (xtemp == NULL) {
    printf("\nd_dbx.c : out of memory error !");
    return NULL;
  }

 /*
  * set xtemp = x
  */
  max = min = x[0];
  for (i=0; i<row_dim * col_dim; i++) {
    xtemp[i] = x[i];
    if (x[i] > max) max = x[i];
    else if (x[i] < min) min = x[i];
  }

 /*
  * scale for maximum dynamic range
  */ 
  if (max == min) {
    free(xtemp);
    printf("\nd_dbx : divide by zero, image has no dynamic range !");
    makepos(x,&xxtemp,row_dim,col_dim);
  }
  else {
    for (i=0; i<row_dim*col_dim; i++)
      xtemp[i] -= min;
    factor = (double) 255.0 / (max - min);
    xxtemp = d_scaleimage(xtemp,row_dim,col_dim,factor);
    free(xtemp);
  }
  
  y = double2byte(xxtemp,row_dim,col_dim);
  free(xxtemp);
  return y;
} 
#endif
