/*
 * d_multimage.c
 *
 *   function returns the pointwise product of two double images:
 *
 *        z(i,j) = x(i,j)*y(i,j)
 *
 *   where x and y are declared double x[row_dim][col_dim], and z has the
 *   same dimensions.  The output image y is allocated on the heap.
 *
 *  jph 10 June 1992
 */

#ifndef D_MULTIMAGE_C
#define D_MULTIMAGE_C
#include <stdio.h>
#include "teager2D.h"

double* d_multimage(x,y,row_dim,col_dim)

  double *x,*y;      /* input images */
  int row_dim;    /* row dimension of x,y */
  int col_dim;    /* col dimension of x,y */
{

  double *z;         /* pointer to output image */
  int i;          /* loop counter */

 /*
  * detect zero dimension input
  */
  if ((row_dim==0) || (col_dim==0)) return NULL;

 /* 
  * allocate memory for z, return error if out of memory
  */
  z = (double *) malloc(row_dim * col_dim * sizeof(double));
  if (z == NULL) {
    printf("\nd_multimage.c : out of memory error !");
    return NULL;
  }

 /*
  * set z = x*y
  */
  for (i=0; i<row_dim * col_dim; i++) {
    *(z + i) = (*(x + i)) * (*(y + i));
  }
  return z;
} 
#endif
