/*
 * d_shiftimage.c
 *
 *   function returns a shifted version of the input image according
 *   to:
 *
 *        y(i,j) = x(i+delta_row,j+delta_col)
 *
 *   where x is declared double x[row_dim][col_dim], and y has the
 *   same dimensions.  The output image y is allocated on the heap.
 *   When values outside the domain of x are required, the convention
 *   for edge effects is to reflect about the edge pixle.  For example,
 *   it is assumed that x[-1][k] = x[1][k] (reflection about x[0][k].
 *
 *  jph 10 June 1992
 */

#ifndef D_SHIFTIMAGE_C
#define D_SHIFTIMAGE_C
#include <stdio.h>
#include "teager2D.h"

double* d_shiftimage(x,row_dim,col_dim,delta_row,delta_col)

  double* x;         /* input image */
  int row_dim;    /* row dimension of x */
  int col_dim;    /* col dimension of x */
  int delta_row;  /* number of rows to shift, can be + or - */
  int delta_col;  /* number of cols to shift, can be + or - */
{

  double *y;         /* pointer to output image */
  double *src;       /* points to an intermediate image  result */
  int i,j;        /* loop counters */

 /*
  * detect zero dimension input
  */
  if ((row_dim==0) || (col_dim==0)) return NULL;

 /*
  * detect delta_row > row_dim or delta_col > col_dim
  */
  if ((delta_row > row_dim) || (delta_col > col_dim)) {
    printf("\nd_shiftimage.c : delta > image dimension !");
    return NULL;
  }

 /*
  * is there a row shift ? If so, do it!
  */
  if (delta_row > 0) 
    src = d_ShiftUpRows(x,row_dim,col_dim,delta_row);
  else if (delta_row < 0)
         src = d_ShiftDownRows(x,row_dim,col_dim,abs(delta_row));
       else src = x;

 /*
  * is there a col shift ? If so, do it!
  */
  if (delta_col > 0)
    y = d_ShiftLeftCols(src,row_dim,col_dim,delta_col);
  else if (delta_col < 0)
         y = d_ShiftRightCols(src,row_dim,col_dim,abs(delta_col));
       else y = src;

 /*
  * if a temp image was allocated, free it
  */
  if ((delta_row != 0) && (delta_col != 0)) {
    free(src);
  }

  return y;
} 
#endif
