/*
 * d_sqimage.c
 *
 *   function returns the square of a double image, defined
 *   as:
 *
 *        y(i,j) = x(i,j)*x(i,j)
 *
 *   where x is declared double x[row_dim][col_dim], and y has the
 *   same dimensions.  The output image y is allocated on the heap.
 *
 *  jph 3 June 1992
 */

#ifndef SQIMAGE_C
#define SQIMAGE_C
#include <stdio.h>
#include "teager2D.h"

double* d_sqimage(x,row_dim,col_dim)

  double* x;         /* input image */
  int row_dim;    /* row dimension of x */
  int col_dim;    /* col dimension of x */
{

  double *y;         /* pointer to output image */
  int i,j;        /* loop counters */

 /*
  * detect zero dimension input
  */
  if ((row_dim==0) || (col_dim==0)) return NULL;

 /* 
  * allocate memory for y, return error if out of memory
  */
  y = (double *) malloc(row_dim * col_dim * sizeof(double));
  if (y == NULL) {
    printf("\nd_sqimage.c : out of memory error !");
    return NULL;
  }

 /*
  * set y = x*x
  */
  for (i=0; i<row_dim * col_dim; i++) {
    *(y + i) = (*(x + i)) * (*(x + i));
  }
  return y;
} 
#endif
