/*
 * d_teager2D.c
 *
 *   function returns an image that at every point is equal to the 2D discrete
 *   Teager operator invoked on the input image at that point:
 *
 *        y(i,j) = 2x(i,j)^2 - x(i+1,j)x(i-1,j) - x(i,j-1)x(i,j+1)
 *
 *   where x is declared float x[row_dim][col_dim], and y has the
 *   same dimensions.  The output image y is allocated on the heap.
 *
 *  jph 10 June 1992
 */

#ifndef D_TEAGER2D_C
#define D_TEAGER2D_C
#include <stdio.h>
#include "teager2D.h"

double* d_teager2D(x,row_dim,col_dim)

  double* x;         /* input image */
  int row_dim;    /* row dimension of x */
  int col_dim;    /* col dimension of x */
{

  double *a;         /* temp image */
  int i;          /* loop counter */

 /*
  * detect zero dimension input
  */
  if ((row_dim==0) || (col_dim==0)) return NULL;

 /*
  * a = 2x(i,j)^2 - x(i+1,j)x(i-1,j) - x(i,j+1)x(i,j-1)
  */
  a = d_subtractimage(
        d_subtractimage(
          d_scaleimage(
            d_sqimage(x,row_dim,col_dim),
            row_dim,col_dim,(double)2.0
          ),
          d_multimage(
            d_shiftimage(x,row_dim,col_dim, 1,0),
            d_shiftimage(x,row_dim,col_dim,-1,0),
            row_dim,col_dim
          ),
          row_dim,col_dim
        ),
        d_multimage(
          d_shiftimage(x,row_dim,col_dim,0, 1),
          d_shiftimage(x,row_dim,col_dim,0,-1),
          row_dim,col_dim
        ),
        row_dim,col_dim
      );

 /*
  * fix up negative Teager-Kaiser energy due to discrete approximation errors
  */
  for (i=0; i<row_dim*col_dim; i++)
    if ( (*(a+i)) < 0.0) (*(a+i)) = 0.0;
   
  return a;
} 
#endif
