/*
 * fCovar.c
 *
 *   Function returns the sample covariance of two float images.
 *
 *  jph 21 February 1996
 */

#ifndef FCOVAR_C
#define FCOVAR_C
#include <stdio.h>
#include <math.h>
#include "teager2D.h"

float fCovar(x,y,row_dim,col_dim)

  float    *x;       /* 1st input image */
  float    *y;       /* 2nd input image */
  int       row_dim; /* row dimension of x */
  int       col_dim; /* col dimension of x */
{

  int      N;        /* number of pixels in x */
  int      i;        /* loop counter */
  float    Cov;      /* computed covariance of x and y */
  float    sum;      /* sum of (x_i - Ex)(y_i - Ey) */
  float    Ex;       /* sample mean of x */
  float    Ey;       /* sample mean of y */

 /*
  * detect zero dimension input
  */
  if ((row_dim==0) || (col_dim==0)) return((float)0.0);

 /*
  * Find means
  */
  Ex = fmean(x,row_dim,col_dim);
  Ey = fmean(y,row_dim,col_dim);

 /*
  *  Find covariance
  */
  N = row_dim * col_dim;
  for (sum=(float)0.0,i=0; i < N; i++) {
    sum += (x[i] - Ex) * (y[i] - Ey);
  }
  Cov = sum / (float)N;

  return(Cov);
} 
#endif
