/*
 * fFFTConvolve.c
 *
 *  2D linear convolution by pointwise multiplication of DFTs.
 *  The input (X) and filter (G) images must be allocated and initialized in 
 *  both the space and frequency domains.  The output image (Y) must be 
 *  allocated, but does not have to be initialized.  It will be overwritten.
 *  All three images must be the same size.
 *
 *  jph 2/13/99
 */

#ifndef FFFTCONVOLV_C
#define FFFTCONVOLV_C
#include <stdio.h>
#include "teager2D.h"

void fFFTConvolve(Y,X,G)

  pFloatImage   Y;  /* output image */
  pFloatImage   X;  /* input  image */
  pFloatImage   G;  /* filter image */
{

  int         k;       /* counter for image pixels */
  int         n;       /* number of image pixels */

 /*
  * Compute the spectrum of the output
  */
  n = X->size * X->size;
  for (k=0; k < n; k++) {
    fCMult(Y->FFTReal[k],Y->FFTImag[k],
           X->FFTReal[k],X->FFTImag[k],
           G->FFTReal[k],G->FFTImag[k]);
  }

 /*
  * Compute the image domain filtered response by inverse fft
  */
  fft2d(Y->Real,Y->Imag,Y->FFTReal,Y->FFTImag,X->size,-1);

  return;
} 
#endif
