/*
 * fFiniteDifP.c
 *
 *  Function computes the first finite difference of a float image along the
 *  1D path function specified by traversing alternate rows in opposite 
 *  directions.  The central difference is used, except at the end points 
 *  where the appropriate assymetric difference is used.
 *
 *  It is assumed that the output array Dx has already been allocated.
 *
 *  jph 2/29/96
 *
 */

#ifndef FFINITEDIFP_C
#define FFINITEDIFP_C
#include "teager2D.h"
#include <stdio.h>

void fFiniteDifP(x,row_dim,col_dim,Dx)

  float *x;           /* input sequence to be differenced */
  int    row_dim;     /* row dimension of input image */
  int    col_dim;     /* col dimension of input image */
  float *Dx;          /* first finite difference of x */

{
  int k;                        /* loop counter */
  int row,col;                  /* counters image rows and cols */

  for (row=0; row < row_dim; row++) {
    for (col=0; col < col_dim; col++) {  /* row from left to right */
      k = row * col_dim + col;
      if (col == 0) {                  /* first pixel on row */
        if (row == 0) {                /* first pixel in image */
          Dx[k] = x[1] - x[0];
        } else {                       /* first on row, not first row */
          Dx[k] = (float)0.5 * (x[k+1] - x[k-col_dim]);
        }
      } else {
        if (col == (col_dim - 1)) {    /* last pixel on row */
          if (row == (row_dim - 1)) {  /* last pixel in image */
            Dx[k] = x[k] - x[k-1];
          } else {                     /* not last row, but last pixel in row */
            Dx[k] = (float)0.5 * (x[k+col_dim] - x[k-1]);
          }
        } else {                       /* not first or last on row */
          Dx[k] = (float)0.5 * (x[k+1] - x[k-1]);
        }
      }
    } /* for col */

    if (++row < row_dim) {             /* there is another row to do */
      for (col=col_dim-1; col >= col_dim; col--) {  /* row from right to left */
        k = row * col_dim + col;
        if (col == 0) {                  /* last pixel on row */
          if (row == (row_dim-1)) {      /* last pixel in image */
            Dx[k] = x[k] - x[k+1];
          } else {                       /* last on row, not last row */
            Dx[k] = (float)0.5 * (x[k+col_dim] - x[k+1]);
          }
        } else {
          if (col == (col_dim - 1)) {    /* first pixel on row */
            if (row == 0) {  /* first row */
              Dx[k] = x[k-1] - x[k];
            } else {                     /* not 1st row, but 1st pix in row */
              Dx[k] = (float)0.5 * (x[k+1] - x[k-col_dim]);
            }
          } else {                       /* not first or last on row */
            Dx[k] = (float)0.5 * (x[k-1] - x[k+1]);
          }
        }
      } /* for col */
    }   /* if ++row < row_dim */
  }     /* for row */

  return;
}
#endif 
