
/*
 * fSigCmprs.c
 *
 * Scaled log compression of outliers in image.
 *
 * The output image is allocated on the heap.
 *
 * USE OF fmed3x3 prior to this program is HIGHLY recommended.
 *
 * The threshold is set to "sigmas" times the std dev of the image.
 *
 * If a pixel deviates from the mean by more than thresh, then the ammount
 * by which it exceeds thresh will be logarithmically scaled.
 *
 * The log of the deviation is further scaled so that the maximum deviation from
 * the mean in the image is "sigmas" + 1 standard deviations.
 *
 * jph 4/5/96
 *
 */

#ifndef FSIGCMPRS_C
#define FSIGCMPRS_C

#include "teager2D.h"

float *fSigCmprs(xx,size,Sigmas)

  float  *xx;                /* input image  */
  int     size;              /* row , col dim */
  float   Sigmas;
{
  float  *x;                 /* input image  */
  int     k;
  float   min,med,max;
  float   mean,sigma;
  double  Thresh;     
  double  deviation;
  double  MaxDev;            /* maximum deviation from mean in image */
  double  Fact;              /* scale factor, makes max dev Sigmas+1 sigmas */
 
 /*
  * Allocate x on the heap
  */
  if ((x = (float *)malloc(size*size*sizeof(float))) == NULL) {
    printf("\n\nfSigCmprs: free store exhausted.\n");
    exit(-1);
  } 

 /*
  * copy xx to x
  */
  for (k=0; k < size*size; k++) {
    x[k] = xx[k];
  }


  fsstat(x,size,size,&min,&med,&max,&mean,&sigma);

  Thresh = (double)Sigmas * (double)sigma;
  MaxDev = (double)0.0;
  Fact   = (double)1.0;

 /*
  * find the max deviation 
  */
  for (k=0; k < size*size; k++) {
    deviation = fabs((double)(x[k] - mean)); 
    if (deviation > Thresh) {
      if (deviation > MaxDev) {
        MaxDev = deviation;
      }
    }
  }
  if (MaxDev > (double)0.0000001) {
    Fact = ((double)(Sigmas + (float)1.0) * (double)sigma) 
             / log(MaxDev - Thresh + (double)1.0);
  }

  for (k=0; k < size*size; k++) {
    deviation = fabs((double)(x[k] - mean)); 
    if (deviation > Thresh) {
      deviation = Thresh + Fact * log(deviation - Thresh + 1.0);
      if (x[k] < mean) {
        x[k] = mean - deviation;
      } else {
        x[k] = mean + deviation;
      }
    }
  }

  return(x);
} 
#endif
