/*
 * f_dbx.c
 *
 *   function converts a float image to unsigned char (byte) with
 *   maximum dynamic range.
 *
 *   the byte image is allocated on the heap.
 *
 *  jph 9/11/93
 */

#ifndef F_DBX_C
#define F_DBX_C
#include <stdio.h>
#include "teager2D.h"

unsigned char* f_dbx(x,row_dim,col_dim)

  float* x;         /* input image */
  int row_dim;    /* row dimension of x */
  int col_dim;    /* col dimension of x */
{

  unsigned char *y;         /* pointer to output image */
  int i;             /* loop counters */
  float *xtemp;     /* temporary double image */
  float *xxtemp;    /* temporary double image */
  float min,max;    /* min and max pixel values in x */
  float factor;     /* multiplicative factor */

 /*
  * detect zero dimension input
  */
  if ((row_dim==0) || (col_dim==0)) return NULL;

 /* 
  * allocate memory for xtemp, return error if out of memory
  */
  xtemp = (float *) malloc(row_dim * col_dim * sizeof(float));
  if (xtemp == NULL) {
    printf("\nf_dbx.c : out of memory error !");
    return NULL;
  }

 /*
  * set xtemp = x
  */
  max = min = x[0];
  for (i=0; i<row_dim * col_dim; i++) {
    xtemp[i] = x[i];
    if (x[i] > max) max = x[i];
    else if (x[i] < min) min = x[i];
  }

 /*
  * scale for maximum dynamic range
  */ 
  if (max == min) {
    free(xtemp);
    printf("\nf_dbx : divide by zero, image has no dynamic range !");
    fmakepos(x,&xxtemp,row_dim,col_dim);
  }
  else {
    for (i=0; i<row_dim*col_dim; i++)
      xtemp[i] -= min;
    factor = (float) 255.0 / (max - min);
    xxtemp = f_scaleimage(xtemp,row_dim,col_dim,factor);
    free(xtemp);
  }
  
  y = f2b(xxtemp,row_dim,col_dim);
  free(xxtemp);
  return y;
} 
#endif
