/*
 * f_scaleimage.c
 *
 *   function returns the scalar multiple of an image, defined
 *   as:
 *
 *        y(i,j) = c * x(i,j)
 *
 *   where x is declared float x[row_dim][col_dim], c is a (float) scalar,
 *   and y has the same dimensions as x.  The output image y is allocated 
 *   on the heap.
 *
 *  jph 9/11/93
 */

#ifndef F_SCALEIMAGE_C
#define F_SCALEIMAGE_C
#include <stdio.h>
#include "teager2D.h"

float* f_scaleimage(x,row_dim,col_dim,c)

  float* x;         /* input image */
  int row_dim;    /* row dimension of x */
  int col_dim;    /* col dimension of x */
  float c;       /* factor by which to scale */
{

  float *y;         /* pointer to output image */
  int i,j;        /* loop counters */

 /*
  * detect zero dimension input
  */
  if ((row_dim==0) || (col_dim==0)) return NULL;

 /* 
  * allocate memory for y, return error if out of memory
  */
  y = (float *) malloc(row_dim * col_dim * sizeof(float));
  if (y == NULL) {
    printf("\nf_scaleimage.c : out of memory error !");
    return NULL;
  }

 /*
  * set y = c*x
  */
  for (i=0; i<row_dim * col_dim; i++) {
    y[i] = c * x[i];
  }
  return y;
} 
#endif
