/*
 * f_u_norm.c
 *
 *   Scale a complex image to take values between -1 and +1.  Input real 
 *   image xR is not NULL.  Input imaginary image xI may be NULL.  If xI
 *   is NULL, normalize xR to lie in [-1,1].  If both xR and xI are not
 *   NULL, then normalize so that both lie in [-1,1] using the global 
 *   min and max of both images.
 *
 *  jph 27 October 1993
 */

#ifndef F_U_NORM
#define F_U_NORM
#include <stdio.h>
#include "teager2D.h"

void f_u_norm(xR,xI,row_dim,col_dim)

  float* xR;         /* Real Input Image */
  float* xI;         /* Imaginary Input Image */
  int row_dim;       /* row dimension of input image */
  int col_dim;       /* col dimension of input image */
{

 /*
  * Local variable frame
  */
  float MaxMag;      /* half the range of input image x */
  float Mag;         /* magnitude of an image element */
  float RangeFact;   /* one over HalfRange */
  int   i;           /* loop counter */
  int   size;        /* number of pixels in input image x */
 
 /*
  * detect zero dimension input
  */
  if ((row_dim==0) || (col_dim==0)) return;
  size = row_dim * col_dim;

 /*
  * Find max magnitude of xR
  */
  MaxMag = (float)fabs((double)xR[0]);
  for (i=1; i<size; i++) {
    if ((Mag = (float)fabs((double)xR[i])) > MaxMag) MaxMag = Mag;
  }

 /*
  * If xI is not NULL, search it for the global maximum
  */
  if (xI != NULL) {
    for (i=0; i<size; i++) {
      if ((Mag = (float)fabs((double)xI[i])) > MaxMag) MaxMag = Mag;
    }
  }

 /*
  * Normalize 
  */
  RangeFact = (float) 1.0 / MaxMag;
  for (i=0; i<size; i++) {
    xR[i] *= RangeFact;
  }
  if (xI != NULL) {
    for (i=0; i<size; i++) {
      xI[i] *= RangeFact;
    }
  }
  return;
} 
#endif
