/*
 * fmed3x3.c 
 *
 *   Do a 3x3 median filter on a float image.
 *
 *   The output image y is allocated on the heap.
 *
 *   Filtering is performed in a rectangular region with upper left corner
 *   (row1,col1) and lower right corner (row2,col2).
 */

#ifndef FMED3X3_C
#define FMED3X3_C

#include "teager2D.h"

float *fmed3x3(x,size,row1,col1,row2,col2)

  float * x;                     /* input */
  int     size;                  /* row , col dim */
  int     row1,col1,row2,col2;   /* limits of where to run filter */
{

  float * y;             /* output image */
  float   X[9];          /* neighborhood */
  float   Xtemp;         /* temp floating variable */
  int     k;             /* loop counter */
  int     i;             /* loop counter */
  int     j;             /* loop counter */
  int     row,col;       /* loop counters */

 /*
  * Allocate y on the heap
  */
  if ((y = (float *)malloc(size*size*sizeof(float))) == NULL) {
    printf("\n\nfmed3x3: free store exhausted.\n");
    exit(-1);
  }

 /*
  * copy x to y
  */
  for (k=0; k < size*size; k++) {
    y[k] = x[k];
  }

 /*
  * invoke median filter on y in the indicated region
  */
  for (row=row1; row <=row2; row++) {
    for (col=col1; col <= col2; col++) {
      k = row*size+col;
      X[0] = x[k-size-1];       
      X[1] = x[k-size];       
      X[2] = x[k-size+1];       
      X[3] = x[k-1];       
      X[4] = x[k];       
      X[5] = x[k+1];       
      X[6] = x[k+size-1];       
      X[7] = x[k+size];       
      X[8] = x[k+size+1];       
      for (i=0; i < 5; i++) {
        for (j=i+1; j < 9; j++) {
          if (X[j] > X[i]) {
            Xtemp = X[j];
            X[j]  = X[i];
            X[i]  = Xtemp;
          }
        }
      }
      y[k] = X[4];
    }
  }

  return(y);
}
#endif
