/*
 * fsstat.c
 *
 *   function computes simple statistics of the pixel values of a
 *   float image.
 *
 *
 *  jph 15 June 1992
 */

#ifndef FSSTAT_C
#define FSSTAT_C
#include <stdio.h>
#include <math.h>
#include "teager2D.h"

void fsstat(x,row_dim,col_dim,min,median,max,mean,sigma)

  float *x;          /* input image */
  int row_dim;       /* row dimension of x */
  int col_dim;       /* col dimension of x */
  float *min;        /* computed minimum mean pixel value in x */
  float *median;     /* computed median pixel value in x */
  float *max;        /* computed maximum pixel value in x */
  float * mean;      /* computed mean pixel value in x */
  float * sigma;     /* computed standard deviation of x */
{

  int i;             /* loop counters */
  float sum;         /* sum of pixel values in x */
  float sumsq;       /* sum of squared pixel values in x */
  double sigsq;      /* sigma squared */
  int N;             /* number of pixels in x */
  int No2;           /* N over 2 */
  int hist[256];     /* histogram of x */
  int ltmdn;         /* number of pix < current guess for median */

 /*
  * detect zero dimension input
  */
  if ((row_dim==0) || (col_dim==0)) return;

 /*
  *  Find min, max, mean, and sigma
  */
  No2 = (N = row_dim * col_dim) / 2;
  sum = sumsq = 0.0;

  *min = *max = x[0];
  for (i=0; i<N; i++) {
    sum += x[i];
    sumsq += x[i] * x[i];
    if (x[i] > (*max)) *max = x[i];
    else if (x[i] < (*min)) *min = x[i];
  }
  *mean = sum / (float) N;
  sigsq = (double)sumsq / (double)N - pow((double)*mean,(double)2.0);
  if (sigsq >= (double)0.0) {
    *sigma = sqrt(sigsq);
  } else {
    *sigma = (float)0.0;
  }

 /*
  * find median by histogram method
  */
  if (min==max) median = min;
  else {
    for (i=0; i<256; i++)  hist[i] = 0;
    for (i=0; i<N; i++) hist[(int)((x[i]-*min)/(*max-*min) * 256)]++;
    for(i=-1,ltmdn=0; ltmdn<No2; ) {
      i++;
      ltmdn += hist[i];
    }
    *median = (float) i / (float) 256 * (*max - *min) + *min;
  }

  return;
} 
#endif
