/*
 * fvariance.c
 *
 *   Function returns the variance of a float image.
 *   The mean must already have been computed using fmean.c.
 *
 *  jph 21 February 1996
 */

#ifndef FVARIANCE_C
#define FVARIANCE_C
#include <stdio.h>
#include <math.h>
#include "teager2D.h"

float fvariance(x,row_dim,col_dim,mean)

  float    *x;       /* input image */
  int       row_dim; /* row dimension of x */
  int       col_dim; /* col dimension of x */
  float     mean;    /* mean pixel value in x */
{

  int      N;        /* number of pixels in x */
  int      i;        /* loop counter */
  float    Var;      /* computed variance of x */
  float    sumsq;    /* sum of squared pixel values in x */

 /*
  * detect zero dimension input
  */
  if ((row_dim==0) || (col_dim==0)) return((float)0.0);

 /*
  *  Find variance
  */
  N = row_dim * col_dim;
  for (sumsq=(float)0.0,i=0; i < N; i++) {
    sumsq += x[i] * x[i];
  }
  Var = sumsq / (float)N - mean*mean;

  return(Var);
} 
#endif
