/*
 * gMakeImag.c
 *
 *  Function returns a float array equal to the Imaginary part of the filter 
 *  function of the specified gabor filter.  If zerophase=1, a zero phase 
 *  filter is returned.  If zerophase=0, a filter centered at the center of 
 *  the image is returned.  The image is assumed square, with row (col) 
 *  dimension size.  The float array is allocated on the heap.
 *
 *  jph 9/10/93
 *
 */

#ifndef gMAKEIMAG_C
#define gMAKEIMAG_C
#include "teager2D.h"
#include <stdio.h>

float *gMakeImag(f,zerophase,size)

  FilterPtr   f;         /* filter to be evaluated */
  int         zerophase; /* 1: zero-phase filter, 0: filter centered in plane */
  int         size;      /* row (col) dimension of the image */

{

  float      *gImag;     /* array of filter Real values */
  int         row;       /* loop counter for image rows */
  int         col;       /* loop counter for image cols */
  int         i;         /* indexes the array */

 /*
  * Allocate the filter on the heap
  */
  if ((gImag = (float *)malloc(size*size*sizeof(float))) == NULL) {
    printf("\n\ngMakeImag: free store exhausted.\n");
    exit(-1);
  }
  
 /*
  * Make the data
  */
  for (i=0, row=0; row < size; row++) {
    for (col=0; col < size; col++, i++) {
      gImag[i] = g_EvalImag(f,(double)col,(double)row,zerophase,size);
    }
  }

  return(gImag);

}
#endif
