/*
 * g_EvalImag.c
 *
 *  Function image inputs (x,y) image coordinates, a filter, and the row (col)
 *  dimension of the image (the image is assumed to be square).  The Imaginary 
 *  value of the filter at the point (x,y) is returned.  If zerophase=1, the 
 *  value for a zero phase filter is returned.  If zerophase=0, the value for 
 *  a filter centered at the center of the image is returned.
 *
 *  For the zero phase filter, image coordinates and domain coordinates are the
 *  same.  Periodic reflection is performed, so the function will show up spread
 *  across the outside corners of the four quadrants.  See Book VI pp. 118-119 
 *  for an explanation.
 *
 *  For the centered filter, the function will show up in the center of the 
 *  image.  This means that domain coordinates and image coordinates differ by 
 *  half the row (col) dimension of the image, so that the (0,0) point of the 
 *  function is in the center of the image.
 *
 *  jph 9/10/93
 *
 */

#ifndef g_EVALIMAG_C
#define g_EVALIMAG_C
#include "teager2D.h"
#include <stdio.h>

float g_EvalImag(f,x,y,zerophase,size)

  FilterPtr   f;      /* filter to be evaluated */
  double      x;      /* horiz image coordinate at which to evaluate filter */
  double      y;      /* vert  image coordinate at which to evaluate filter */
  int         zerophase; /* 1: zero-phase filter, 0: filter centered in plane */
  int         size;   /* row (col) dimension of the image */

{

  int         so2;    /* size / 2 */
  float       xx;     /* horiz domain coordinate at which to evaluate filter */
  float       yy;     /* vert  domain coordinate at which to evaluate filter */
  double      twopi;  /* two time pi */


  so2   = size >> 1;
  twopi = (double)2.0 * fabs(acos((double)-1.0));

  if (zerophase) {
    xx = (x < so2) ? x : x - size;
    yy = (y < so2) ? y : y - size;
  }
  else {
    xx = x - so2;
    yy = y - so2;
  }
  return(f->gMax * (float)
                   (exp(f->gArgGaussCoef * 
                    (pow((double)xx,(double)2.0) + pow((double)yy,(double)2.0)))
                   * sin(twopi*(f->u0*(double)xx + f->v0*(double)yy))));
}
#endif
