/*
 * g_EvalMag.c
 *
 *  Function image inputs (x,y) image coordinates, a filter, and the row (col)
 *  dimension of the image (the image is assumed to be square).  The magnitude 
 *  of the filter at the point (x,y) is returned.  If zerophase=1, the value 
 *  for a zero phase filter is returned.  If zerophase=0, the value for a filter *  centered at the center of the image is returned.
 *
 *  For the zero phase filter, image coordinates and domain coordinates are the
 *  same.  Periodic reflection is performed, so the function will show up spread
 *  across the outside corners of the four quadrants.  See Book VI pp. 118-119 
 *  for an explanation.
 *
 *  For the centered filter, the function will show up in the center of the 
 *  image.  This means that domain coordinates and image coordinates differ by 
 *  half the row (col) dimension of the image, so that the (0,0) point of the 
 *  function is in the center of the image.
 *
 *  jph 9/10/93
 *
 */

#ifndef g_EVALMAG_C
#define g_EVALMAG_C
#include "teager2D.h"
#include <stdio.h>

float g_EvalMag(f,x,y,zerophase,size)

  FilterPtr   f;      /* filter to be evaluated */
  double      x;      /* horizontal domain coordinate at which to evaluate filter */
  double      y;      /* vertical domain coordinate at which to evaluate filter */
  int         zerophase; /* 1: zero-phase filter, 0: filter centered in plane */
  int         size;   /* row (col) dimension of the image */

{

return(fNorm(
      g_EvalReal(f,x,y,zerophase,size), g_EvalImag(f,x,y,zerophase,size)));
 
}
#endif
