/*
 * logimage.c
 *
 *   function returns the natural logarithm of a double image, defined
 *   as:
 *
 *        y(i,j) = log(x(i,j))
 *
 *   where x is declared double x[row_dim][col_dim], and y has the
 *   same dimensions.  The output image y is allocated on the heap.
 *
 *  jph 15 June 1992
 */

#ifndef LOGIMAGE_C
#define LOGIMAGE_C
#include <stdio.h>
#include <math.h>
#include "teager2D.h"

double* logimage(x,row_dim,col_dim)

  double* x;         /* input image */
  int row_dim;    /* row dimension of x */
  int col_dim;    /* col dimension of x */
{

  double *y;         /* pointer to output image */
  int i;             /* loop counters */

 /*
  * detect zero dimension input
  */
  if ((row_dim==0) || (col_dim==0)) return NULL;

 /* 
  * allocate memory for y, return error if out of memory
  */
  y = (double *) malloc(row_dim * col_dim * sizeof(double));
  if (y == NULL) {
    printf("\nlogimage.c : out of memory error !");
    return NULL;
  }

 /*
  * set y = log(x)
  */
  for (i=0; i<row_dim * col_dim; i++) {
    if (x[i] <= (double) 0.0) 
      printf("\nlogimage.c: take log of nonpositive quantity !\n");
    y[i] = log(x[i]);
  }
  return y;
} 
#endif
