/*
 * makepos.c
 *
 *   If all values of double input image x are positive, then no action is
 *   taken.  Otherwise, (1 - minval) is added to each pixel, where
 *   minval is the smallest value in the image.  This
 *   will make the output image have a minimum value of 1.0.
 *
 *   the offset (1-minval) is returned in the latter case, while zero is
 *   returned in the former.
 *
 *   x is declared double x[row_dim][col_dim] in the calling routine, and y 
 *   has the same dimensions.  The output image y is allocated on the heap.
 *
 *  jph 15 June 1992
 */

#ifndef MAKEPOS_C
#define MAKEPOS_C
#include <stdio.h>
#include "teager2D.h"

double makepos(x,y,row_dim,col_dim)

  double* x;         /* input image */
  double** y;         /* pointer to pointer to output image */
  int row_dim;    /* row dimension of x */
  int col_dim;    /* col dimension of x */
{

  int i;             /* loop counters */
  double minval;     /* smallest value in input image */
  double offset;     /* value to add to input image */

 /*
  * detect zero dimension input
  */
  if ((row_dim==0) || (col_dim==0)) return (double) 0.0;

 /* 
  * allocate memory for y, return error if out of memory
  */
  *y = (double *) malloc(row_dim * col_dim * sizeof(double));
  if (*y == NULL) {
    printf("\nmakepos.c : out of memory error !");
    return 0.0;
  }

 /*
  * find the minimum pixel value in the input image and the offset
  */
  minval = x[0];
  for (i=1; i<row_dim*col_dim; i++)
    if (x[i] < minval) minval = x[i];

  if (minval > (double) 0.0) offset = 0.0;
  else offset = (double) 1.0 - minval;

 /*
  * make y equal to a strictly positive version of x
  */
  for (i=0; i<row_dim * col_dim; i++) {
    (*y)[i] = x[i] + offset;
  }
  return offset;
} 
#endif
