/*
 * print_needle.c
 *
 *   Function prints a needle diagram to a postscript file.
 *   Plotting is done by calling GnuPlot.
 *
 *   After "writegnu.c" by Keith Bartels
 *
 *  jph 22 June 1992
 */

#ifndef PRINT_NEEDLE_C
#define PRINT_NEEDLE_C
#include <stdio.h>
#include <math.h>
#include "teager2D.h"

void print_needle(u,v,domain,row_dim,col_dim,heads,startrow,startcol,
                                                   stoprow,stopcol)

  float  *u, *v;           /* needles in cartesian coordinates */
  unsigned char *domain;  /* nonzero where a needle should originate */
  int heads;
  int startrow,startcol,stoprow,stopcol;
{
  int row,col;            /* loop counters */
  int gnu_row, gnu_col;   /* loop counters */
  FILE  *gnufile;         /* temp file to hold gnuplot commands */
  char fn[100];           /* PostScript filename */
  char cmd[100];          /* system command */

 /*
  * create the gnuplot command file
  */
  gnufile = fopen("___needle_tmp_gnu_file","w");
  fprintf(gnufile,"set nokey\nset size square 2.00,2.00\n");
  fprintf(gnufile,"set noxtics\n");
  fprintf(gnufile,"set nox2tics\n");
  fprintf(gnufile,"set noytics\n");
  fprintf(gnufile,"set noy2tics\n");
  fprintf(gnufile,"set nomxtics\n");
  fprintf(gnufile,"set nomx2tics\n");
  fprintf(gnufile,"set nomytics\n");
  fprintf(gnufile,"set nomy2tics\n");
  fprintf(gnufile,"set xlabel \" \"\n");
  fprintf(gnufile,"set ylabel \" \"\n");
  fprintf(gnufile,"set terminal postscript eps 22\n");
  printf("\n\nEncapsulated PostScript Filename ? ");
  scanf("%s",fn);
  fprintf(gnufile,"set output \"%s\"\n",fn);
  for (row=startrow,gnu_row=row_dim-startrow-1; row<stoprow; row++,gnu_row--)
    for (col=gnu_col=startcol; col<stopcol; col++,gnu_col++)
      if (domain[row*col_dim+col])
        if (heads)
          fprintf(gnufile,"set arrow from %d,%d to %f,%f \n",
            gnu_col,gnu_row,(float)gnu_col+(float)u[row*col_dim+col],
                            (float)gnu_row+(float)v[row*col_dim+col]);
          else
          fprintf(gnufile,"set arrow from %d,%d to %f,%f nohead\n",
            gnu_col,gnu_row,(float)gnu_col+(float)u[row*col_dim+col],
                            (float)gnu_row+(float)v[row*col_dim+col]);
  fprintf(gnufile,"plot [0:%d][0:%d] 0\n",col_dim-1,row_dim-1);
  fclose(gnufile);

 /*
  * Do the plot
  */
  system("gnuplot ___needle_tmp_gnu_file 2>/dev/null");

 /*
  * Save gnu file?
  */
  printf("\n\nSave the gnu -> postscript file? 0=No 1=Yes ");
  scanf("%d",&row);
  if (row) {
    printf("\n\nEncapsulated PostScript Filename ? ");
    scanf("%s",fn);
    sprintf(cmd,"mv ___needle_tmp_gnu_file %s",fn);
    system(cmd);
  } else {
    system("rm ___needle_tmp_gnu_file");
  }
}
#endif
