/*
 * printgnu_needle.c
 *
 *   Function prints a needle diagram gnu file.
 *   Plotting is done by calling GnuPlot.
 *
 *   After "writegnu.c" by Keith Bartels
 *
 *  jph 20 August 1994
 */

#ifndef PRINTGNU_NEEDLE_C
#define PRINTGNU_NEEDLE_C
#include <stdio.h>
#include <math.h>
#include "teager2D.h"

void printgnu_needle(u,v,domain,row_dim,col_dim,heads,startrow,startcol,
                                                      stoprow,stopcol)

  float  *u, *v;           /* needles in cartesian coordinates */
  unsigned char *domain;  /* nonzero where a needle should originate */
  int heads;
  int startrow,startcol,stoprow,stopcol;
{
  int row,col;            /* loop counters */
  int gnu_row, gnu_col;   /* loop counters */
  int Narrows;            /* number of arrows */
  FILE  *gnufile;         /* temp file to hold gnuplot commands */
  char fn[100];           /* PostScript filename */
  char cmd[100];          /* system command */

 /*
  * create the gnuplot command file
  */
  gnufile = fopen("___needle_tmp_gnu_file","w");
  fprintf(gnufile,"set nokey\nset size square 1.00,1.00\n");
  fprintf(gnufile,"set noxtics\n");
  fprintf(gnufile,"set nox2tics\n");
  fprintf(gnufile,"set noytics\n");
  fprintf(gnufile,"set noy2tics\n");
  fprintf(gnufile,"set nomxtics\n");
  fprintf(gnufile,"set nomx2tics\n");
  fprintf(gnufile,"set nomytics\n");
  fprintf(gnufile,"set nomy2tics\n");
  fprintf(gnufile,"set xlabel \" \"\n");
  fprintf(gnufile,"set ylabel \" \"\n");
  for (Narrows=0,row=startrow,gnu_row=row_dim-startrow-1; 
       row<stoprow; row++,gnu_row--) {
    for (col=gnu_col=startcol; col<stopcol; col++,gnu_col++) {
      if (domain[row*col_dim+col]) {
        if ((u[row*col_dim+col] != (float)0.0) || 
           (v[row*col_dim+col] != (float)0.0)) {
          Narrows++;
          if (heads) {
            fprintf(gnufile,"set arrow from %d,%d to %f,%f \n",
              gnu_col,gnu_row,(float)gnu_col+(float)u[row*col_dim+col],
                              (float)gnu_row+(float)v[row*col_dim+col]);
          } else {
            fprintf(gnufile,"set arrow from %d,%d to %f,%f nohead\n",
              gnu_col,gnu_row,(float)gnu_col+(float)u[row*col_dim+col],
                              (float)gnu_row+(float)v[row*col_dim+col]);
          }
        }
      }
    }
  }
  fprintf(gnufile,"plot [0:%d][0:%d] 0\n",col_dim-1,row_dim-1);
  fclose(gnufile);

 /*
  * Do the plot
  */
  printf("\n save name for needle diagram gnu file ?" );
  scanf("%s",fn);
  gnufile = fopen(fn,"w");
  fprintf(gnufile,"# Number of arrows = %d\n",Narrows);
  fprintf(gnufile,"# SIZE = %d\n",row_dim);
  fclose(gnufile);
  sprintf(cmd,"cat ___needle_tmp_gnu_file >> %s",fn);
  system(cmd);
  system("rm ___needle_tmp_gnu_file");
}
#endif
