/*
 * scaleimage.c
 *
 *   function returns the rounded scalar multiple of an integer image, defined
 *   as:
 *
 *        y(i,j) = (int) (c * (double) x(i,j) + 0.5)
 *
 *   where x is declared int x[row_dim][col_dim], c is a (double) scalar,
 *   and y has the same dimensions as x.  The output image y is allocated 
 *   on the heap.
 *
 *  jph 4 June 1992
 */

#ifndef SCALEIMAGE_C
#define SCALEIMAGE_C
#include <stdio.h>
#include "teager2D.h"

int* scaleimage(x,row_dim,col_dim,c)

  int* x;         /* input image */
  int row_dim;    /* row dimension of x */
  int col_dim;    /* col dimension of x */
  double c;       /* factor by which to scale */
{

  int *y;         /* pointer to output image */
  int i,j;        /* loop counters */

 /*
  * detect zero dimension input
  */
  if ((row_dim==0) || (col_dim==0)) return NULL;

 /* 
  * allocate memory for y, return error if out of memory
  */
  y = (int *) malloc(row_dim * col_dim * sizeof(int));
  if (y == NULL) {
    printf("\nscaleimage.c : out of memory error !");
    return NULL;
  }

 /*
  * set y = c*x
  */
  for (i=0; i<row_dim * col_dim; i++) {
    y[i] = (int) (c * (double)x[i] + ( x[i] < 0? -0.5 : 0.5));
  }
  return y;
} 
#endif
