/*
 * squarepad.c
 *
 *   y[i][j] = x[i][j] for 0 <= i,j < row_dim,col_dim
 *   y[i][j] = 0 otherwise
 *
 *   x is a byte image dimensioned row_dim x col_dim in the calling
 *   routine.  y is a byte image allocated on the heap of dimension
 *   max(row_dim,col_dim) x max(row_dim,col_dim).  The return value
 *   is max(row_dim,col_dim)
 *
 *  jph 17 June 1992
 */

#ifndef SQUAREPAD_C
#define SQUAREPAD_C
#include <stdio.h>
#include "teager2D.h"

int squarepad(x,y,row_dim,col_dim)

  unsigned char* x;         /* input image */
  unsigned char** y;        /* pointer to pointer to output image */
  int row_dim;    /* row dimension of x */
  int col_dim;    /* col dimension of x */
{

  int i,j;           /* loop counters */
  int size;          /* max(row_dim,col_dim) */

 /*
  * detect zero dimension input
  */
  if ((row_dim==0) || (col_dim==0)) return 0;

 /* 
  * allocate memory for y, return error if out of memory
  */
  size = (row_dim > col_dim) ? row_dim : col_dim;
  *y = (unsigned char *) malloc(size * size * sizeof(unsigned char));
  if (*y == NULL) {
    printf("\nsquarepad.c : out of memory error !");
    return 0;
  }

 /*
  * make y equal to zero
  */
  for (i=0; i<size*size; i++) (*y)[i] = 0;

 /*
  * make y equal to x wherever possible
  */
  for (i=0; i<row_dim; i++) 
    for (j=0; j<col_dim; j++) 
      (*y)[i*size+j] = x[i*col_dim+j];

  return size;
} 
#endif
