/*
 * stat.c
 *
 *   function computes simple statistics of the pixel values of an
 *   unsigned char (byte) image.
 *
 *
 *  jph 15 June 1992
 */

#ifndef SSTAT_C
#define SSTAT_C
#include <stdio.h>
#include <math.h>
#include "teager2D.h"

void sstat(x,row_dim,col_dim,min,median,max,mean,sigma)

  unsigned char* x;     /* input image */
  int row_dim;    /* row dimension of x */
  int col_dim;    /* col dimension of x */
  unsigned char *min;   /* computed minimum mean pixel value in x */
  unsigned char *median;   /* computed median pixel value in x */
  unsigned char *max;   /* computed maximum pixel value in x */
  double * mean;        /* computed mean pixel value in x */
  double * sigma;       /* computed standard deviation of x */
{

  int i;             /* loop counters */
  double sum;        /* sum of pixel values in x */
  double sumsq;      /* sum of squared pixel values in x */
  int N;             /* number of pixels in x */
  int No2;           /* N over 2 */
  int hist[256];     /* histogram of x */
  int ltmdn;         /* number of pix < current guess for median */

 /*
  * detect zero dimension input
  */
  if ((row_dim==0) || (col_dim==0)) return;

 /*
  *  Find min, max, mean, and sigma
  */
  No2 = (N = row_dim * col_dim) / 2;
  sum = sumsq = 0.0;

  *min = *max = x[0];
  for (i=0; i<N; i++) {
    sum += (double) x[i];
    sumsq += ((double)x[i] * (double)x[i]);
    if (x[i] > (*max)) *max = x[i];
    else if (x[i] < (*min)) *min = x[i];
  }
  *mean = sum / (double) N;
  *sigma = sqrt(sumsq / (double) N - pow(*mean,2.0));

 /*
  * find median by histogram method
  */
  for (i=0; i<256; i++) hist[i] = 0;
  for (i=0; i<N; i++) hist[(int)x[i]]++;
  for(i=-1,ltmdn=0; ltmdn<No2; ) {
    i++;
    ltmdn += hist[i];
  }
  *median = (unsigned char) i;

  return;
} 
#endif
