/*
 * Include files
 */

#ifndef TEAGER2D_H
#define TEAGER2D_H

/*#include <malloc.h>*/
#include <math.h>
#include <stdio.h>
#include <stdlib.h>
#include <fcntl.h>
#include <string.h>

/*
 * miscellaneous symbols
 */
#define BYTE unsigned char
#define UINT unsigned int
#define FNSIZE 160

/*
 * FloatImage: type for floating point images.
 */
struct FloatImage {

  int    size;       /* row/col dimension if square */
  int    square;     /* 1 if image is square, 0 otherwise */
  int    rows;       /* number of rows if not square */
  int    cols;       /* number of columns if not square */
  float *Real;       /* real part in image domain */
  float *Imag;       /* imag part in image domain */
  float *FFTReal;    /* real part in freq domain */
  float *FFTImag;    /* imag part in freq domain */
};
typedef struct FloatImage FloatImage;

/*
 * pFloatImage: type for pointer to floating point image
 */
typedef FloatImage *pFloatImage;

/*
 * More Include Files
 */
#include "gabor.h"

/*
 * Function prototypes for 2D teager work
 */
#ifndef FOURN_C
void fourn();
#endif
#ifndef ADDIMAGE_C
int *addimage();
#endif
#ifndef A_DSPLY_NEEDLE_C
void a_dsply_needle();
#endif
#ifndef ALLOCFLOATIMAGE_C
void AllocFloatImage();
#endif
#ifndef B_SCALEIMAGE_C
void b_scaleimage();
#endif
#ifndef BNORM_C
unsigned char bNorm();
#endif
#ifndef BYTE2DISK_C
void byte2disk();
#endif
#ifndef BYTE2MONITOR_C
void byte2monitor();
#endif
#ifndef BYTE2DOUBLE_C
double *byte2double();
#endif
#ifndef BYTE2FLOAT_C
float *byte2float();
#endif
#ifndef BYTEDISPLAY_C
void bytedisplay();
#endif
#ifndef C_BYTE2MONITOR_C
void c_byte2monitor();
#endif
#ifndef CHOP_C
void chop();
#endif
#ifndef D_DCOL_C
int *d_dCol();
#endif
#ifndef D_D_DCOL_C
double *d_d_dCol();
#endif
#ifndef D_D_DROW_C
double *d_d_dRow();
#endif
#ifndef D_DINTERP_C
void d_d_interp();
#endif
#ifndef D_FFT2D_C
void d_fft2d();
#endif
#ifndef D_FOURN_C
void d_fourn();
#endif
#ifndef D_INTERP_C
void d_interp();
#endif
#ifndef D_DROW_C
int *d_dRow();
#endif
#ifndef D_DBX_C
unsigned char *d_dbx();
#endif
#ifndef D_MED_C
double d_med();
#endif
#ifndef D_SCALEIMAGE_C
double *d_scaleimage();
#endif
#ifndef D_SHIFTDOWNROWS_C
double *d_ShiftDownRows();
#endif
#ifndef D_SHIFTIMAGE_C
double *d_shiftimage();
#endif
#ifndef D_SHIFTLEFTCOLS_C
double *d_ShiftLeftCols();
#endif
#ifndef D_SHIFTRIGHTCOLS_C
double *d_ShiftRightCols();
#endif
#ifndef D_SHIFTUPROWS_C
double *d_ShiftUpRows();
#endif
#ifndef D_SUBTRACTIMAGE_C
double *d_subtractimage();
#endif
#ifndef D2DISK_C
void d2disk();
#endif
#ifndef D2F_C
float* d2f();
#endif
#ifndef DBX_C
void dbx();
#endif
#ifndef DIFFARRAY_C
void DIFFARRAY();
#endif
#ifndef DISK2BYTE_C
void disk2byte();
#endif
#ifndef DISK2D_C
void disk2d();
#endif
#ifndef DISK2F_C
void disk2f();
#endif
#ifndef DISK2IMAGE_C
void *disk2image();
#endif
#ifndef DNORM_C
double dNorm();
#endif
#ifndef DOUBLE2BYTE_C
unsigned char *double2byte();
#endif
#ifndef DSPLY_NEEDLE_C
void dsply_needle();
#endif
#ifndef DSSTAT_C
void dsstat();
#endif
#ifndef F_DBX_C
unsigned char* f_dbx();
#endif
#ifndef F_U_NORM
void f_u_norm();
#endif
#ifndef F2B_C
unsigned char* f2b();
#endif
#ifndef F2D_C
double* f2d();
#endif
#ifndef F2DISK_C
void f2disk();
#endif
#ifndef FCMULT_C
void fCMult();
#endif
#ifndef FCOVAR_C
float fCovar();
#endif
#ifndef FFFTCONVOLV_C
void fFFTConvolve();
#endif
#ifndef FFINITEDIFP_C
void fFiniteDifP();
#endif
#ifndef FFT2D_C
void fft2d();
#endif
#ifndef F_INTERP_C
void f_interp();
#endif
#ifndef FILTER_C
void Filter();
#endif
#ifndef REVFILTER_C
void REVFilter();
#endif
#ifndef FLOATALLOC_C
float *floatalloc();
#endif
#ifndef FNORM_C
float fNorm();
#endif
#ifndef FMAKEPOS_C
float fmakepos();
#endif
#ifndef FMEAN_C
float fmean();
#endif
#ifndef FMED3X3_C
float *fmed3x3();
#endif
#ifndef FMEDNXN_C
float *fmedNxN();
#endif
#ifndef FSIGCMPRS_C
float *fSigCmprs();
#endif
#ifndef F_SCALEIMAGE_C
float* f_scaleimage();
#endif
#ifndef FSSTAT_C
void fsstat();
#endif
#ifndef FVARIANCE_C
float fvariance();
#endif
#ifndef HEAPCHECK_C
void heapcheck();
#endif
#ifndef HISTO_C
void histo();
#endif
#ifndef IMAGE2DISK_C
void image2disk();
#endif
#ifndef INT2DOUBLE_C
double *int2double();
#endif
#ifndef INT_2D_1COMPONENT_ESA_C
void int_2D_1component_ESA();
#endif
#ifndef KILLFLOAT_C
pFloatImage KillFloat();
#endif
#ifndef LOGIMAGE_C
double *logimage();
#endif
#ifndef MAKEHARDY_C
void MakeHardy();
#endif
#ifndef MAKEHARDY2_C
void MakeHardy2();
#endif
#ifndef MAKEPOS_C
double makepos();
#endif
#ifndef MULTIMAGE_C
int *multimage();
#endif
#ifndef PA_C
void PA();
#endif
#ifndef PC_C
void PC();
#endif
#ifndef PRINTA_C
void PRINTA();
#endif
#ifndef PRINTC_C
void PRINTC();
#endif
#ifndef P_DSPLY_NEEDLE_C
void p_dsply_needle();
#endif
#ifndef PRINT_HISTO_C
void print_histo();
#endif
#ifndef PRINT_NEEDLE_C
void print_needle();
#endif
#ifndef PRINTGNU_NEEDLE_C
void printgnu_needle();
#endif
#ifndef PTESA2DNOFILT_C
void PtEsa2DNoFilt();
#endif
#ifndef READASF_C
float *readasf();
#endif
#ifndef RECOMPUTEASM_C
void ReComputeAsm();
#endif
#ifndef SCALEIMAGE_C
int *scaleimage();
#endif
#ifndef SHEAPCHECK_C
void Sheapcheck();
#endif
#ifndef SHIFTDOWNROWS_C
int *ShiftDownRows();
#endif
#ifndef SHIFTIMAGE_C
int *shiftimage();
#endif
#ifndef SHIFTLEFTCOLS_C
int *ShiftLeftCols();
#endif
#ifndef SHIFTRIGHTCOLS_C
int *ShiftRightCols();
#endif
#ifndef SHIFTUPROWS_C
int *ShiftUpRows();
#endif
#ifndef SQUAREPAD_C
int squarepad();
#endif
#ifndef SQIMAGE_C
int *sqimage();
#endif
#ifndef SPECMAG2DISK_C
void SpecMag2disk();
#endif
#ifndef SSTAT_C
void sstat();
#endif
#ifndef SUBTRACTIMAGE_C
int *subtractimage();
#endif
#ifndef D_TEAGER2D_C
double *d_teager2D();
#endif
#ifndef TEAGER2D_C
int *teager2D();
#endif
#ifndef UNIT_NORMALIZE_C
double unit_normalize();
#endif
#ifndef V_BYTEDISPLAY_C
void v_bytedisplay();
#endif
#ifndef V_DSPLY_NEEDLE_C
void v_dsply_needle();
#endif
#ifndef V_HISTO_C
void v_histo();
#endif
#ifndef WRITEA_C
void WRITEA();
#endif
#ifndef WRITEC_C
void WRITEC();
#endif
#ifndef ZEROU0_C
void ZeroU0();
#endif
#ifndef ZEROV0_C
void ZeroV0();
#endif
#endif
