/*
 * unit_normalize.c
 *
 *   the input image x is declared double x[row_dim][col_dim] in the calling
 *   routine.  It is assumed that all values in x are strictly positive...
 *   this might have been accomplished using "makepos.c", for example.
 *
 *   The action of the function is to make the largest value in the image
 *   one, i.e.
 *
 *      y[i][j] = x[i][j] / maxval,
 *
 *   where maxval is the largest value in x.  
 *
 *   maxval is returned, y is allocated on the heap.
 *
 *  jph 15 June 1992
 */

#ifndef UNIT_NORMALIZE_C
#define UNIT_NORMALIZE_C
#include <stdio.h>
#include "teager2D.h"

double unit_normalize(x,y,row_dim,col_dim)

  double* x;         /* input image */
  double** y;         /* pointer to pointer to output image */
  int row_dim;    /* row dimension of x */
  int col_dim;    /* col dimension of x */
{

  int i;             /* loop counters */
  double maxval;     /* largest value in input image */
  double factor;     /* reciprocal of maxval */

 /*
  * detect zero dimension input
  */
  if ((row_dim==0) || (col_dim==0)) return 0.0;

 /*
  * find the maximum pixel value in the input and the factor
  */
  maxval = x[0];
  for (i=1; i<row_dim*col_dim; i++)
    if (x[i] > maxval) maxval = x[i];

  if (maxval == (double) 0.0) {
    printf("\nunit_normalize.c : max val is zero !\n");
    return 0.0;
  }

  factor = (double) 1.0 / maxval;
  
 /*
  * make y equal to a unit scaled version of x
  */
  *y = d_scaleimage(x,row_dim,col_dim,factor);

  return factor;
} 
#endif
