/*
 * v_histo.c
 *
 *   This modified version of histo.c displays on vision instead of sausage.
 *   Function puts up a histogram of the byte image x on the Xterm.
 *   Plotting is done by calling GnuPlot.
 *
 *  jph 16 June 1992
 */

#ifndef V_HISTO_C
#define V_HISTO_C
#include <stdio.h>
#include "teager2D.h"

void v_histo(x,row_dim,col_dim)

  unsigned char* x;     /* input image */
  int row_dim;    /* row dimension of x */
  int col_dim;    /* col dimension of x */
{

  int histogram[64];  /* histogram of x */
  int i;              /* loop counter */
  int maxbin;         /* max count to display in a bin */
  FILE *tmpfile;      /* temporary file */
  unsigned char min;  /* min pix value in x */
  unsigned char med;  /* median pix value in x */
  unsigned char max;  /* max pix value in x */
  double mean;        /* mean pixel value in x */
  double sigma;       /* standard deviation of pixel values in x */
  char title[100];    /* histogram title, gives statistics */

 /*
  * detect zero dimension input
  */
  if ((row_dim==0) || (col_dim==0)) return;

 /*
  * compute the histogram and write it to a file
  */
  for (i=0; i<64; i++) histogram[i] = 0;
  for (i=0; i<row_dim*col_dim; i++) histogram[(int)x[i] / 4]++;
  maxbin = (row_dim * col_dim) / 4;
  for (i=0; i<64; i++) if (histogram[i] > maxbin) histogram[i] = maxbin;
  
  tmpfile = fopen("___histo_tmp_data_file","w");;
  for (i=0; i<64; i++)
    fprintf(tmpfile,"\n%d\t%d",4*i,histogram[i]);
  fclose(tmpfile);

 /*
  * create the gnuplot command file
  */
  sstat(x,row_dim,col_dim,&min,&med,&max,&mean,&sigma);
  sprintf(title,"\"MIN=%d  MEDIAN=%d   MEAN=%lf   MAX=%d   SIGMA=%lf\"",
             min,med,mean,max,sigma);
  tmpfile = fopen("___histo_tmp_gnu_file","w");
  fprintf(tmpfile,"set terminal x11");
  fprintf(tmpfile,"\nset title %s",title);
  fprintf(tmpfile,"\nplot '___histo_tmp_data_file' with impulses");
  fprintf(tmpfile,"\npause -1 \"hit any key...\"\n");
  fclose(tmpfile);

 /*
  * Do the plot
  */
  system("(export DISPLAY;DISPLAY=vision:0.0;gnuplot ___histo_tmp_gnu_file 2>/dev/null)");

 /*
  * Clean house
  */
  system("rm ___histo_tmp_data_file");
  system("rm ___histo_tmp_gnu_file");

  return;
} 
#endif
