/*
 * Epib.c
 *
 *  Epilogue function for trackb.c, which tracks as track.c, and also outputs
 *  data for performing reconstruction.
 *
 */

#ifndef EPIB_C
#define EPIB_C

#include "track.h"

/*-----------------------------------------------------------------------------
   EPILOGUE
-----------------------------------------------------------------------------*/

void Epilogue(cmd,OrigSize,ZP,G,T,BCinterval,minRow2Track,MaxRow2Track,
              minCol2Track,MaxCol2Track,X,TARootFn,TURootFn,TVRootFn,BCRootFn,
              RptFn)

  char        *cmd;                  /* argv[0] */
  int          OrigSize;             /* original size of image */
  int          ZP;                   /* 1 if zero padding */
  filterbank  *G;                    /* The filterbank */
  track_t     *T;                    /* tracks */
  int          BCinterval;           /* spacing between boundary conditions */
  int          minRow2Track;         /* minimum row where tracking performed */
  int          MaxRow2Track;         /* maximum row where tracking performed */
  int          minCol2Track;         /* minimum col where tracking performed */
  int          MaxCol2Track;         /* maximum col where tracking performed */
  pFloatImage  X;                    /* input signal */
  char        *TARootFn;             /* root fn for tracked amplitudes */
  char        *TURootFn;             /* root fn for tracked horiz frequencies */
  char        *TVRootFn;             /* root fn for tracked vert  frequencies */
  char        *BCRootFn;             /* root fn for track boundary conditions */
  char        *RptFn;                /* ascii track report filename */
{

  pFloatImage   Y;              /* filtered signal */
  int           i;              /* counter */
  int           j;              /* counter */
  int           k;              /* counter */
  int           l;              /* counter */
  int           indx;           /* BC index counter */
  int           row;            /* counter */
  int           col;            /* counter */
  int           DatSize;        /* size of dat array */
  int          *ObsCount;       /* count of obs taken from each channel */
  int           Ntracks;        /* counters for Num tracks */
  int           Nactive;        /* counters for Num tracks */
  int           Ndormant;       /* counter for Num dormant tracks */
  int           delta;          /* tracking direction for row, -1 = RtoL */
  int           OddRowDelta;    /* sign of delta on odd rows */
  int           EvenRowDelta;   /* sign of delta on even rows */
  int           TrackPts;       /* number of pixels in a track */
  int           TrkNeedsChan;   /* 1 if CurTrk needs BC from CurChan */
  int           NBC;            /* Max Num Boundary Conditions for a track */
  int           CurTrk;         /* counter for loop on tracks */
  int           CurChan;        /* counter for loop on channels */
  int          *DomPts;         /* List of domain coordinates */
  int          *IndexPts;       /* List of BC coordinates */
  float        *dat;            /* array of tracked values for A,U, or V */
  float         min;            /* min of dat */
  float         med;            /* median of dat */
  float         max;            /* max of dat */
  float         mean;           /* mean of dat */
  float         sigma;          /* std dev of dat */
  double        theta;          /* inst phase of a component */
  char          fn[FNSIZE];     /* filename to write to */
  FILE         *fp;             /* ascii file pointer for histogram output */
  pBCctrlRecord BCctrl;         /* ptr to boundary cond ctrl structure */
  pBoundCon     BC;             /* boundary conditions */
  BYTE         *BCOut;          /* boundary condition output structure */
  int           BCOutSize;      /* Num Bytes in a BCOut structure */
  float        *BCptr;          /* Ptr for writing into BCOut arrays */
  int           BCperRow;       /* num BC's on an interior row of the track */
  int           BCrows;         /* num interior rows in track, not 1st, last */

 /*
  * Find out what delta was on odd and even rows 
  */
  if (minRow2Track % 2) {  
    OddRowDelta = 1;
  }
  else {
    OddRowDelta = -1;
  }
  EvenRowDelta = -1 * OddRowDelta;

 /*
  * Report
  */
  for (i=Ntracks=Nactive=Ndormant=0; i < MAXNTRACK; i++) {
    if (T[i].in_use) {
      Ntracks++;
      if (T[i].active) {
        Nactive++;
      } else {
        if (T[i].dormant) Ndormant++;
      }
    }
  }
  printf("\nTracking FINISHED: %d tracks, %d active, %d dormant.",
         Ntracks,Nactive,Ndormant);
  fp = fopen(RptFn,"w");

  dat = (float *) malloc(OrigSize * OrigSize * sizeof(float));
  if ((ObsCount = (int *)malloc(G->TotNumFilters * sizeof(int))) == NULL) {
    printf("\nEpilogue: free store exhausted.\n");
    exit(-1);
  }

 /*
  * Set stopcol and stoprow for active tracks
  */
  for (i=0; i < MAXNTRACK; i++) {
    if (T[i].in_use) {
      if (T[i].active) {
        T[i].Hist.Stop.row = MaxRow2Track;
        if (T[i].Hist.Stop.row % 2) {
          delta = OddRowDelta;
        }
        else {
          delta = EvenRowDelta;
        }
        if (delta < 0) {
          T[i].Hist.Stop.col = minCol2Track;
        }
        else {
          T[i].Hist.Stop.col = MaxCol2Track;
        }
      }  /* if active */
    }
  }

 /*
  * Allocate and Initialize boundary condition control structure
  */
  BCctrl = (pBCctrlRecord) malloc(G->TotNumFilters * sizeof(BCctrlRecord));
  for (i=0; i < G->TotNumFilters; i++) {
    BCctrl[i].NtrksNeed = 0;
    BCctrl[i].DPL = (pDomainPtList)malloc(MAXNTRACK * sizeof(DomainPtList));
    for (j=0; j < MAXNTRACK; j++) {
      BCctrl[i].DPL[j].NDomainPts = 0;
      BCctrl[i].DPL[j].domain = NULL;
      BCctrl[i].DPL[j].Bidx = NULL;
    }
  }

 /*
  * Figure where each track needs Boundary Conditions from each channel
  */
  printf("\nCompiling Boundary Control...");
  NBC = (OrigSize / BCinterval + 1) * ((OrigSize-2)/BCinterval + 2);
  DomPts = (int *)malloc(NBC * sizeof(int));
  IndexPts = (int *)malloc(NBC * sizeof(int));

  for (CurChan=0; CurChan < G->TotNumFilters; CurChan++) {
    for (CurTrk=0; CurTrk < MAXNTRACK; CurTrk++) {
      if (!T[CurTrk].active) continue;
     /*
      * Make a list of points where CurTrk needs BC from CurChan
      */
      TrkNeedsChan = 0;
      indx = 0;
    
     /*
      * First row
      */
      if (!(T[CurTrk].Hist.Start.row % 2)) {
        delta = EvenRowDelta;
      } else {
        delta = OddRowDelta;
      }
      row=T[CurTrk].Hist.Start.row;
      for (col=((delta==1) ? T[CurTrk].Hist.Start.col: minCol2Track);
           col<((delta==1) ? MaxCol2Track          : T[CurTrk].Hist.Start.col);
           col+=BCinterval,indx++) {
        l = row*OrigSize + col;
        if (T[CurTrk].ObsChan[l] == CurChan) {
          if (!TrkNeedsChan) {
            TrkNeedsChan = 1;
            BCctrl[CurChan].NtrksNeed++;
          }
          IndexPts[BCctrl[CurChan].DPL[CurTrk].NDomainPts] = indx;
          DomPts[BCctrl[CurChan].DPL[CurTrk].NDomainPts++] = l;
        }
      }
     /*
      * Take care of final boundary condition, which may be at an odd interval
      */
      l = row * OrigSize + ((delta==1) ? MaxCol2Track:T[CurTrk].Hist.Start.col);
      if (T[CurTrk].ObsChan[l] == CurChan) {
        if (!TrkNeedsChan) {
           TrkNeedsChan = 1;
           BCctrl[CurChan].NtrksNeed++;
         }
         IndexPts[BCctrl[CurChan].DPL[CurTrk].NDomainPts] = indx;
         DomPts[BCctrl[CurChan].DPL[CurTrk].NDomainPts++] = l;
      }
      indx++;

     /*
      * Do the Last row
      */
      if (!(T[CurTrk].Hist.Stop.row % 2)) {
        delta = EvenRowDelta;
      } else {
        delta = OddRowDelta;
      }
      row=T[CurTrk].Hist.Stop.row;
      for (col=((delta==1) ? minCol2Track            : T[CurTrk].Hist.Stop.col);
           col<((delta==1) ? T[CurTrk].Hist.Stop.col : MaxCol2Track); 
           col+=BCinterval,indx++) {
        l = row*OrigSize + col;
        if (T[CurTrk].ObsChan[l] == CurChan) {
          if (!TrkNeedsChan) {
            TrkNeedsChan = 1;
            BCctrl[CurChan].NtrksNeed++;
          }
          IndexPts[BCctrl[CurChan].DPL[CurTrk].NDomainPts] = indx;
          DomPts[BCctrl[CurChan].DPL[CurTrk].NDomainPts++] = l;
        }
      }
     /*
      * Take care of final boundary condition, which may be at an odd interval
      */
      l = row * OrigSize + ((delta==1) ? T[CurTrk].Hist.Stop.col:MaxCol2Track);
      if (T[CurTrk].ObsChan[l] == CurChan) {
        if (!TrkNeedsChan) {
           TrkNeedsChan = 1;
           BCctrl[CurChan].NtrksNeed++;
         }
         IndexPts[BCctrl[CurChan].DPL[CurTrk].NDomainPts] = indx;
         DomPts[BCctrl[CurChan].DPL[CurTrk].NDomainPts++] = l;
      }
      indx++;

     /*
      * Do all the rows in between 
      */
      for (row=T[CurTrk].Hist.Start.row+1;row<T[CurTrk].Hist.Stop.row; 
           row+=BCinterval,indx++) {
        for (col=minCol2Track; col < MaxCol2Track; col+=BCinterval,indx++) {
          l = row*OrigSize + col;
          if (T[CurTrk].ObsChan[l] == CurChan) {
            if (!TrkNeedsChan) {
              TrkNeedsChan = 1;
              BCctrl[CurChan].NtrksNeed++;
            }
            IndexPts[BCctrl[CurChan].DPL[CurTrk].NDomainPts] = indx;
            DomPts[BCctrl[CurChan].DPL[CurTrk].NDomainPts++] = l;
          }
        }
       /*
        * Take care of final boundary condition, which may be at an odd interval
        */
        l = row * OrigSize + MaxCol2Track;
        if (T[CurTrk].ObsChan[l] == CurChan) {
          if (!TrkNeedsChan) {
             TrkNeedsChan = 1;
             BCctrl[CurChan].NtrksNeed++;
           }
           IndexPts[BCctrl[CurChan].DPL[CurTrk].NDomainPts] = indx;
           DomPts[BCctrl[CurChan].DPL[CurTrk].NDomainPts++] = l;
        }
      }

     /*
      * Allocate space for bnd conds and copy from DomPts to new space
      */
      if (BCctrl[CurChan].DPL[CurTrk].NDomainPts) {
        BCctrl[CurChan].DPL[CurTrk].domain =
            (int *)malloc(BCctrl[CurChan].DPL[CurTrk].NDomainPts*sizeof(int));
        memcpy(BCctrl[CurChan].DPL[CurTrk].domain,DomPts,
               BCctrl[CurChan].DPL[CurTrk].NDomainPts*sizeof(int));
        BCctrl[CurChan].DPL[CurTrk].Bidx =
            (int *)malloc(BCctrl[CurChan].DPL[CurTrk].NDomainPts*sizeof(int));
        memcpy(BCctrl[CurChan].DPL[CurTrk].Bidx,IndexPts,
               BCctrl[CurChan].DPL[CurTrk].NDomainPts*sizeof(int));
      }
    }    /* for CurTrk */
  }      /* for CurChan */
  free(DomPts);
  free(IndexPts);

 /*
  * Allocate and Initialize the boundary conditions
  */
  printf("\nAllocating BC structures...");
  BC = (pBoundCon)malloc(MAXNTRACK * sizeof(BoundCon));
  for (i=0; i < MAXNTRACK; i++) {
    BC[i].firstrow =        BC[i].fr_startcol =      BC[i].fr_stopcol =
    BC[i].lastrow  =        BC[i].lr_startcol =      BC[i].lr_stopcol =
    BC[i].minCol2Track =    BC[i].MaxCol2Track =     -1;
    BC[i].BCinterval = BC[i].Nfirst = BC[i].Nlast = BC[i].Npts = 0;
    BC[i].Re        =   BC[i].ReFirst    =    BC[i].ReLast =
    BC[i].ReRest    =   BC[i].Im         =    BC[i].ImFirst =
    BC[i].ImLast    =   BC[i].ImRest     =    NULL;
    if (!T[i].active) continue;

    BC[i].firstrow = T[i].Hist.Start.row;
    if (T[i].Hist.Start.row % 2) {
      delta = OddRowDelta;
    }
    else {
      delta = EvenRowDelta;
    }
    BC[i].fr_startcol = ((delta==1) ? T[i].Hist.Start.col : minCol2Track);
    BC[i].fr_stopcol  = ((delta==1) ? MaxCol2Track : T[i].Hist.Start.col);
    BC[i].lastrow = T[i].Hist.Stop.row;
    if (T[i].Hist.Stop.row % 2) {
      delta = OddRowDelta;
    }
    else {
      delta = EvenRowDelta;
    }
    BC[i].lr_startcol = ((delta==1) ? minCol2Track : T[i].Hist.Stop.col);
    BC[i].lr_stopcol  = ((delta==1) ? T[i].Hist.Stop.col : MaxCol2Track);
    BC[i].minCol2Track = minCol2Track;
    BC[i].MaxCol2Track = MaxCol2Track;
    BC[i].BCinterval = BCinterval;
    BC[i].Nfirst = (BC[i].fr_stopcol-BC[i].fr_startcol+1)/BC[i].BCinterval + 1;
    BC[i].Nlast  = (BC[i].lr_stopcol-BC[i].lr_startcol+1)/BC[i].BCinterval + 1;
    BCperRow = (BC[i].MaxCol2Track-BC[i].minCol2Track+1)/BC[i].BCinterval;
    if ((BC[i].MaxCol2Track-BC[i].minCol2Track+1) % BC[i].BCinterval) {
      BCperRow++;  /* if partial interval at end of row */      
    }
    BCperRow++;   /* last BC on row */
    BCrows = (BC[i].lastrow - BC[i].firstrow - 1)/BC[i].BCinterval;
    if ((BC[i].lastrow - BC[i].firstrow - 1) % BC[i].BCinterval) {
      BCrows++;
    }
    BC[i].Npts = BCperRow * BCrows + BC[i].Nfirst + BC[i].Nlast;
    BC[i].Re = (float *)malloc(BC[i].Npts * sizeof(float));
    BC[i].ReFirst = BC[i].Re;
    BC[i].ReLast  = BC[i].ReFirst + BC[i].Nfirst;
    BC[i].ReRest  = BC[i].ReLast  + BC[i].Nlast;

    BC[i].Im = (float *)malloc(BC[i].Npts * sizeof(float));
    BC[i].ImFirst = BC[i].Im;
    BC[i].ImLast  = BC[i].ImFirst + BC[i].Nfirst;
    BC[i].ImRest  = BC[i].ImLast  + BC[i].Nlast;
  }

 /*
  * Allocate response image Y
  */
  Y = (pFloatImage)malloc(sizeof(FloatImage));
  AllocFloatImage(Y,X->size);
  Y->cols = X->cols; 
  Y->rows = X->rows;
  Y->square = X->square;

 /*
  * Loop through the channels, re-compute the response image, extract BCs 
  */
  for (CurChan=0; CurChan < G->TotNumFilters; CurChan++) {
    printf("\nExtracting Boundary Conditions from Channel %02d...",CurChan);
    if (!BCctrl[CurChan].NtrksNeed) continue;
    HardyConvolveGabor(Y,X,G->filt[CurChan]);
    for (CurTrk=0; CurTrk < MAXNTRACK; CurTrk++) {
      for (i=0; i < BCctrl[CurChan].DPL[CurTrk].NDomainPts; i++) {
        row = BCctrl[CurChan].DPL[CurTrk].domain[i] / OrigSize;
        col = BCctrl[CurChan].DPL[CurTrk].domain[i] % OrigSize;
        k = (ZP ? row*X->size + col : row*OrigSize + col);
        theta = atan2(Y->Imag[k],Y->Real[k]);
        BC[CurTrk].Re[BCctrl[CurChan].DPL[CurTrk].Bidx[i]] = 
                T[CurTrk].A_filt[BCctrl[CurChan].DPL[CurTrk].domain[i]] *
                (float)cos(theta);
        BC[CurTrk].Im[BCctrl[CurChan].DPL[CurTrk].Bidx[i]] = 
                T[CurTrk].A_filt[BCctrl[CurChan].DPL[CurTrk].domain[i]] *
                (float)sin(theta);
      } /* for i */
    }   /* for CurTrk */
  }     /* for CurChan */

 /*
  * Write boundary conditions for each component to a file
  */
  for (CurTrk=0; CurTrk < MAXNTRACK; CurTrk++) {
    if (!T[CurTrk].active) continue;
    BCOutSize =     12 * sizeof(int) +                     /* int header info */
                     2 * BC[CurTrk].Npts * sizeof(float);  /* Cmplx BC data */
    BCOut = (BYTE *)malloc(BCOutSize);
    memcpy(BCOut, &BC[CurTrk].firstrow, 12 * sizeof(int));  /* copy hdr info */

    for (i=0, BCptr=(float *)&BCOut[12*sizeof(int)]; 
         i < BC[CurTrk].Npts; i++,BCptr+=2) {
      *BCptr       = BC[CurTrk].Re[i];
      *(BCptr + 1) = BC[CurTrk].Im[i];
    }
    sprintf(fn,"%s%02d.bin",BCRootFn,CurTrk);
    byte2disk(BCOut, 1, BCOutSize, fn);
    free(BCOut);
  }

 /*
  * Loop on tracks
  */
  for (i=0; i < MAXNTRACK; i++) {
    if (T[i].in_use) {
     /*
      * output the tracked estimates to binary files
      */
      sprintf(fn,"%s%02d.float",TARootFn,i);
      f2disk(T[i].A_filt, OrigSize, OrigSize, fn);
      sprintf(fn,"%s%02d.float",TURootFn,i);
      f2disk(T[i].U_filt, OrigSize, OrigSize, fn);
      sprintf(fn,"%s%02d.float",TVRootFn,i);
      f2disk(T[i].V_filt, OrigSize, OrigSize, fn);

     /*
      * Output tracked estimates in format for reconstruction
      */
      /* first row */ 
      if (!(T[i].Hist.Start.row % 2)) {  /* trk started on even row */
        if (EvenRowDelta == 1) {
          TrackPts = MaxCol2Track - T[i].Hist.Start.col + 1;
        } else {
          TrackPts = T[i].Hist.Start.col - minCol2Track + 1;
        }
      } else {  /* trk started on an odd row */
        if (OddRowDelta == 1) {
          TrackPts = MaxCol2Track - T[i].Hist.Start.col + 1;
        } else {
          TrackPts = T[i].Hist.Start.col - minCol2Track + 1;
        }
      }
     /*
      * Last Row
      */
      if (!(T[i].Hist.Stop.row % 2)) {  /* trk stopped on even row */
        if (EvenRowDelta == 1) {
          TrackPts += T[i].Hist.Stop.col - minCol2Track + 1;
        } else {
          TrackPts += MaxCol2Track - T[i].Hist.Stop.col + 1;
        }
      } else {  /* trk stopped on an odd row */
        if (OddRowDelta == 1) {
          TrackPts += T[i].Hist.Stop.col - minCol2Track + 1;
        } else {
          TrackPts += MaxCol2Track - T[i].Hist.Stop.col + 1;
        }
      } 
      TrackPts += (MaxCol2Track - minCol2Track + 1) * 
                       (T[i].Hist.Stop.row - T[i].Hist.Start.row - 1);
        
     /*
      * Histogram the channel's from which observations where taken
      */
      for (j=0; j < G->TotNumFilters; j++) {
        ObsCount[j] = 0;
      }
      for (j=0; j < OrigSize * OrigSize; j++) {
        if (T[i].ObsChan[j] >= 0) {
          ObsCount[(int)T[i].ObsChan[j]]++;
        }
      }
     /*
      * Report track data
      */
      fprintf(fp,"\nTRACK %02d\n",i);
      fprintf(fp,"\nStarted row:\t%d\tcol:\t%d\tchannel:\t%d\n",
               T[i].Hist.Start.row,
               T[i].Hist.Start.col,
               (int)T[i].ObsChan[T[i].Hist.Start.row
                                 *OrigSize+T[i].Hist.Start.col]);
      fprintf(fp,"Ended   row:\t%d\tcol:\t%d\n",
                    T[i].Hist.Stop.row,T[i].Hist.Stop.col);
      fprintf(fp,"\nMaxQual:\t%g\tAveQual:\t%g\n",
                                            T[i].MaxQual,T[i].AveChanQual);
      fprintf(fp,"\nSummed Innovation Norm:\t%g\n",T[i].SumInnov);
      fprintf(fp,"\nTotal Number of Coasted Observations:\t%d\n",
                                                  T[i].TotNcoast);
      fprintf(fp,"\nTotal Number of dormant Observations:\t%d\n",
                                                  T[i].Ndormant);
      fprintf(fp,"\nTotal Number of good Updates:\t%d\n",
                                                  T[i].NgoodUpd);
     /*
      * Report statistics of tracked parameters
      */
      if (T[i].Hist.Start.row % 2) {   /* track started on odd row */
        delta = OddRowDelta;
      } 
      else {                     /* track started on even row */
        delta = EvenRowDelta;
      }
      if (delta > 0) {           /* first row of track was L to R */
        DatSize = MaxCol2Track - T[i].Hist.Start.col + 1;
      }
      else {                     /* first row of track was R to L */
        DatSize = T[i].Hist.Start.col - minCol2Track + 1;
      }
      DatSize += ((MaxCol2Track - minCol2Track + 1) *  
                  (T[i].Hist.Stop.row - T[i].Hist.Start.row - 1));
      if (T[i].Hist.Stop.row % 2) {   /* track stopped on odd row */
        delta = OddRowDelta;
      } 
      else {                     /* track stopped on even row */
        delta = EvenRowDelta;
      }
      if (delta > 0) {           /* last row of track was L to R */
        DatSize += T[i].Hist.Stop.col - minCol2Track + 1;
      }
      else {                     /* last row of track was R to L */
        DatSize += MaxCol2Track - T[i].Hist.Stop.col + 1;
      }

     /*
      * A 
      */
      if (T[i].Hist.Start.row % 2) {   /* track started on odd row */
        delta = OddRowDelta;
      } 
      else {                     /* track started on even row */
        delta = EvenRowDelta;
      }
      if (delta > 0) {
        for (k=0,j=T[i].Hist.Start.col; j <= MaxCol2Track; j+=delta) {
          dat[k++] = T[i].A_filt[T[i].Hist.Start.row * OrigSize + j]; 
        }
      }
      else {
        for (k=0,j=T[i].Hist.Start.col; j >= minCol2Track; j+=delta) {
          dat[k++] = T[i].A_filt[T[i].Hist.Start.row * OrigSize + j]; 
        }
      }
      l = T[i].Hist.Start.row + 1;
      delta *= -1;
      while (l < T[i].Hist.Stop.row) {
        if (delta > 0) {
          for (j=minCol2Track; j <= MaxCol2Track; j+=delta) {
            dat[k++] = T[i].A_filt[l*OrigSize + j];
          }
        }
        else {
          for (j=MaxCol2Track; j >= minCol2Track; j+=delta) {
            dat[k++] = T[i].A_filt[l*OrigSize + j];
          }
        }  
        l++; 
        delta *= -1;
      }
      if (delta > 0) {
        for (j=minCol2Track; j <= T[i].Hist.Stop.col; j+=delta) {
          dat[k++] = T[i].A_filt[l*OrigSize + j];
        }
      }
      else {
        for (j=MaxCol2Track; j >= T[i].Hist.Stop.col; j+=delta) {
          dat[k++] = T[i].A_filt[l*OrigSize + j];
        }
      }
      fprintf(fp,"\nStatistics of A:\n");
      fsstat(dat,1,DatSize,&min,&med,&max,&mean,&sigma);
      fprintf(fp,"max=%f  med=%f  min=%f  mean=%f  stdev=%f  var=%f\n",
          max,med,min,mean,sigma,sigma*sigma);

     /*
      * U 
      */
      if (T[i].Hist.Start.row % 2) {   /* track started on odd row */
        delta = OddRowDelta;
      } 
      else {                     /* track started on even row */
        delta = EvenRowDelta;
      }
      if (delta > 0) {
        for (k=0,j=T[i].Hist.Start.col; j <= MaxCol2Track; j+=delta) {
          dat[k++] = T[i].U_filt[T[i].Hist.Start.row * OrigSize + j]; 
        }
      }
      else {
        for (k=0,j=T[i].Hist.Start.col; j >= minCol2Track; j+=delta) {
          dat[k++] = T[i].U_filt[T[i].Hist.Start.row * OrigSize + j]; 
        }
      }
      l = T[i].Hist.Start.row + 1;
      delta *= -1;
      while (l < T[i].Hist.Stop.row) {
        if (delta > 0) {
          for (j=minCol2Track; j <= MaxCol2Track; j+=delta) {
            dat[k++] = T[i].U_filt[l*OrigSize + j];
          }
        }
        else {
          for (j=MaxCol2Track; j >= minCol2Track; j+=delta) {
            dat[k++] = T[i].U_filt[l*OrigSize + j];
          }
        }  
        l++; 
        delta *= -1;
      }

      if (delta > 0) {
        for (j=minCol2Track; j <= T[i].Hist.Stop.col; j+=delta) {
          dat[k++] = T[i].U_filt[l*OrigSize + j];
        }
      }
      else {
        for (j=MaxCol2Track; j >= T[i].Hist.Stop.col; j+=delta) {
          dat[k++] = T[i].U_filt[l*OrigSize + j];
        }
      }
      fprintf(fp,"\nStatistics of U:\n");
      fsstat(dat,1,DatSize,&min,&med,&max,&mean,&sigma);
      fprintf(fp,"max=%f  med=%f  min=%f  mean=%f  stdev=%f  var=%f\n",
          max,med,min,mean,sigma,sigma*sigma);
     /*
      * V 
      */
      if (T[i].Hist.Start.row % 2) {   /* track started on odd row */
        delta = OddRowDelta;
      } 
      else {                     /* track started on even row */
        delta = EvenRowDelta;
      }
      if (delta > 0) {
        for (k=0,j=T[i].Hist.Start.col; j <= MaxCol2Track; j+=delta) {
          dat[k++] = T[i].V_filt[T[i].Hist.Start.row * OrigSize + j]; 
        }
      }
      else {
        for (k=0,j=T[i].Hist.Start.col; j >= minCol2Track; j+=delta) {
          dat[k++] = T[i].V_filt[T[i].Hist.Start.row * OrigSize + j]; 
        }
      }
      l = T[i].Hist.Start.row + 1;
      delta *= -1;
      while (l < T[i].Hist.Stop.row) {
        if (delta > 0) {
          for (j=minCol2Track; j <= MaxCol2Track; j+=delta) {
            dat[k++] = T[i].V_filt[l*OrigSize + j];
          }
        }
        else {
          for (j=MaxCol2Track; j >= minCol2Track; j+=delta) {
            dat[k++] = T[i].V_filt[l*OrigSize + j];
          }
        }  
        l++; 
        delta *= -1;
      }
      if (delta > 0) {
        for (j=minCol2Track; j <= T[i].Hist.Stop.col; j+=delta) {
          dat[k++] = T[i].V_filt[l*OrigSize + j];
        }
      }
      else {
        for (j=MaxCol2Track; j >= T[i].Hist.Stop.col; j+=delta) {
          dat[k++] = T[i].V_filt[l*OrigSize + j];
        }
      }
      fprintf(fp,"\nStatistics of V:\n");
      fsstat(dat,1,DatSize,&min,&med,&max,&mean,&sigma);
      fprintf(fp,"max=%f  med=%f  min=%f  mean=%f  stdev=%f  var=%f\n",
          max,med,min,mean,sigma,sigma*sigma);

     /*
      * Report Number of Observations taken from each channel for track
      */
      fprintf(fp, 
        "\nNumber of Observations taken from each channel for track %02d:\n",i);
      for (j=0; j < G->TotNumFilters; j++) {
        if (ObsCount[j] != 0) {
          fprintf(fp,"Channel %02d:  %6d\n",j,ObsCount[j]);
        }
      }
      fprintf(fp,
        "------------------------------------------------------------------\n");
    }        /* if T[i] in use */

  }          /* for i */

  return;

}/*--------------------------------------------  Epilogue  ------------------*/
#endif
