/*
 * InitABgain.c
 *
 *   Initialize a Kalman gain sequence for an Alpha-Beta filter.  A pointer to
 *   the gain sequence (type pABgain) is returned through the function name.
 *   The pABgain variable is allocated on the heap.  The two gain sequences, 
 *   each of type pKgain, are allocated.  The gains are read from files.  
 *   The first four bytes of the gain file are a float giving the number of 
 *   gains in the file.  The remainder of the file is the float gains. 
 *
 * 2/23/96 jph
 */

#ifndef INITABGAIN_C
#define INITABGAIN_C

#include "track.h"
 
pABgain InitABgain(AlphaFn,BetaFn)

  char        *AlphaFn;         /* Filename for the alpha gain sequence */ 
  char        *BetaFn;          /* Filename for the beta  gain sequence */ 
{
 
  pABgain      K;               /* Alpha-Beta filter gains */
  int          ifd;             /* input file descriptor */
  float        NumGains;        /* float number of gains in file */

 /*
  * Allocate ABgain variable
  */ 
  if ((K = (pABgain)malloc(sizeof(ABgain))) == NULL) {
    printf("\n\nInitABgain: Free Store Exhausted.\n");
    exit(-1);
  }

 /*
  * Allocate the Alpha and Beta gain variables
  */
  if ((K->alpha = (pKgain)malloc(sizeof(Kgain))) == NULL) {
    printf("\n\nInitABgain: Free Store Exhausted.\n");
    exit(-1);
  }
  if ((K->beta = (pKgain)malloc(sizeof(Kgain))) == NULL) {
    printf("\n\nInitABgain: Free Store Exhausted.\n");
    exit(-1);
  }

 /*
  * Open the Alpha gain file and read number of gains
  */
  if ((ifd = open(AlphaFn, O_RDONLY)) == -1) {
    printf("\nInitABgain: could not open file %s.\n",AlphaFn);
    exit(-1);
  }
  if (read(ifd,&NumGains,sizeof(float)) != sizeof(float)) {
    printf("\nInitABgain: could not read number of gains from %s.\n",AlphaFn);
    exit(-1);
  }
  K->alpha->Ngains = (int)NumGains;

 /*
  * Allocate Alpha gain array and read from file
  */
  if ((K->alpha->gain = (float *)malloc(K->alpha->Ngains * sizeof(float))) 
      == NULL) { 
    printf("\n\nInitABgain: Free Store Exhausted.\n");
    exit(-1);
  }
  if (read(ifd,K->alpha->gain,K->alpha->Ngains*sizeof(float)) 
            != K->alpha->Ngains*sizeof(float)) {
    printf("\nIinitABgain: Complete disk read of %s did not succeed %s.\n",
           AlphaFn);
    exit(-1);
  }

 /*
  * Close Alpha gain file
  */
  if (close(ifd) == -1) {
    printf("\nInitABgain: Error closing %s.\n",AlphaFn);
    exit(-1);
  }
 
 /*
  * Open the Beta gain file and read number of gains
  */
  if ((ifd = open(BetaFn, O_RDONLY)) == -1) {
    printf("\nInitABgain: could not open file %s.\n",BetaFn);
    exit(-1);
  }
  if (read(ifd,&NumGains,sizeof(float)) != sizeof(float)) { 
    printf("\nInitABgain: could not read number of gains from %s.\n",BetaFn);
    exit(-1);
  }
  K->beta->Ngains = (int)NumGains;
 
 /*
  * Allocate Beta gain array and read from file
  */
  if ((K->beta->gain = (float *)malloc(K->beta->Ngains * sizeof(float))) 
      == NULL) {
    printf("\n\nInitABgain: Free Store Exhausted.\n");
    exit(-1);
  }
  if (read(ifd,K->beta->gain,K->beta->Ngains*sizeof(float)) 
            != K->beta->Ngains*sizeof(float)) {
    printf("\nIinitABgain: Complete disk read of %s did not succeed %s.\n",
           BetaFn);
    exit(-1);
  }
 
 /*
  * Close Beta gain file
  */
  if (close(ifd) == -1) {
    printf("\nInitABgain: Error closing %s.\n",BetaFn);
    exit(-1);
  }

  return(K);
}/*--------------------------------------------  InitABgain  ----------------*/
#endif
