/*
 * Merge.c
 *
 *   Merge tracks that have become coincident
 *
 * 4/25/94 jph
 */

#ifndef MERGE_C
#define MERGE_C

#include "track.h"
 
/*-----------------------------------------------------------------------------
   MERGE
-----------------------------------------------------------------------------*/

void Merge(row,col,T,minCol2Track,MaxCol2Track,minRow2Track,size,Delta,RptFp)
 
  int          row;             /* Image Row of current observations */
  int          col;             /* Image Col of current observations */
  trackP       T;               /* tracks */
  int          minCol2Track;    /* first col to perform track proc. on */
  int          MaxCol2Track;    /* last  col to perform track proc. on */
  int          minRow2Track;    /* first row to perform track proc. on */
  int          size;            /* row/col dimension of images */
  int          Delta;           /* -1 if proc'ing col R to L, 1 otherwise */
  FILE        *RptFp;           /* ptr to stream for track reporting */
 
{
 
  int     i;                    /* loop counter */
  int     j;                    /* loop counter */
  int     k;                    /* loop counter */
  int     delta;                /* local version of Delta */
  int     Mcol;                 /* column being evaluated in Merge test */
  int     NUpdSameChan;         /* Num times 2 tracks upd'd from same chan */
  int     CurTrk;               /* loop counter for tracks */
  int     MergeTrk;             /* loop counter for tracks */
  int     m;                    /* loop counter */
  int     KillIndx;             /* index of track to kill */
  int     KeepIndx;             /* index of track to keep */
  int     CandAge;              /* Tracks younger than this won't be merged */
  int     MergeObs;             /* Num coincident obs chans to cause merge */
  int     temp;                 /* temp int for swapping */
  float  *Cpt;                  /* ptr into ObsChan array of CurTrk */
  float  *Mpt;                  /* ptr into ObsChan array of MergeTrk */
  double  AsumC;                /* sum of A estimates for CurTrk */
  double  AsumM;                /* sum of A estimates for MergeTrk */

 
  CandAge = (int)((double)size / (double)8.0);
  MergeObs = CandAge >> 1;
  m = row * size + col;
 /*
  * Loop over tracks looking for ones to merge
  */
  for (CurTrk=0; CurTrk < MAXNTRACK; CurTrk++) {
    if (T[CurTrk].active && !T[CurTrk].dormant) {
      if (T[CurTrk].age >= CandAge) {
        for (MergeTrk=CurTrk+1; MergeTrk < MAXNTRACK; MergeTrk++) {
          if (!T[MergeTrk].active) continue;
          delta = Delta;
          for(Cpt=&T[CurTrk].ObsChan[m],
              Mpt=&T[MergeTrk].ObsChan[m],
              Mcol=col,NUpdSameChan=0,j=0; j < MergeObs; j++) {
            if (*Cpt == *Mpt) {
              NUpdSameChan++;
            }
            if (delta == 1) {
              if (Mcol == minCol2Track) {
                delta = -1;
                Cpt -= size;
                Mpt -= size;
              } else {
                Mcol--;
                Cpt--;
                Mpt--;
              }
            } else {
              if (Mcol == MaxCol2Track) {
                delta = 1;
                Cpt -= size;
                Mpt -= size;
              } else {
                Mcol++;
                Cpt++;
                Mpt++;
              }
            }
          }
          if ((double)NUpdSameChan > (double)0.8 * (double)MergeObs) {
           /*
            * Merge Condition -- keep track with bigger amplitude sum
            */
            AsumC = AsumM = (double)0.0;
            for (k=i=0; i <= row; i++) {
              for (j=0; j < size; j++,k++) {
                AsumC += (double)T[CurTrk].A_filt[k];
                AsumM += (double)T[MergeTrk].A_filt[k];
              }
            }
            if (AsumM > AsumC) {
              KeepIndx = MergeTrk;
              KillIndx = CurTrk;
            } else {
              KeepIndx = CurTrk;
              KillIndx = MergeTrk;
            }
            if (T[KillIndx].protect) {
              temp = KillIndx;
              KillIndx = KeepIndx;
              KeepIndx = temp;
            }
            fprintf(RptFp,
              "\nMerge: killing trk %d @ row %d, col %d: Merge with trk %d\n",
                   KillIndx, row, col, KeepIndx);
            KillTrack(T+KillIndx,size);
          }
          if (!T[CurTrk].active) break;
        }
      }
    }
  }
  return;

}/*--------------------------------------------  Merge  ---------------------*/
#endif
