/*
 * NoFilt.c
 *
 *  This program performs the ESA on a single component image.  No filtering
 *  is performed.  
 *
 *  It was made by modifying the program 
 *     /anchovy/export/home/joebob/track/MkObs.c
 *
 *  usage: NoFilt ParameterFileName
 *
 *  jph 11/10/94
 *
 *  Major Mods 2/28/96
 */

#ifndef NOFILT_C
#define NOFILT_C

#include <stdio.h>
#include <math.h>
#include <fcntl.h>
#include <string.h>

#include "teager2D.h"

/*
 * Function prototypes
 */
void Initialize();
void ReadParametersFile();
void NoFilt_esa();
void AllocArrays();

main(argc,argv)

  int   argc;
  char *argv[];

{

  pFloatImage        X;              /* input image */
  int                size;           /* row/col dim of the square input image */
  int                n;              /* size squared */
  int                i;              /* counter */
  int                row;            /* image row counter */
  int                col;            /* image col counter */
  float             *A;              /* A estimates */
  float             *U;              /* U estimates */
  float             *V;              /* V estimates */
  BYTE              *a_valid;        /* 1 where the a estimate is valid */
  BYTE              *u_valid;        /* 1 where the u estimate is valid */
  BYTE              *v_valid;        /* 1 where the v estimate is valid */
  char               Afn[FNSIZE];    /* filename for amplitude estimates */
  char               Ufn[FNSIZE];    /* filename for horiz freq estimates */
  char               Vfn[FNSIZE];    /* filename for vert  freq estimates */

 /*
  * Check for right number of arguments
  */
  if (argc != 2) {
    printf("\n\n%s: 2D Unfiltered AM/FM analysis.",argv[0]);
    printf("\nusage: %s ParameterFileName\n",argv[0]);
    exit(-1);
  }
#ifdef DEBUG
 /*
  * Link to PA so we can use it in the debugger 
  */
  if (!argc) {
    PA((void *)X, 0, -1, 'b');
    PRINTA((void *)X,0,-1,'b',"delete.me");
    WRITEA((void *)X,0,-1,'b',"delete.me");
    image2disk((void *)X,1,1,'*',"delete.me");
    SpecMag2disk(X,"delete.me");
    PC(X,0,0,-1,'b',0);
    PRINTC(X,0,0,-1,'b',0,"delete.me");
    WRITEC(X,0,0,-1,'b',0,"delete.me");
  }
#endif

  X = (pFloatImage)malloc(sizeof(FloatImage));
  Initialize(argv[1],&size,&X->Real,&X->Imag,&X->FFTReal,&X->FFTImag,Afn,Ufn,
             Vfn);
  n = size * size;
  X->size = X->cols = X->rows = size;
  X->square = 1; 
  AllocArrays(argv,X,&a_valid,&u_valid,&v_valid,&A,&U,&V);

 /*
  * Invoke the ESA
  */
  NoFilt_esa(X,A,U,V,a_valid,u_valid,v_valid);

 /*
  * Interpolate for yet indeterminate values
  */
  for (i=0,row=0; row<size; row++) {
    for (col=0; col<size; col++,i++) {
      if (!u_valid[i]) f_interp(U,row,col,size,size,u_valid);
      if (!v_valid[i]) f_interp(V,row,col,size,size,v_valid);
      if (!a_valid[i]) f_interp(A,row,col,size,size,a_valid);
    }    /* for col */
  }      /* for row */

 /*
  * Write result files 
  */
  if (strcmp(Afn,"none")) {
    f2disk(A,X->size,X->size,Afn);
  }
  if (strcmp(Ufn,"none")) {
    f2disk(U,X->size,X->size,Ufn);
  }
  if (strcmp(Vfn,"none")) {
    f2disk(V,X->size,X->size,Vfn);
  }
  printf("\n\n");
  return;

} /*-------------------------------------------- main --------------*/

/*
 * Initialize
 *
 *  Process the paramters file
 *  Read the input image, if real make hardy to get the imaginary part
 *  Compute the input spectrum
 */

void Initialize(ParmFn,size,xReal,xImag,XReal,XImag,Afn,Ufn,Vfn)

  char        *ParmFn;           /* filename of parameters file */
  int         *size;             /* row/col dim of the square input image */
  float      **xReal;            /* real part of input image */
  float      **xImag;            /* imag part of input image */
  float      **XReal;            /* real part of input spectrum */
  float      **XImag;            /* imag part of input spectrum */
  char        *Afn;              /* filename for Amplitude Estimates */
  char        *Ufn;              /* filename for horiz frequency estimates */
  char        *Vfn;              /* filename for vert  frequency estimates */
{

  char        datatype;          /* type of in dat: b=byte f=float d=double */
  int         Conjugate;         /* 1=conjugate input.  See Book X, p. 64 */
  char        Rfn[FNSIZE];       /* file for Real part of input image */
  char        Ifn[FNSIZE];       /* file for Imag part of input image */
  int         i;                 /* counter */

 /*
  * Read the parameter file
  */
  ReadParametersFile(ParmFn,&datatype,&Conjugate,Rfn,Ifn,size,Afn,Ufn,Vfn);
 
 /*
  * Allocate Spectra
  */
  if ((*XReal = (float *)malloc(*size**size*sizeof(float))) == NULL) {
    printf("\nInitialize: free store exhausted\n");
    exit(-1);
  }
  if ((*XImag = (float *)malloc(*size**size*sizeof(float))) == NULL) {
    printf("\nInitialize: free store exhausted\n");
    exit(-1);
  }

 /*
  * Get the input image and its spectrum
  */
  *xReal = readasf(*size,*size,Rfn,datatype);

  if (strcmp(Ifn,"none")) {                     /* filename is not "none" */
    *xImag = readasf(*size,*size,Ifn,datatype);
    if (Conjugate) {
      for (i=0; i < *size**size; i++) {
        (*xImag)[i] *= (float)-1.0;
      }
    }
/*    printf("\n\n >>>> Invoking f_u_norm.");*/
/*    f_u_norm(*xReal, *xImag, *size, *size);*/
    fft2d(*xReal,*xImag,*XReal,*XImag,*size,1);
  }
  else {                         /* input is real, must create imag part */
/*    f_u_norm(*xReal, NULL, *size, *size);*/
    if ((*xImag = (float *)malloc(*size**size*sizeof(float))) == NULL) {
      printf("\nInitialize: free store exhausted\n");
      exit(-1);
    }
    MakeHardy(*xReal,*xImag,*XReal,*XImag,*size);
    if (Conjugate) {
      printf("\n\n >>>  CONJUGATING input made by MakeHardy...\n");
      for (i=0; i < *size**size; i++) {
        (*xImag)[i] *= (float)-1.0;
      }
      fft2d(*xReal,*xImag,*XReal,*XImag,*size,1);
    }
  }

  return;
} /*---------------------------------------------- Initialize ----------*/

/*
 * ReadParametersFile
 *
 *  read the data out of the parameters file
 *
 */

void ReadParametersFile(ParmFn,pdatatype,pConjugate,Rfn,Ifn,psize,Afn,Ufn,Vfn)

  char   *ParmFn;
  char   *pdatatype;
  int    *pConjugate;
  char   *Rfn;
  char   *Ifn;
  int    *psize;
  char   *Afn;
  char   *Ufn;
  char   *Vfn;
{

  FILE      *parmfp;            /* ptr to parameter file */
  char       instr[256];        /* buffer to read into  */

  if ((parmfp = fopen(ParmFn,"r")) == NULL) {
    printf("\n\nReadParametersFile: unable to open %s\n",ParmFn);
    exit(-1);
  }

  fscanf(parmfp,"Input Data Type= %s\n",instr);
  *pdatatype = instr[0];
  fscanf(parmfp,"Conjugate Input= %s\n",instr);
  if (instr[0] == 'N') {
    *pConjugate = 0;
  }
  else {
    *pConjugate = 1;
  }
  fscanf(parmfp,"Real Image File= %s\n",Rfn);
  fscanf(parmfp,"Imag Image File= %s\n",Ifn);
  fscanf(parmfp,"Size= %d\n",psize);
  fscanf(parmfp,"A File= %s\n",Afn);
  fscanf(parmfp,"U File= %s\n",Ufn);
  fscanf(parmfp,"V File= %s\n",Vfn);
  fclose(parmfp);
  return;
} /*--------------------------- ReadParametersFile -------------------------*/

/*
 * NoFilt_esa
 *
 *  function evaluates the unfiltered esa at every point in a signal.
 *
 */

void NoFilt_esa(X,a,u,v,a_valid,u_valid,v_valid)

  pFloatImage     X;              /* response image */
  float          *a;              /* a estimates */
  float          *u;              /* u estimates */
  float          *v;              /* v estimates */
  BYTE           *a_valid;        /* 1 where the a estimate is valid */
  BYTE           *u_valid;        /* 1 where the u estimate is valid */
  BYTE           *v_valid;        /* 1 where the v estimate is valid */
{

  int             row,col;        /* loop counters */
  int             k;              /* cursor */


 /*
  * Apply the ESA at each point in the image
  */
  for (k=row=0; row < X->size; row++) {
    for (col=0; col < X->size; col++,k++) {
      PtEsa2DNoFilt(X,row,col,&a[k],&u[k],&v[k],&a_valid[k],&u_valid[k],
                    &v_valid[k]);
    }
  }
  return;
} /*----------------------------------------- dom_esa -----------------------*/
 
/*
 * AllocArrays
 *
 *  Allocate Working Arrays.
 */
 
void AllocArrays(argv,X,a_valid,u_valid,v_valid,A,U,V)
 
  char              *argv[];         /* argument list from command line */
  pFloatImage        X;              /* input signal */
  BYTE             **a_valid;        /* 1 where the a estimate is valid */
  BYTE             **u_valid;        /* 1 where the u estimate is valid */
  BYTE             **v_valid;        /* 1 where the v estimate is valid */
  float            **A;              /* A estimates */
  float            **U;              /* U estimates */
  float            **V;              /* V estimates */
{
 
 /*
  * Allocate working arrays
  */
  if ((*a_valid = (BYTE *) malloc(X->size*X->size*sizeof(BYTE))) == NULL) {
    printf("\n%s: free store exhausted.\n",argv[0]);
    exit(-1);
  }
  if ((*u_valid = (BYTE *) malloc(X->size*X->size*sizeof(BYTE))) == NULL) {
    printf("\n%s: free store exhausted.\n",argv[0]);
    exit(-1);
  }
  if ((*v_valid = (BYTE *) malloc(X->size*X->size*sizeof(BYTE))) == NULL) {
    printf("\n%s: free store exhausted.\n",argv[0]);
    exit(-1);
  }
  if ((*A = (float *)malloc(X->size*X->size*sizeof(float))) == NULL) {
    printf("\n%s: free store exhausted.\n",argv[0]);
    exit(-1);
  }
  if ((*U = (float *)malloc(X->size*X->size*sizeof(float))) == NULL) {
    printf("\n%s: free store exhausted.\n",argv[0]);
    exit(-1);
  }
  if ((*V = (float *)malloc(X->size*X->size*sizeof(float))) == NULL) {
    printf("\n%s: free store exhausted.\n",argv[0]);
    exit(-1);
  }
  return;
} /*---------------------  AllocArrays  --------------------------------------*/
#endif
