/*
 * NoFiltTK.c
 *
 *  This program applies the 2D discrete Teager-Kaiser operator to an image
 *  to compute frequency and amplitude estimates.  No filtering is performed.  
 *
 *  It was made by modifying the program 
 *     /anchovy/export/home2/joebob/track/NoFilt.c
 *
 *  usage: NoFiltTK ParameterFileName
 *
 *  jph 06/13/96
 *
 */

#ifndef NOFILTTK_C
#define NOFILTTK_C

#include <stdio.h>
#include <math.h>
#include <fcntl.h>
#include <string.h>

#include "teager2D.h"

/*
 * Function prototypes
 */
void Initialize();
void ReadParametersFile();
void NoFilt_esa();
float Phi_d();
void AllocArrays();

main(argc,argv)

  int   argc;
  char *argv[];

{

  pFloatImage        X;              /* input image */
  int                size;           /* row/col dim of the square input image */
  int                n;              /* size squared */
  int                i;              /* counter */
  int                row;            /* image row counter */
  int                col;            /* image col counter */
  float             *A;              /* A estimates */
  float             *U;              /* U estimates */
  float             *V;              /* V estimates */
  BYTE              *a_valid;        /* 1 where the a estimate is valid */
  BYTE              *u_valid;        /* 1 where the u estimate is valid */
  BYTE              *v_valid;        /* 1 where the v estimate is valid */
  char               Afn[FNSIZE];    /* filename for amplitude estimates */
  char               Ufn[FNSIZE];    /* filename for horiz freq estimates */
  char               Vfn[FNSIZE];    /* filename for vert  freq estimates */

 /*
  * Check for right number of arguments
  */
  if (argc != 2) {
    printf("\n\n%s: 2D Unfiltered AM/FM analysis by 2D TK operator.",argv[0]);
    printf("\nusage: %s ParameterFileName\n",argv[0]);
    exit(-1);
  }
#ifdef DEBUG
 /*
  * Link to PA so we can use it in the debugger 
  */
  if (!argc) {
    PA((void *)X, 0, -1, 'b');
    PRINTA((void *)X,0,-1,'b',"delete.me");
    WRITEA((void *)X,0,-1,'b',"delete.me");
    image2disk((void *)X,1,1,'*',"delete.me");
    SpecMag2disk(X,"delete.me");
    PC(X,0,0,-1,'b',0);
    PRINTC(X,0,0,-1,'b',0,"delete.me");
    WRITEC(X,0,0,-1,'b',0,"delete.me");
  }
#endif

  X = (pFloatImage)malloc(sizeof(FloatImage));
  Initialize(argv[1],&size,&X->Real,&X->Imag,&X->FFTReal,&X->FFTImag,Afn,Ufn,
             Vfn);
  n = size * size;
  X->size = X->cols = X->rows = size;
  X->square = 1; 
  AllocArrays(argv,X,&a_valid,&u_valid,&v_valid,&A,&U,&V);

 /*
  * Invoke the ESA
  */
  NoFilt_esa(X,A,U,V,a_valid,u_valid,v_valid);

 /*
  * Interpolate for yet indeterminate values
  */
  for (i=0,row=0; row<size; row++) {
    for (col=0; col<size; col++,i++) {
      if (!u_valid[i]) f_interp(U,row,col,size,size,u_valid);
      if (!v_valid[i]) f_interp(V,row,col,size,size,v_valid);
      if (!a_valid[i]) f_interp(A,row,col,size,size,a_valid);
    }    /* for col */
  }      /* for row */

 /*
  * Write result files 
  */
  if (strcmp(Afn,"none")) {
    f2disk(A,X->size,X->size,Afn);
  }
  if (strcmp(Ufn,"none")) {
    f2disk(U,X->size,X->size,Ufn);
  }
  if (strcmp(Vfn,"none")) {
    f2disk(V,X->size,X->size,Vfn);
  }
  printf("\n\n");
  return;

} /*-------------------------------------------- main --------------*/

/*
 * Initialize
 *
 *  Process the paramters file
 *  Read the input image, if real make hardy to get the imaginary part
 *  Compute the input spectrum
 */

void Initialize(ParmFn,size,xReal,xImag,XReal,XImag,Afn,Ufn,Vfn)

  char        *ParmFn;           /* filename of parameters file */
  int         *size;             /* row/col dim of the square input image */
  float      **xReal;            /* real part of input image */
  float      **xImag;            /* imag part of input image */
  float      **XReal;            /* real part of input spectrum */
  float      **XImag;            /* imag part of input spectrum */
  char        *Afn;              /* filename for Amplitude Estimates */
  char        *Ufn;              /* filename for horiz frequency estimates */
  char        *Vfn;              /* filename for vert  frequency estimates */
{

  char        datatype;          /* type of in dat: b=byte f=float d=double */
  char        Rfn[FNSIZE];       /* file for Real part of input image */
  int         i;                 /* counter */

 /*
  * Read the parameter file
  */
  ReadParametersFile(ParmFn,&datatype,Rfn,size,Afn,Ufn,Vfn);
 
 /*
  * Do Not Allocate Unneeded Spectra
  */
  *XReal = *XImag = NULL;

 /*
  * Get the input image and its spectrum
  */
  *xReal = readasf(*size,*size,Rfn,datatype);

  return;
} /*---------------------------------------------- Initialize ----------*/

/*
 * ReadParametersFile
 *
 *  read the data out of the parameters file
 *
 */

void ReadParametersFile(ParmFn,pdatatype,Rfn,psize,Afn,Ufn,Vfn)

  char   *ParmFn;
  char   *pdatatype;
  char   *Rfn;
  int    *psize;
  char   *Afn;
  char   *Ufn;
  char   *Vfn;
{

  FILE      *parmfp;            /* ptr to parameter file */
  char       instr[256];        /* buffer to read into  */

  if ((parmfp = fopen(ParmFn,"r")) == NULL) {
    printf("\n\nReadParametersFile: unable to open %s\n",ParmFn);
    exit(-1);
  }

  fscanf(parmfp,"Input Data Type= %s\n",instr);
  *pdatatype = instr[0];
  fscanf(parmfp,"Real Image File= %s\n",Rfn);
  fscanf(parmfp,"Size= %d\n",psize);
  fscanf(parmfp,"A File= %s\n",Afn);
  fscanf(parmfp,"U File= %s\n",Ufn);
  fscanf(parmfp,"V File= %s\n",Vfn);
  fclose(parmfp);
  return;
} /*--------------------------- ReadParametersFile -------------------------*/

/*
 * NoFilt_esa
 *
 *  function evaluates the unfiltered esa at every point in a signal.
 *
 */

void NoFilt_esa(X,a,u,v,a_valid,u_valid,v_valid)

  pFloatImage     X;              /* response image */
  float          *a;              /* a estimates */
  float          *u;              /* u estimates */
  float          *v;              /* v estimates */
  BYTE           *a_valid;        /* 1 where the a estimate is valid */
  BYTE           *u_valid;        /* 1 where the u estimate is valid */
  BYTE           *v_valid;        /* 1 where the v estimate is valid */
{

  int             row,col;        /* loop counters */
  int             k;              /* cursor */
  pFloatImage     Unumer;         /* numerator   difference image */
  pFloatImage     Vnumer;         /* denominator difference image */
  float           Denom;          /* ESA denominator */
  float           Frac;           /* Phi_d(Numer) / Denom */

 /*
  * Allocate Unumer and Vnumer
  */
  Unumer = (pFloatImage)malloc(sizeof(FloatImage));
  AllocFloatImage(Unumer,X->size);
  Vnumer = (pFloatImage)malloc(sizeof(FloatImage));
  AllocFloatImage(Vnumer,X->size);

 /*
  * Compute diff image for U,V on 1st, 2nd, ultimate, penultimate row and col
  */
  for (k=0; k < X->size; k++) {
    Unumer->Real[k*X->size] = Unumer->Real[k*X->size+1] 
                      = X->Real[k*X->size+1] - X->Real[k*X->size];
    Unumer->Real[k*X->size+X->size-2] = Unumer->Real[k*X->size+X->size-1] 
                          = X->Real[k*X->size+X->size-1] 
                            - X->Real[k*X->size+X->size-2];
    Vnumer->Real[k] = Vnumer->Real[X->size+k] = X->Real[X->size+k] - X->Real[k];
    Vnumer->Real[(X->size-1)*X->size+k] = Vnumer->Real[(X->size-2)*X->size+k] 
                                  = X->Real[(X->size-1)*X->size+k] 
                                  - X->Real[(X->size-2)*X->size+k];
  }

 /*
  * Compute diff image for U,V at the rest of the interior pixels
  */
  for (row=2; row < X->size-2; row++) {
    for (col=2; col < X->size-2; col++) {
      k = row*X->size + col;
      Unumer->Real[k] = X->Real[k+1] - X->Real[k-1];
      Vnumer->Real[k] = X->Real[k+X->size] - X->Real[k-X->size];
    }
  }

 /*
  * Apply the ESA at each point in the image
  */
  for (k=row=0; row < X->size; row++) {
    for (col=0; col < X->size; col++,k++) {
     /*
      * Compute U,V
      */
      Denom = (float)4.0 * Phi_d(X,row,col);
      if (Denom <= (float)0.0) {
        u[k] = v[k] = (float)0.0;
        u_valid[k] = v_valid[k] = (BYTE)0;
      } else {
       /*
        * U
        */
        Frac = Phi_d(Unumer,row,col) / Denom;
        if (Frac < (float)0.0) {
          u[k] = (float)0.0;
          u_valid[k] = (BYTE)0;
        } else {
          Frac = (float)pow((double)Frac,(double)0.5);
          if (Frac < (float)-1.0) {
            Frac = (float)-1.0;
          } else {
            if (Frac > (float)1.0) {
              Frac = (float)1.0;
            }
          }
          u[k] = (float)asin((double)Frac);
          u_valid[k] = (BYTE)1;
        }
       /*
        * V
        */
        Frac = Phi_d(Vnumer,row,col) / Denom;
        if (Frac < (float)0.0) {
          v[k] = (float)0.0;
          v_valid[k] = (BYTE)0;
        } else {
          Frac = (float)pow((double)Frac,(double)0.5);
          if (Frac < (float)-1.0) {
            Frac = (float)-1.0;
          } else {
            if (Frac > (float)1.0) {
              Frac = (float)1.0;
            }
          }
          v[k] = (float)asin((double)Frac);
          v_valid[k] = (BYTE)1;
        }
      }
     /*
      * Compute A
      */
      if (u_valid[k] && v_valid[k]) {
        Denom = (float)(sin((double)u[k]) * sin((double)u[k]) 
                      + sin((double)v[k]) * sin((double)v[k]));
        if (Denom <= (float)0.0) {
          a[k] = (float)0.0;
          a_valid[k] = (BYTE)0;
        } else {
          Frac = Phi_d(X,row,col) / Denom;
          if (Frac < (float)0.0) {
           a[k] = (float)0.0;
           a_valid[k] = (BYTE)0;
          } else {
            a[k] = (float)pow((double)Frac,(double)0.5);
            a_valid[k] = (BYTE)1;
          }
        }
      } else {
        a[k] = (float)0.0;
        a_valid[k] = (BYTE)0;
      }
    }    /* for i */
  }      /* for j */

 /*
  * Deallocate Unumer and Vnumer
  */
  Unumer = KillFloat(Unumer);
  Vnumer = KillFloat(Vnumer);

  return;
} /*----------------------------------------- NoFilt_esa --------------------*/

/*
 * Phi_d
 *
 *  function returns the two-dimensional, discrete TK operator evaulated
 *  at the point row,col in the image X.
 *
 */

float Phi_d(X,row,col)

  pFloatImage     X;              /* image */
  int             row;            /* row of pixel to apply operator to */
  int             col;            /* col of pixel to apply operator to */
{

  int             k;              /* cursor into image */
  float           Rtn;            /* return value */

  if ((row==0) || (col==0) || (row==X->size-1) || (col==X->size-1)) {
    return((float)0.0);
  } else {
    k = row*X->size + col;
    Rtn = (float)2.0 * X->Real[k] * X->Real[k] 
                   - X->Real[k-1] * X->Real[k+1]
                   - X->Real[k-X->size] *  X->Real[k+X->size];
    return(Rtn);
  }
} /*----------------------------------------- Phi_d -------------------------*/
 
/*
 * AllocArrays
 *
 *  Allocate Working Arrays.
 */
 
void AllocArrays(argv,X,a_valid,u_valid,v_valid,A,U,V)
 
  char              *argv[];         /* argument list from command line */
  pFloatImage        X;              /* input signal */
  BYTE             **a_valid;        /* 1 where the a estimate is valid */
  BYTE             **u_valid;        /* 1 where the u estimate is valid */
  BYTE             **v_valid;        /* 1 where the v estimate is valid */
  float            **A;              /* A estimates */
  float            **U;              /* U estimates */
  float            **V;              /* V estimates */
{
 
 /*
  * Allocate working arrays
  */
  if ((*a_valid = (BYTE *) malloc(X->size*X->size*sizeof(BYTE))) == NULL) {
    printf("\n%s: free store exhausted.\n",argv[0]);
    exit(-1);
  }
  if ((*u_valid = (BYTE *) malloc(X->size*X->size*sizeof(BYTE))) == NULL) {
    printf("\n%s: free store exhausted.\n",argv[0]);
    exit(-1);
  }
  if ((*v_valid = (BYTE *) malloc(X->size*X->size*sizeof(BYTE))) == NULL) {
    printf("\n%s: free store exhausted.\n",argv[0]);
    exit(-1);
  }
  if ((*A = (float *)malloc(X->size*X->size*sizeof(float))) == NULL) {
    printf("\n%s: free store exhausted.\n",argv[0]);
    exit(-1);
  }
  if ((*U = (float *)malloc(X->size*X->size*sizeof(float))) == NULL) {
    printf("\n%s: free store exhausted.\n",argv[0]);
    exit(-1);
  }
  if ((*V = (float *)malloc(X->size*X->size*sizeof(float))) == NULL) {
    printf("\n%s: free store exhausted.\n",argv[0]);
    exit(-1);
  }
  return;
} /*---------------------  AllocArrays  --------------------------------------*/
#endif
