/*
 *
 * ChanSum.c
 *
 *   Run the filterbank.
 *   Scale each response by peak value of channel filter freq response.
 *   Add scaled responses.
 *
 */

#ifndef CHANRECON_C
#define CHANRECON_C

#include <stdio.h>
#include <math.h>
#include <fcntl.h>
#include <string.h>

#include "teager2D.h"

/*
 * Function prototypes
 */
void Initialize();
void ReadParametersFile();

main(argc,argv)

  int   argc;
  char *argv[];

{

  pFloatImage        X;              /* input image */
  pFloatImage        Y;              /* filtered image */
  float             *Ysum;           /* sum of response images */
  float              OneOnGmax;      /* 1/Gmax */
  filterbank        *G;              /* the filter bank */
  pFloatImage        g;              /* postfilter image */
  int                size;           /* row/col dim of the square input image */
  int                so2;            /* size / 2 */
  int                n;              /* size squared */
  int                i;              /* counter */
  int                j;              /* counter */
  char               RootFn[FNSIZE]; /* Root filename for Comp Real data */
  char               Fn[FNSIZE];     /* filename for Component Real data */

 /*
  * Check for right number of arguments
  */
  if (argc != 2) {
    printf("\n\n%s: Reconstruct AM/FM components for each channel.",argv[0]);
    printf("\nusage: %s ParameterFileName\n",argv[0]);
    exit(-1);
  }
#ifdef DEBUG
 /*
  * Link to PA so we can use it in the debugger 
  */
  if (!argc) {
    PA((void *)X, 0, -1, 'b');
    PRINTA((void *)X,0,-1,'b',"delete.me");
    WRITEA((void *)X,0,-1,'b',"delete.me");
    image2disk((void *)X,1,1,'*',"delete.me");
    SpecMag2disk(X,"delete.me");
    PC(X,0,0,-1,'b',0);
    PRINTC(X,0,0,-1,'b',0,"delete.me");
    WRITEC(X,0,0,-1,'b',0,"delete.me");
  }
#endif
  X = (pFloatImage)malloc(sizeof(FloatImage));
  Y = (pFloatImage)malloc(sizeof(FloatImage));

  Initialize(argv[1],&size,&X->Real,&X->Imag,&X->FFTReal,&X->FFTImag,&G,RootFn);
  X->size = X->cols = X->rows = size;
  X->square = 1; 
  AllocFloatImage(Y,size);
  n = size * size;
  if ((Ysum = (float *)malloc(Y->size*Y->size*sizeof(float))) == NULL) {
    printf("\n%s: free store exhausted.\n",argv[0]);
    exit(-1);
  }
  for (i=0; i < n; i++) {
    Ysum[i] = (float)0.0;
  }


 /*
  * Do DC Filter, if required
  */
  if (G->DCfilt != NULL) {
    printf("\nDoing DC Filter...");
/*    ConvolveGabor(Y,X,G->DCfilt);*/
    HardyConvolveGabor(Y,X,G->DCfilt);
    OneOnGmax = (float)1.0 / G->DCfilt->GMax;
    for (j=0; j < n; j++) {
      Ysum[j] += Y->Real[j] * OneOnGmax;
    }
  }              /* if G->DCfilt */

 /*
  * Loop for filters
  */
  for (i=0; i < G->TotNumFilters; i++) {
    printf("\nDoing Filter %d...",i);
/*    ConvolveGabor(Y,X,G->filt[i]);*/
    HardyConvolveGabor(Y,X,G->filt[i]);
    OneOnGmax = (float)1.0 / G->filt[i]->GMax;
    for (j=0; j < n; j++) {
      Ysum[j] += Y->Real[j] * OneOnGmax;
    }
  }              /* for i */

 /*
  * Write the reconstructed components out to disk files as BYTE data
  */
  sprintf(Fn,"%sSum.float",RootFn);
  f2disk(Ysum,size,size,Fn);

  printf("\n\n");
  return;

} /*-------------------------------------------- main --------------*/

/*
 * Initialize
 *
 *  Process the paramters file
 *  Read the input image, if real make hardy to get the imaginary part
 *  Compute the input spectrum
 *  Construct the filter bank
 */

void Initialize(ParmFn,size,xReal,xImag,XReal,XImag,G,RootFn)

  char        *ParmFn;           /* filename of parameters file */
  int         *size;             /* row/col dim of the square input image */
  float      **xReal;            /* real part of input image */
  float      **xImag;            /* imag part of input image */
  float      **XReal;            /* real part of input spectrum */
  float      **XImag;            /* imag part of input spectrum */
  filterbank **G;                /* the filter bank */
  char        *RootFn;           /* root filename for recon components */
{

  int         i;                 /* counter */
  int         row;               /* counter for image rows */
  int         col;               /* counter for image cols */
  int         so2;               /* *size / 2 */
  char        datatype;          /* type of in dat: b=byte f=float d=double */
  int         Conjugate;         /* 1=conjugate input.  See Book X, p. 64 */
  char        Rfn[FNSIZE];       /* file for Real part of input image */
  char        Ifn[FNSIZE];       /* file for Imag part of input image */
  int         DoDCfilt;          /* 1 if filterbank incorporates a DC filter */
  double      r0;                /* rad. center freq. of 1st filt on each ray */
  float       R;                 /* common ratio of radii of adjcnt filtrs */
  float       obw;               /* Eta-peak octave bandwidth of filters */
  float       Eta;               /* OBW is defined in terms of Eta-peak. */
  float      *tfloat;            /* temporary float array */
  float      *to;                /* area in memory to copy floats to */
  float      *from;              /* area in memory to copy floats from */
  float      *pfloat;            /* ptr to location to fill w/ zero */

 /*
  * Read the parameter file
  */
  ReadParametersFile(ParmFn,&datatype,&Conjugate,Rfn,Ifn,size,&r0,&R,&Eta,&obw,
                     &DoDCfilt,RootFn);
  r0 = r0 / (double)(*size);
 
  so2 = *size >> 1;

 /*
  * Make the filter bank
  */ 
  *G = CreateFilterBank(r0,R,obw,Eta,(int)0,(float)1,(float)1,(float)1);
  if (DoDCfilt) {
    (*G)->DCfilt = DesignDCfilt(*G);
  }

 /*
  * Allocate Spectra
  */
  if ((*XReal = (float *)malloc(*size**size*sizeof(float))) == NULL) {
    printf("\nInitialize: free store exhausted\n");
    exit(-1);
  }
  if ((*XImag = (float *)malloc(*size**size*sizeof(float))) == NULL) {
    printf("\nInitialize: free store exhausted\n");
    exit(-1);
  }

 /*
  * Get the input image and its spectrum
  */
  *xReal = readasf(*size,*size,Rfn,datatype);

  if (strcmp(Ifn,"none")) {                     /* filename is not "none" */
    *xImag = readasf(*size,*size,Ifn,datatype);
    if (Conjugate) {
      for (i=0; i < *size**size; i++) {
        (*xImag)[i] *= (float)-1.0;
      }
    }
    fft2d(*xReal,*xImag,*XReal,*XImag,*size,1);
  } else {                         /* input is real, must create imag part */
    if ((*xImag = (float *)malloc(*size**size*sizeof(float))) == NULL) {
      printf("\nInitialize: free store exhausted\n");
      exit(-1);
    }
    MakeHardy(*xReal,*xImag,*XReal,*XImag,*size);
    if (Conjugate) {
      printf("\n\n >>>  CONJUGATING input made by MakeHardy...\n");
      for (i=0; i < *size**size; i++) {
        (*xImag)[i] *= (float)-1.0;
      }
      fft2d(*xReal,*xImag,*XReal,*XImag,*size,1);
    }
  }

  return;
} /*---------------------------------------------- Initialize ----------*/

/*
 * ReadParametersFile
 *
 *  read the data out of the parameters file
 *
 */

void ReadParametersFile(ParmFn,pdatatype,pConjugate,Rfn,Ifn,psize,pr0,pR,pEta,
                        pobw,pDoDCfilt,RootFn)
  char   *ParmFn;
  char   *pdatatype;
  int    *pConjugate;
  char   *Rfn;
  char   *Ifn;
  int    *psize;
  double *pr0;
  float  *pR;
  float  *pEta;
  float  *pobw;
  int    *pDoDCfilt;
  char   *RootFn;
{

  FILE      *parmfp;            /* ptr to parameter file */
  char       instr[256];        /* buffer to read into  */

  if ((parmfp = fopen(ParmFn,"r")) == NULL) {
    printf("\n\nReadParametersFile: unable to open %s\n",ParmFn);
    exit(-1);
  }

  fscanf(parmfp,"Input Data Type= %s\n",instr);
  *pdatatype = instr[0];
  fscanf(parmfp,"Conjugate Input= %s\n",instr);
  if ((instr[0] == 'N') || (instr[0] == 'n')) {
    *pConjugate = 0;
  }
  else {
    *pConjugate = 1;
  }
  fscanf(parmfp,"Real Image File= %s\n",Rfn);
  fscanf(parmfp,"Imag Image File= %s\n",Ifn);
  fscanf(parmfp,"Size= %d\n",psize);
  fscanf(parmfp,"r0= %lf\n",pr0);
  fscanf(parmfp,"R= %f\n",pR);
  fscanf(parmfp,"Eta= %f\n",pEta);
  fscanf(parmfp,"OBW= %f\n",pobw);
  fscanf(parmfp,"Use DC Filter= %s\n",instr);
  if ((instr[0] == 'N') || (instr[0] == 'n')) {
    *pDoDCfilt = 0;
  }
  else {
    *pDoDCfilt = 1;
  }
  fscanf(parmfp,"Root Out Fn= %s\n",RootFn);
  fclose(parmfp);
  return;
} /*--------------------------- ReadParametersFile -------------------------*/
#endif
