/*
 * UpdTrkFrmObs.c
 *
 *   Contains function UpdateTrackFromObs
 *
 *   Update a track given observations from a channel.
 *
 *   REVISED 8/18/94 see book VIII p. 89.
 *
 *   3/14/96  MAJOR REVISIONS
 *
 * 4/25/94 jph
 */

#ifndef UPDTRKFRMOBS_C
#define UPDTRKFRMOBS_C

#include "track.h"
 
/*-----------------------------------------------------------------------------
   UPDATE TRACK FROM OBS
-----------------------------------------------------------------------------*/
/*
 * UpdateTrackFromObs
 *
 *   Update a track given observations from a channel.
 */
 
void UpdateTrackFromObs(T,idx,Chan,ChanQ,Aob,Uob,Vob,j,MaxCol2Track, 
                        minCol2Track,KA,KU,KV,size,wrap,RptFp)
 
  trackP  T;                    /* Track to be updated */
  int     idx;                  /* number of track being updated */
  int     Chan;                 /* Number of updating channel */
  float   ChanQ;                /* Quality of udating channel */
  float   Aob;                  /* Amp Observation */
  float   Uob;                  /* U   Observation */
  float   Vob;                  /* V   Observation */
  int     j;                    /* row*size + col */
  int     MaxCol2Track;         /* last  col to perform track proc. on */
  int     minCol2Track;         /* first col to perform track proc. on */
  pABgain KA;                   /* gains for A tracker */
  pABgain KU;                   /* gains for U tracker */
  pABgain KV;                   /* gains for V tracker */
  int     size;                 /* row/col dimension of images */
  int     wrap;                 /* 1 if update is across 1st & last rays */
  FILE   *RptFp;                /* ptr to stream for track reporting */
{
 
  float   eA;                   /* innovation in A */
  float   eU;                   /* innovation in U */
  float   eV;                   /* innovation in V */
  int     k;                    /* counter */

 
  T->updated = 1;
  if (ChanQ > T->MaxQual) {
    T->MaxQual = ChanQ;
  }
  T->AveQualRow += ChanQ / (float)(MaxCol2Track-minCol2Track);
  T->coast = T->Ncoast = 0;
  T->NgoodUpd++;
  T->NgoodUpdRow++;
  T->dormant = 0;
  T->dormantOnRow = 0;
  T->age++;
  T->HorUpd->k++; 
  T->HorUpd->LastChan = Chan;

 /*
  * If a wrap condition has occurred, reverse the direction of the wavefront
  */
  if (wrap) {
    fprintf(RptFp,"\n WRAP CONDITION: track %d row %d col %d.",
            idx,j/size,j%size);
    T->HorUpd->pred.U *= (float)-1.0;
    T->HorUpd->pred.V *= (float)-1.0;
  }

 /*
  * Compute innovations: if NO_OBSERVATION, coast
  */
  if (Aob == NO_OBSERVATION) {
    Aob = T->HorUpd->pred.A;
  }
  if (Uob == NO_OBSERVATION) {
    Uob = T->HorUpd->pred.U;
  }
  if (Vob == NO_OBSERVATION) {
    Vob = T->HorUpd->pred.V;
  }
  eA = Aob - T->HorUpd->pred.A;
  eU = Uob - T->HorUpd->pred.U;
  eV = Vob - T->HorUpd->pred.V;
  T->SumInnov += (float)pow((double)0.25*
                        pow((double)eA, (double)2.0) +
                        pow((double)eU, (double)2.0) +
                        pow((double)eV, (double)2.0), (double)0.5);
 /*
  * Compute filtered estimates
  */
  T->A_filt[j] = T->HorUpd->pred.A 
                 + KA->alpha->gain[Kidx(T->HorUpd->k,KA->alpha)] * eA;
  if (T->A_filt[j] < (float)0.0) {
    T->A_filt[j] = (float)0.0;
  }
  if (T->A_filt[j] > (float)1.0) { T->A_filt[j] = (float)1.0; }

  T->U_filt[j] = T->HorUpd->pred.U 
                 + KU->alpha->gain[Kidx(T->HorUpd->k,KU->alpha)] * eU;
  if (T->U_filt[j] > (float)0.5) {
    T->U_filt[j] = (float)0.5;
  }
  if (T->U_filt[j] < (float)-0.5) {
    T->U_filt[j] = (float)-0.5;
  }

  T->V_filt[j] = T->HorUpd->pred.V 
                 + KV->alpha->gain[Kidx(T->HorUpd->k,KV->alpha)] * eV;
  if (T->V_filt[j] > (float) 0.5) {
    T->V_filt[j] = (float) 0.5;
  }
  if (T->V_filt[j] < (float)-0.5) {
    T->V_filt[j] = (float)-0.5;
  }
  T->ObsChan[j] = Chan;
 
 /*
  * Age the predictions
  */
  T->HorUpd->v_pred.A += KA->beta->gain[Kidx(T->HorUpd->k,KA->beta)] * eA;
  T->HorUpd->v_pred.U += KU->beta->gain[Kidx(T->HorUpd->k,KU->beta)] * eU;
  T->HorUpd->v_pred.V += KV->beta->gain[Kidx(T->HorUpd->k,KV->beta)] * eV;
  T->HorUpd->pred.A = T->A_filt[j] + T->HorUpd->v_pred.A;
  if (T->HorUpd->pred.A < (float)0.0) {
    T->HorUpd->v_pred.A = (float)0.0;
  }
  if (T->HorUpd->pred.A > (float)1.0) {
    T->HorUpd->pred.A = (float)1.0;
  }
 
  T->HorUpd->pred.U = T->U_filt[j] + T->HorUpd->v_pred.U;
  if (T->HorUpd->pred.U < (float)-0.5) {
    T->HorUpd->pred.U = (float)-0.5;
  }
  if (T->HorUpd->pred.U > (float)0.5) {
    T->HorUpd->pred.U = (float)0.5;
  }
 
  T->HorUpd->pred.V = T->V_filt[j] + T->HorUpd->v_pred.V;
  if (T->HorUpd->pred.V < (float)-0.5) {
    T->HorUpd->pred.V = (float)-0.5;
  }
  if (T->HorUpd->pred.V > (float)0.5) {
    T->HorUpd->pred.V = (float)0.5;
  }
 
  return;
 
}/*--------------------------------------------  UpdateTrackFromObs  ---------*/
#endif
