/*
 * bRdPFile.c
 *
 *  Function reads the parameter file for trackb.c, which puts out boundary
 *  conditions for reconstruction.  This file is a modified version of 
 *  RdPFile.c
 *
 *  03/01/96:  MAJOR MODS
 *
 *  jph 12/6/94
 */

#ifndef BRDPFILE_C
#define BRDPFILE_C

#include "track.h"

/*-----------------------------------------------------------------------------
   READ PARAMETERS FILE
-----------------------------------------------------------------------------*/
 
void bReadParametersFile(ParmFn,pdatatype,Rfn,Ifn,psize,pZP,pReflect,
                         pConjugate,pr0,pR,pEta,pobw,pBCinterval,pminRow2Track,
                         pMaxRow2Track,pminCol2Track,pMaxCol2Track,pLimitRays,
                         pfScale,pTEScale,AalphaFn,AbetaFn,UalphaFn,UbetaFn,
                         ValphaFn,VbetaFn,Afn,Ufn,Vfn,TARootFn,TURootFn,
                         TVRootFn,BCRootFn,RptFn,pRpt2StdOut,RptStream)

  char   *ParmFn;            /* Name of the parameter file */
  char   *pdatatype;         /* Data type of samples in the input sequence */
  char   *Rfn;               /* data file for real part of input image */
  char   *Ifn;               /* data file for imag part of input image */
  int    *psize;             /* row/col dimension of images */
  int    *pZP;               /* 1 if we are to use zero padding */
  int    *pReflect;          /* 1=reflect to fill zero padded areas */
  int    *pConjugate;        /* 1=conjugate input data */
  double *pr0;               /* rad freq of 1st filter on ray in cpi */
  float  *pR;                /* ratio of cntr freq of filters on a ray */
  float  *pEta;              /* % peak resp where filters intersect */
  float  *pobw;              /* octave bandwidth of filters */
  int    *pBCinterval;       /* Reconstruction Bound Cond interval */
  int    *pminRow2Track;     /* first row to perform track proc. on */
  int    *pMaxRow2Track;     /* last  row to perform track proc. on */
  int    *pminCol2Track;     /* first col to perform track proc. on */
  int    *pMaxCol2Track;     /* last  col to perform track proc. on */
  int    *pLimitRays;        /* 1=limit num rays visited by a track */
  int    *pfScale;           /* 1=Scale track start criterion by freq */
  int    *pTEScale;          /* 1=Scale track start criterion Teager Energy */
  char   *AalphaFn;          /* file of alpha Kalman gains for A */
  char   *AbetaFn;           /* file of beta  Kalman gains for A */
  char   *UalphaFn;          /* file of alpha Kalman gains for U */
  char   *UbetaFn;           /* file of beta  Kalman gains for U */
  char   *ValphaFn;          /* file of alpha Kalman gains for V */
  char   *VbetaFn;           /* file of beta  Kalman gains for V */
  char   *Afn;               /* name of A observations file */
  char   *Ufn;               /* name of U observations file */
  char   *Vfn;               /* name of V observations file */
  char   *TARootFn;          /* root fn for tracked amplitudes */
  char   *TURootFn;          /* root fn for U tracked frequencies */
  char   *TVRootFn;          /* root fn for V tracked frequencies */
  char   *BCRootFn;          /* root fn for track boundary cond's */
  char   *RptFn;             /* ascii track report filename */
  int    *pRpt2StdOut;       /* 1=send track reporting to stdout */
  char   *RptStream;         /* Stream to send track reporting to */
{
 
  FILE      *parmfp;            /* ptr to parameter file */
  char       instr[256];        /* buffer to read into  */

  if ((parmfp = fopen(ParmFn,"r")) == NULL) {
    printf("\n\nReadParametersFile: unable to open %s\n",ParmFn);
    exit(-1);
  }

  fscanf(parmfp,"Input Data Type= %s\n",instr);
  *pdatatype = instr[0];
  fscanf(parmfp,"Real Image File= %s\n",Rfn);
  fscanf(parmfp,"Imag Image File= %s\n",Ifn);
  fscanf(parmfp,"Size= %d\n",psize);
  fscanf(parmfp,"Zero Pad= %s\n",instr);
  if ((instr[0] == 'Y') || (instr[0] == 'y')) {
    *pZP = 1;
  }
  else {
    *pZP = 0;
  }
  fscanf(parmfp,"Reflect= %s\n",instr);
  if ((instr[0] == 'Y') || (instr[0] == 'y')) {
    *pReflect = 1;
  }
  else {
    *pReflect = 0;
  }
  fscanf(parmfp,"Conjugate Input= %s\n",instr);
  if (instr[0] == 'N') {
    *pConjugate = 0;
  }
  else {
    *pConjugate = 1;
  }
  fscanf(parmfp,"r0= %lf\n",pr0);
  fscanf(parmfp,"R= %f\n",pR);
  fscanf(parmfp,"Eta= %f\n",pEta);
  fscanf(parmfp,"OBW= %f\n",pobw);
  fscanf(parmfp,"BCinterval= %d\n",pBCinterval);
  fscanf(parmfp,"Min Row to Track= %d\n",pminRow2Track);
  fscanf(parmfp,"Max Row to Track= %d\n",pMaxRow2Track);
  fscanf(parmfp,"Min Col to Track= %d\n",pminCol2Track);
  fscanf(parmfp,"Max Col to Track= %d\n",pMaxCol2Track);
  fscanf(parmfp,"Limit Rays= %s\n",instr);
  if ((instr[0] == 'Y') || (instr[0] == 'y')) {
    *pLimitRays = 1;
  }
  else {
    *pLimitRays = 0;
  }
  fscanf(parmfp,"1/f Scale= %s\n",instr);
  if ((instr[0] == 'Y') || (instr[0] == 'y')) {
    *pfScale = 1;
  }
  else {
    *pfScale = 0;
  }
  fscanf(parmfp,"Teager Energy Scale= %s\n",instr);
  if ((instr[0] == 'Y') || (instr[0] == 'y')) {
    *pTEScale = 1;
  }
  else {
    *pTEScale = 0;
  }
  fscanf(parmfp,"A Alpha File= %s\n",AalphaFn);
  fscanf(parmfp,"A Beta File= %s\n",AbetaFn);
  fscanf(parmfp,"U Alpha File= %s\n",UalphaFn);
  fscanf(parmfp,"U Beta File= %s\n",UbetaFn);
  fscanf(parmfp,"V Alpha File= %s\n",ValphaFn);
  fscanf(parmfp,"V Beta File= %s\n",VbetaFn);
  fscanf(parmfp,"A Obs File= %s\n", Afn);
  fscanf(parmfp,"U Obs File= %s\n", Ufn);
  fscanf(parmfp,"V Obs File= %s\n", Vfn);
  fscanf(parmfp,"Track A Root Fn= %s\n", TARootFn);
  fscanf(parmfp,"Track U Root Fn= %s\n", TURootFn);
  fscanf(parmfp,"Track V Root Fn= %s\n", TVRootFn);
  fscanf(parmfp,"BC Root Fn= %s\n", BCRootFn);
  fscanf(parmfp,"Track Report File= %s\n", RptFn);
  fscanf(parmfp,"Report to stdout= %s\n",instr);
  if ((instr[0] == 'Y') || (instr[0] == 'y')) {
    *pRpt2StdOut = 1;
  }
  else {
    *pRpt2StdOut = 0;
  }
  fscanf(parmfp,"Report Stream= %s\n", RptStream);

  fclose(parmfp);
  return;
}/*--------------------------------------------  bReadParameteresFile  -------*/
#endif
