/*
 * binit.c
 *
 *  binit.c is the same as init.c, but for use with trackb which outputs
 *  boundary conditions for reconstruction.
 *
 *  initialize all program variables
 *  function does initialization for track.c;  reads the parameter file.
 *
 *  calls to f_u_norm commented out 11/2/94
 *
 *  03/02/96:  MAJOR MODS
 *  
 *  jph 4/24/94
 */

#ifndef BINIT_C
#define BINIT_C

#include "track.h"

/*-----------------------------------------------------------------------------
   INITIALIZE
-----------------------------------------------------------------------------*/

void bInitialize(cmd,ParmFn,pOrigSize,pXsize,pZP,pBCinterval,xReal,xImag,XReal,
                 XImag,G,pReadSize,pKA,pKU,pKV,pAob,pUob,pVob,pfdA,pfdU,pfdV,T,
                 pminRow2Track,pMaxRow2Track,pminCol2Track,pMaxCol2Track,
                 pLimitRays,pfScale,pTEScale,TARootFn,TURootFn,TVRootFn,
                 BCRootFn,RptFn,pRptFp)

  char         *cmd;            /* unix command used to invoke this prgrm */
  char         *ParmFn;         /* parameter file name */
  int          *pOrigSize;      /* row/col dimension of original images */
  int          *pXsize;         /* size of possibly zero padded X,Y images */
  int          *pZP;            /* 1 if zero padding is to be used */
  int          *pBCinterval;    /* Reconstruction Bound Cond interval */
  float       **xReal;          /* real part of input signal */
  float       **xImag;          /* imag part of input signal */
  float       **XReal;          /* real part of input spectrum */
  float       **XImag;          /* imag part of input spectrum */
  filterbank  **G;              /* ptr to the filter bank */
  int          *pReadSize;      /* size of a row of obs for A,U, or V */
  pABgain      *pKA;            /* gains for A tracker */
  pABgain      *pKU;            /* gains for U tracker */
  pABgain      *pKV;            /* gains for V tracker */
  float       **pAob;           /* row of observations of A */
  float       **pUob;           /* row of observations of u */
  float       **pVob;           /* row of observations of v */
  int          *pfdA;           /* file descriptor for A obs file */
  int          *pfdU;           /* file descriptor for U obs file */
  int          *pfdV;           /* file descriptor for V obs file */
  trackP        T;              /* tracks */
  int          *pminRow2Track;  /* first row to perform track proc. on */
  int          *pMaxRow2Track;  /* last  row to perform track proc. on */
  int          *pminCol2Track;  /* first col to perform track proc. on */
  int          *pMaxCol2Track;  /* last  col to perform track proc. on */
  int          *pLimitRays;     /* 1=limit num rays visited by a track */
  int          *pfScale;        /* 1=scale track start criterion by freq */
  int          *pTEScale;       /* 1=scale trk strt crit by Teager Energy */
  char         *TARootFn;       /* root fn for tracked amplitudes */
  char         *TURootFn;       /* root fn for tracked U frequencies */
  char         *TVRootFn;       /* root fn for tracked V frequencies */
  char         *BCRootFn;       /* root fn for track boundary cond's */
  char         *RptFn;          /* ascii track report filename */
  FILE        **pRptFp;         /* ptr to stream for track reporting */
{

  char          datatype;       /* type of in dat: b=byte f=float d=double */
  char          Rfn[FNSIZE];    /* file for Real part of input signal */
  char          Ifn[FNSIZE];    /* file for Imag part of input signal */
  int           Conjugate;      /* 1=conjugate input */
  int           Reflect;        /* 1=reflect to fill zero padded areas */
  double        r0;             /* rad freq of 1st filter on ray in cpi */
  float         R;              /* common ratio of center frequencies */
  float         Eta;            /* % peak resp where filters intersect */
  float         obw;            /* octave bandwidth of filters */
  char          AalphaFn[FNSIZE]; /* file of alpha Kalman gains for A */
  char          AbetaFn[FNSIZE];  /* file of beta  Kalman gains for A */
  char          UalphaFn[FNSIZE]; /* file of alpha Kalman gains for U */
  char          UbetaFn[FNSIZE];  /* file of beta  Kalman gains for U */
  char          ValphaFn[FNSIZE]; /* file of alpha Kalman gains for V */
  char          VbetaFn[FNSIZE];  /* file of beta  Kalman gains for V */
  char          Afn[FNSIZE];      /* name of A observations file */
  char          Ufn[FNSIZE];      /* name of U observations file */
  char          Vfn[FNSIZE];      /* name of V observations file */
  int           i;              /* loop counter */
  int           row;            /* counter for image rows */
  int           col;            /* counter for image cols */
  int           so2;            /* *size / 2 */
  float         *tfloat;        /* temporary float array */
  float         *to;            /* area in memory to copy floats to */
  float         *from;          /* area in memory to copy floats from */
  float         *pfloat;        /* ptr to location to fill w/ zero */
  int           Rpt2StdOut;     /* 1=send track reporting to stdout */
  char          RptStream[FNSIZE];  /* Stream to send track reporting to */

 /*
  * Read the parameter file
  */
  bReadParametersFile(ParmFn,&datatype,Rfn,Ifn,pOrigSize,pZP,&Reflect,
                      &Conjugate,&r0,&R,&Eta,&obw,pBCinterval,pminRow2Track,
                      pMaxRow2Track,pminCol2Track,pMaxCol2Track,pLimitRays,
                      pfScale,pTEScale,AalphaFn,AbetaFn,UalphaFn,UbetaFn,
                      ValphaFn,VbetaFn,Afn,Ufn,Vfn,TARootFn,TURootFn,TVRootFn,
                      BCRootFn,RptFn,&Rpt2StdOut,RptStream);
  r0 /= (double)(*pOrigSize);

  if (Rpt2StdOut) {
    *pRptFp = stdout;
    printf("\n\nTrack Reporting will be sent to stdout...\n");
  } else {
    *pRptFp = fopen(RptStream,"w");
    printf("\n\nTrack Reporting will be sent to %s...\n",RptStream);
  }

 /*
  * If zero padding, double size
  */
  if (*pZP) {
    *pXsize = *pOrigSize << 1;
  } else {
    *pXsize = *pOrigSize;
  }
  so2 = *pXsize >> 1;
 
 /*
  * Make the filter bank
  */
  *G = CreateFilterBank(r0,R,obw,Eta,1,(float)0.707,(float)0.707,(float)0.707);

 /*
  * Allocate Spectra
  */
  if ((*XReal = (float *)malloc(*pXsize**pXsize*sizeof(float))) == NULL) {
    printf("\nInitialize: free store exhausted\n");
    exit(-1);
  }
  if ((*XImag = (float *)malloc(*pXsize**pXsize*sizeof(float))) == NULL) {
    printf("\nInitialize: free store exhausted\n");
    exit(-1);
  }

 /*
  * Get the input image and its spectrum
  */
  if (*pZP) {
    tfloat = readasf(so2,so2,Rfn,datatype);
    if ((*xReal = (float *)malloc(*pXsize**pXsize*sizeof(float))) == NULL) {
      printf("\nInitialize: free store exhausted\n");
      exit(-1);
    }
    for (to=*xReal,from=tfloat,row=0; row < so2; row++,to+=*pXsize,from+=so2) {
      memcpy(to, from, so2*sizeof(float));
      for (pfloat=to+so2,col=so2; col < *pXsize; col++) {
        if (!Reflect) {
          *(pfloat++) = (float)0.0;
        } else {
          *(pfloat++) = *(to + *pXsize-col-1);
        }
      }
    }
    free(tfloat);
    if (!Reflect) {
      for (pfloat=to,i=so2**pXsize; i < *pXsize**pXsize; i++) {
        *(pfloat++) = (float)0.0;
      }
    } else {
      for (pfloat=to,row=1; row < *pXsize; row+=2) {
        for (col=0; col < *pXsize; col++,pfloat++) {
          *pfloat = *(pfloat - row**pXsize);
        }
      }
    }
  }
  else {
    *xReal = readasf(*pXsize,*pXsize,Rfn,datatype);
  }
 
  if (strcmp(Ifn,"none")) {                     /* filename is not "none" */
    if (*pZP) {
      tfloat = readasf(so2,so2,Ifn,datatype);
      if ((*xImag = (float *)malloc(*pXsize**pXsize*sizeof(float))) == NULL) {
        printf("\nInitialize: free store exhausted\n");
        exit(-1);
      }
      for (to=*xImag,from=tfloat,row=0; row<so2; row++,to+=*pXsize,from+=so2) {
        memcpy(to, from, so2*sizeof(float));
        for (pfloat=to+so2,col=so2; col < *pXsize; col++) {
          if (!Reflect) {
            *(pfloat++) = (float)0.0;
          } else {
            *(pfloat++) = *(to + *pXsize-col-1);
          }
        }
      }
      free(tfloat);
      if (!Reflect) {
        for (pfloat=to,i=so2**pXsize; i < *pXsize**pXsize; i++) {
          *(pfloat++) = (float)0.0;
        }
      } else {
        for (pfloat=to,row=1; row < *pXsize; row+=2) {
          for (col=0; col < *pXsize; col++,pfloat++) {
            *pfloat = *(pfloat - row**pXsize);
          }
        }
      }
    }
    else {
      *xImag = readasf(*pXsize,*pXsize,Ifn,datatype);
    }
    if (Conjugate) {
      for (i=0; i < *pXsize**pXsize; i++) {
        (*xImag)[i] *= (float)-1.0;
      }
    }
    fft2d(*xReal,*xImag,*XReal,*XImag,*pXsize,1);
  } else {                         /* input is real, must create imag part */
    if ((*xImag = (float *)malloc(*pXsize**pXsize*sizeof(float))) == NULL) {
      printf("\nInitialize: free store exhausted\n");
      exit(-1);
    }
    MakeHardy(*xReal,*xImag,*XReal,*XImag,*pXsize);
    if (Conjugate) {
      printf("\n\n >>>  CONJUGATING input made by MakeHardy...\n");
      for (i=0; i < *pXsize**pXsize; i++) {
        (*xImag)[i] *= (float)-1.0;
      }
      fft2d(*xReal,*xImag,*XReal,*XImag,*pXsize,1);
    }
  }
 
 /*
  * Initialize Kalman gain arrays
  */
  *pKA = InitABgain(AalphaFn,AbetaFn);
  *pKU = InitABgain(UalphaFn,UbetaFn);
  *pKV = InitABgain(ValphaFn,VbetaFn);

 /*
  * Determine ReadSize for observations
  */
  *pReadSize = *pOrigSize * (*G)->TotNumFilters * sizeof(float);

 /*
  * Allocate the observation arrays 
  */
  if ((*pAob = (float *)malloc(*pReadSize)) == NULL) {
    printf("\n%s: free store exhausted.\n",cmd);
  }
  if ((*pUob = (float *)malloc(*pReadSize)) == NULL) {
    printf("\n%s: free store exhausted.\n",cmd);
  }
  if ((*pVob = (float *)malloc(*pReadSize)) == NULL) {
    printf("\n%s: free store exhausted.\n",cmd);
  }
 
 /*
  * Open the observation files
  */
  if ((*pfdA = open(Afn, O_RDONLY))==-1) {
    printf("\n%s: could not open file %s.\n",cmd,Afn);
    exit(-1);
  }
  if ((*pfdU = open(Ufn, O_RDONLY))==-1) {
    printf("\n%s: could not open file %s.\n",cmd,Ufn);
    exit(-1);
  }
  if ((*pfdV = open(Vfn, O_RDONLY))==-1) {
    printf("\n%s: could not open file %s.\n",cmd,Vfn);
    exit(-1);
  }

 /*
  * Read the first row of observations from each observation file
  */
  if ((read(*pfdA,*pAob,*pReadSize) != *pReadSize)) {
    printf("\n%s: Complete read of %s did not succeed.\n",cmd,Afn);
    exit(-1);
  }
  if ((read(*pfdU,*pUob,*pReadSize) != *pReadSize)) {
    printf("\n%s: Complete read of %s did not succeed.\n",cmd,Ufn);
    exit(-1);
  }
  if ((read(*pfdV,*pVob,*pReadSize) != *pReadSize)) {
    printf("\n%s: Complete read of %s did not succeed.\n",cmd,Vfn);
    exit(-1);
  }

 /*
  * Initialize the tracks
  */
  for (i=0; i < MAXNTRACK; i++) {
    T[i].in_use = T[i].candidate = T[i].active = 0;
  }

  return;
}/*--------------------------------------------  Initialize  ----------------*/
#endif
