/*
 *
 * fpextractuv.c
 *
 *  Extract dominant horiz and vertical frequency estimates from around the
 *  border of a latency in a fingerprint image.
 *
 *  usage: fpextract ParameterFileName
 *
 *  jph 6/20/99
 */

#ifndef FPEXTRACT_C
#define FPEXTRACT_C

#include <stdio.h>
#include <math.h>
#include <fcntl.h>
#include <string.h>

#include "teager2D.h"

/*
 * Function prototypes
 */
void ReadParametersFile();

main(argc,argv)

  int   argc;
  char *argv[];

{

  int                size;           /* row/col dim of the square input image */
  int                n;              /* size squared */
  int                i;              /* counter */
  int                row;            /* image row counter */
  int                col;            /* image col counter */
  int                k_image;        /* cursor for image pixels */
  int                k_out;          /* cursor for 1D output arrays */
  int                ULrow;          /* row of UL corner of ROI */
  int                ULcol;          /* col of UL corner of ROI */
  int                LRrow;          /* row of LR corner of ROI */
  int                LRcol;          /* col of LR corner of ROI */
  int                ROIrows;        /* number of rows in ROI */
  int                ROIcols;        /* number of cols in ROI */
  int                ROIsize;        /* number of points in border of ROI */
  float             *U;              /* DomComp U estimates */
  float             *V;              /* DomComp V estimates */
  float             *Uout;           /* U estimates from border of latency */
  float             *Vout;           /* V estimates from border of latency */
  float             *Latent;         /* Fingerprint image w/ latency */
  float             *DomCompR;       /* Real part of the dominant component */
  BYTE              *LatentOvrly;    /* Byte latent overlay output image */
  BYTE              *DomOvrly;       /* Byte DomComp overlay output image */
  char               Ufn[FNSIZE];    /* filename for horiz freq estimates */
  char               Vfn[FNSIZE];    /* filename for vert  freq estimates */
  char               LatentFn[FNSIZE];/* filename for float image w/ latency */
  char               DomCompRfn[FNSIZE];/* filename for float DomComp recon */
  char               UoutFn[FNSIZE]; /* filename for U ascii output file */
  char               VoutFn[FNSIZE]; /* filename for V ascii output file */
  char               LatentOvrlyFn[FNSIZE];/* filename for byte latency ovly */
  char               DomOvrlyFn[FNSIZE];/* filename for DomComp ovrly */

 /*
  * Check for right number of arguments
  */
  if (argc != 2) {
    printf("\n\n%s: extract dominant frequencies from fingerprint latency.",
           argv[0]);
    printf("\nusage: %s ParameterFileName\n",argv[0]);
    exit(-1);
  }
#ifdef DEBUG
 /*
  * Link to PA so we can use it in the debugger 
  */
  if (!argc) {
    PA((void *)X, 0, -1, 'b');
    PRINTA((void *)X,0,-1,'b',"delete.me");
    WRITEA((void *)X,0,-1,'b',"delete.me");
    image2disk((void *)X,1,1,'*',"delete.me");
    SpecMag2disk(X,"delete.me");
    PC(X,0,0,-1,'b',0);
    PRINTC(X,0,0,-1,'b',0,"delete.me");
    WRITEC(X,0,0,-1,'b',0,"delete.me");
  }
#endif

 /*
  * Read parameters file
  */
  ReadParametersFile(argv[1],&size,&ULrow,&ULcol,&LRrow,&LRcol,Ufn,Vfn,LatentFn,
                     DomCompRfn,UoutFn,VoutFn,LatentOvrlyFn,DomOvrlyFn);

  ROIrows = LRrow - ULrow + 1;
  ROIcols = LRcol - ULcol + 1;
  ROIsize = 2*ROIrows + 2*ROIcols - 4;
  n = size * size;

 /*
  * Allocate working arrays
  */
  if ((U = (float *)malloc(n * sizeof(float))) == NULL) {
    printf("\n%s: free store exhausted.\n",argv[0]);
    exit(-1);
  }
  if ((V = (float *)malloc(n * sizeof(float))) == NULL) {
    printf("\n%s: free store exhausted.\n",argv[0]);
    exit(-1);
  }
  if ((Latent = (float *)malloc(n * sizeof(float))) == NULL) {
    printf("\n%s: free store exhausted.\n",argv[0]);
    exit(-1);
  }
  if ((DomCompR = (float *)malloc(n * sizeof(float))) == NULL) {
    printf("\n%s: free store exhausted.\n",argv[0]);
    exit(-1);
  }
  if ((Uout = (float *)malloc(ROIsize * sizeof(float))) == NULL) {
    printf("\n%s: free store exhausted.\n",argv[0]);
    exit(-1);
  }
  if ((Vout = (float *)malloc(ROIsize * sizeof(float))) == NULL) {
    printf("\n%s: free store exhausted.\n",argv[0]);
    exit(-1);
  }

 /*
  * Read input files
  */
  disk2f(U,size,size,Ufn);
  disk2f(V,size,size,Vfn);
  disk2f(Latent,size,size,LatentFn);
  disk2f(DomCompR,size,size,DomCompRfn);

 /*
  * Make byte copies of the Latent and DomComp images for overlay outputs
  */
  LatentOvrly = f_dbx(Latent,size,size);
  DomOvrly = f_dbx(DomCompR,size,size);
  free(Latent);
  free(DomCompR);
  for (i=0; i < n; i++) {
    LatentOvrly[i] >>= 1;
    DomOvrly[i] >>= 1;
  }
  

 /*
  * Extract desired frequencies and make float overlay output images
  */
  /* part I: across the top of the ROI */
  row = ULrow;
  col = ULcol;
  k_image = row * size + col;
  k_out = 0;
  for (; col <= LRcol; col++,k_image++,k_out++) {
    Uout[k_out] = U[k_image];
    Vout[k_out] = V[k_image];
    LatentOvrly[k_image] = DomOvrly[k_image] = 255;
  }
  /* part II: down the right side of the ROI */
  for (row++,col--,k_image+=size-1; row <= LRrow; row++,k_out++,k_image+=size) {
    Uout[k_out] = U[k_image];
    Vout[k_out] = V[k_image];
    LatentOvrly[k_image] = DomOvrly[k_image] = 255;
  }
  /* part III: backwards across the bottom row of the ROI */
  for (row--,col--,k_image-=size+1; col >= ULcol; col--,k_out++,k_image--) {
    Uout[k_out] = U[k_image];
    Vout[k_out] = V[k_image];
    LatentOvrly[k_image] = DomOvrly[k_image] = 255;
  }
  /* part IV: up the left side of the ROI */
  for (row--,col++,k_image-=size-1; row > ULrow; row--,k_out++,k_image-=size) {
    Uout[k_out] = U[k_image];
    Vout[k_out] = V[k_image];
    LatentOvrly[k_image] = DomOvrly[k_image] = 255;
  }

 /*
  * Write output files
  */
  if (strcmp(UoutFn,"none")) {
    PRINTA(Uout,0,ROIsize-1,'f',UoutFn);
  }
  if (strcmp(VoutFn,"none")) {
    PRINTA(Vout,0,ROIsize-1,'f',VoutFn);
  }
  if (strcmp(LatentOvrlyFn,"none")) {
    byte2disk(LatentOvrly,size,size,LatentOvrlyFn);
  }
  if (strcmp(DomOvrlyFn,"none")) {
    byte2disk(DomOvrly,size,size,DomOvrlyFn);
  }
 
  printf("\n\n");
  return;

} /*-------------------------------------------- main --------------*/

/*
 * ReadParametersFile
 *
 *  read the data out of the parameters file
 *
 */

void ReadParametersFile(ParmFn,psize,pULrow,pULcol,pLRrow,pLRcol,Ufn,Vfn,
                        LatentFn,DomCompRfn,UoutFn,VoutFn,LatentOvrlyFn,
                        DomOvrlyFn)
  char   *ParmFn;
  int    *psize;
  int    *pULrow;
  int    *pULcol;
  int    *pLRrow;
  int    *pLRcol;
  char   *Ufn;
  char   *Vfn;
  char   *LatentFn;
  char   *DomCompRfn;
  char   *UoutFn;
  char   *VoutFn;
  char   *LatentOvrlyFn;
  char   *DomOvrlyFn;
{

  FILE      *parmfp;            /* ptr to parameter file */
  char       instr[256];        /* buffer to read into  */

  if ((parmfp = fopen(ParmFn,"r")) == NULL) {
    printf("\n\nReadParametersFile: unable to open %s\n",ParmFn);
    exit(-1);
  }

  fscanf(parmfp,"Size= %d\n",psize);
  fscanf(parmfp,"ULrow= %d\n",pULrow);
  fscanf(parmfp,"ULcol= %d\n",pULcol);
  fscanf(parmfp,"LRrow= %d\n",pLRrow);
  fscanf(parmfp,"LRcol= %d\n",pLRcol);
  fscanf(parmfp,"U File= %s\n",Ufn);
  fscanf(parmfp,"V File= %s\n",Vfn);
  fscanf(parmfp,"LatentFile= %s\n",LatentFn);
  fscanf(parmfp,"DomFile= %s\n",DomCompRfn);
  fscanf(parmfp,"UoutFile= %s\n",UoutFn);
  fscanf(parmfp,"VoutFile= %s\n",VoutFn);
  fscanf(parmfp,"LatentOvrly= %s\n",LatentOvrlyFn);
  fscanf(parmfp,"DomOvrly= %s\n",DomOvrlyFn);
  fclose(parmfp);
  return;
} /*--------------------------- ReadParametersFile -------------------------*/

#endif
