/*
 * trackb.c
 *
 *  Do component tracking and put out data for reconstruction.  
 *  This program copied from anchovy:/export/home/joebob/track/src/track.c
 *  
 *  usage: trackb ParmFn
 *
 *   where ParmFn is the name of the file containing the run parameters
 *
 *  2/29/96  MAJOR REVISIONS
 *
 *
 *  jph 12/6/94
 */

#ifndef TRACK_C
#define TRACK_C

#include "track.h"


main(argc,argv)

  int   argc;
  char *argv[];

{

/*-----------------------------------------------------------------------------
  Main Variable Frame
-----------------------------------------------------------------------------*/
  int             row;                /* image row counter */
  int             col;                /* image col counter */
  int             i;                  /* counter */
  int             j;                  /* counter */
  int             k;                  /* counter */
  int             jt;                 /* joe's track */
  int             OrigSize;           /* original row/col dim of image */
  int             Xsize;              /* row/col dim of possibly ZP'd image */
  int             ReadSize;           /* size of a row of obs for A,U, or V */
  int             minRow2Track;       /* first row to perform track proc. on */
  int             MaxRow2Track;       /* last  row to perform track proc. on */
  int             minCol2Track;       /* first col to perform track proc. on */
  int             MaxCol2Track;       /* last  col to perform track proc. on */
  int             LimitRays;          /* 1=limit num rays visited by a track */
  int             fScale;             /* 1=scale track start criterion by freq*/
  int             TEScale;            /* 1=scale trkstrt crit by Teager Energy*/
  int             delta;              /* -1 if proc col R to L, 1 otherwise */
  int             ZeroPad;            /* 1 if zero padding is to be used */
  int             BCinterval;         /* Bound Cond Spacing for Reconstruction*/
  int            *ChannelStatus;      /* track assoc status of channel */
  int            *Qorder;             /* no.s of channels sorted by Q */
  float          *ChannelQ;           /* channel quality */
  float          *ChanQcopy;          /* copy of channel quality for sorting */
  pABgain         KA;                 /* gains for A tracker */
  pABgain         KU;                 /* gains for U tracker */
  pABgain         KV;                 /* gains for V tracker */
  float          *Aob;                /* row of observations of A */
  float          *Uob;                /* row of observations of u */
  float          *Vob;                /* row of observations of v */
  float          *Alast;              /* last row of observations of A */
  float          *Ulast;              /* last row of observations of u */
  float          *Vlast;              /* last row of observations of v */
  filterbank     *G;                  /* the filter bank */
  pFloatImage     X;                  /* input signal */
  track_t         T[MAXNTRACK];       /* tracks */
  image_point    *LastStart;          /* for each chan, pix of last trackstart*/
  int             fdA;                /* file descriptor for A obs file */
  int             fdU;                /* file descriptor for U obs file */
  int             fdV;                /* file descriptor for V obs file */
  char            TARootFn[FNSIZE];   /* root fn for tracked amplitudes */
  char            TURootFn[FNSIZE];   /* root fn for tracked horiz freq */
  char            TVRootFn[FNSIZE];   /* root fn for tracked vert  freq */
  char            BCRootFn[FNSIZE];   /* root fn for track boundary cond's */
  char            RptFn[FNSIZE];      /* ascii track report filename */
  FILE           *RptFp;              /* ptr to stream for track reporting */
  
/*-----------------------------------------------------------------------------
  MAIN ROUTINE
-----------------------------------------------------------------------------*/
#ifdef DEBUG
 /*
  * Link to my debugger library routines
  */
  if (!argc) {
    PA((void *)G, 0, -1, 'b');
    PRINTA((void *)G,0,-1,'b',"delete.me");
    WRITEA((void *)G,0,-1,'b',"delete.me");
    image2disk((void *)G,1,1,'*',"delete.me");
    SpecMag2disk((void *)G,"delete.me");
    PC((void *)G,0,0,-1,'b',0);
    PRINTC((void *)G,0,0,-1,'b',0,"delete.me");
    WRITEC((void *)G,0,0,-1,'b',0,"delete.me");
  }
#endif

 /*
  * Check for right number of arguments
  */
  if (argc != 2) {
    printf("\n\n%s: AM-FM multi-component tracker with reconstruction data.\n",
                 argv[0]);
    printf("\nusage: %s ParmFileName\n", argv[0]);
    exit(-1);
  }
  jt = 1;

 /*
  * Initialize
  */
  X = (pFloatImage)malloc(sizeof(FloatImage));
  bInitialize(argv[0],argv[1],&OrigSize,&Xsize,&ZeroPad,&BCinterval,&X->Real,
              &X->Imag,&X->FFTReal,&X->FFTImag,&G,&ReadSize,&KA,&KU,&KV,&Aob,
              &Uob,&Vob,&fdA,&fdU,&fdV,T,&minRow2Track,&MaxRow2Track,
              &minCol2Track,&MaxCol2Track,&LimitRays,&fScale,&TEScale,TARootFn,
              TURootFn,TVRootFn,BCRootFn,RptFn,&RptFp);
  X->size = X->cols = X->rows = Xsize;
  X->square = 1;

 /*
  * Allocate working arrays
  */
  AllocWAb(argv[0],G,ReadSize,&ChannelStatus,&Qorder,&ChannelQ,&ChanQcopy,
           &LastStart,&Alast,&Ulast,&Vlast);

 /*==================================================================*/
 /* MAIN LOOP                                                        */
 /*==================================================================*/
 /*
  * Loop through the image in the 1D linear scan order p. 73 book VII.
  */
  for (row=1; row < minRow2Track; row++) {
    ReadNextRowObservations(argv[0],&Aob,&Uob,&Vob,&Alast,&Ulast,&Vlast,fdA,
                            fdU,fdV,ReadSize);
  }
  for (; row <= MaxRow2Track; row++) {
   /*
    * Read next row of observations, loop through tracking each column
    */ 
    ReadNextRowObservations(argv[0],&Aob,&Uob,&Vob,&Alast,&Ulast,&Vlast,fdA,
                            fdU,fdV,ReadSize);
    for (col=minCol2Track,delta=1; col <= MaxCol2Track; col++) {
      Track(jt,row,col,delta,minCol2Track,MaxCol2Track,T,ChannelQ,G,Aob,Uob,Vob,
            ChannelStatus,Qorder,ChanQcopy,OrigSize,LastStart,Alast,Ulast,
            Vlast,KA,KU,KV,minRow2Track,MaxRow2Track,LimitRays,fScale,TEScale,
            RptFp);
    }    /* for col */
    if (++row <= MaxRow2Track) {
     /*
      * Read next row of observations, loop through tracking each column
      */ 
      ReadNextRowObservations(argv[0],&Aob,&Uob,&Vob,&Alast,&Ulast,&Vlast,fdA,
                              fdU,fdV,ReadSize);
      for (col=MaxCol2Track,delta=-1; col >= minCol2Track; col--) {
        Track(jt,row,col,delta,minCol2Track,MaxCol2Track,T,ChannelQ,G,Aob,Uob,
              Vob,ChannelStatus,Qorder,ChanQcopy,OrigSize,LastStart,Alast,
              Ulast,Vlast,KA,KU,KV,minRow2Track,MaxRow2Track,LimitRays,fScale,
              TEScale,RptFp);
      }    /* for col */
    }      /* if ++row < size */
  }        /* for row */

 /*
  * Close the observation files
  */
  if (close(fdA) == -1) printf("\n%s: error closing Aobs file.\n",argv[0]);
  if (close(fdU) == -1) printf("\n%s: error closing Uobs file.\n",argv[0]);
  if (close(fdV) == -1) printf("\n%s: error closing Vobs file.\n",argv[0]);

 /*
  * Free Working Arrays
  */
  FreeWAb(&KA,&KU,&KV,&Aob,&Uob,&Vob,&Alast,&Ulast,&Vlast,&ChannelStatus,
          &Qorder,&ChannelQ,&ChanQcopy,&LastStart);
 /*
  * Report and output boundary conditions for reconstruction
  */
  Epilogue(argv[0],OrigSize,ZeroPad,G,T,BCinterval,minRow2Track,MaxRow2Track,
           minCol2Track,MaxCol2Track,X,TARootFn,TURootFn,TVRootFn,BCRootFn,
           RptFn);

  printf("\n\n");
  exit(0);
}/*--------------------------------------------  main  ----------------------*/
#endif
