
/*
 * 1imageTo1.c
 *
 *   Read 1 byte image and make them into one (big one).  
 *
 * jph 4/17/96
 *
 */

#include <stdio.h>
#include <math.h>
#include <fcntl.h>
#include "teager2D.h"

main(argc,argv)

  int   argc;
  char *argv[];

{


  BYTE   *x1;                /* input image 1 */
  BYTE   *y;                 /* output image */
  char   *fn1;               /* filename for x1 */
  char   *outfn;             /* filename for output image */
  int     x1rows;            /* num rows in x1 */
  int     x1cols;            /* num cols in x1 */
  int     Nx1;               /* num pixels in x1 */
  int     yrows;             /* row dim of output image */
  int     ycols;             /* col dim of output image */
  int     Ny;                /* num pixels in output image */
  int     x1startrow;        /* starting row for image 1 */
  int     x1startcol;        /* starting col for image 1 */
  int     i;                 /* counter */
  int     j;                 /* counter */
  int     ii;                /* counter */
  int     jj;                /* counter */
  int     bkgrnd;            /* background value */
 
 /*
  * Check for right number of command line arguments
  */
  if (argc !=  8) {
    printf("\n\n%s: Turn one byte images into one big one.",argv[0]);
    printf("\nusage: %s fn1 rows cols outfn rows cols bkgrnd\n",argv[0]);
    return;
  }

 /*
  * parse args
  */
  fn1 = argv[1];
  x1rows = atoi(argv[2]);
  x1cols = atoi(argv[3]);
  outfn = argv[4];
  yrows = atoi(argv[5]);
  ycols = atoi(argv[6]);
  bkgrnd = atoi(argv[7]);
  
  Nx1 = x1rows * x1cols;
  Ny = yrows * ycols;

 /*
  * Allocate
  */
  if ((x1 = (BYTE *)malloc(x1rows*x1cols*sizeof(BYTE))) == NULL) {
    printf("\n%s: free store exhausted.\n",argv[0]);
    exit(-1);
  }
  if ((y = (BYTE *)malloc(yrows*ycols*sizeof(BYTE))) == NULL) {
    printf("\n%s: free store exhausted.\n",argv[0]);
    exit(-1);
  }

 /*
  * Initialize output image
  */
  for (i=0; i < Ny; i++) {
    y[i] = (BYTE)bkgrnd;
  }

 /*
  * Compute starting row/col for each input imgage
  */
  x1startcol = (ycols - x1cols) / 2 + 1;
  x1startrow = (yrows - x1rows) / 2 + 1;

 /*
  * get input images
  */
  disk2byte(x1,x1rows,x1cols,fn1);

 /*
  * Expand dynamic range
  */
  dbx(x1,x1rows,x1cols);

 /*
  * Put first input image into output image
  */
  for (i=0,ii=x1startrow; i < x1rows; i++,ii++) {
    for (j=0,jj=x1startcol; j < x1cols; j++,jj++) {
      y[ii*ycols + jj] = x1[i*x1cols + j];
    }
  }

 /*
  * write output image
  */
  byte2disk(y,yrows,ycols,outfn);

  printf("\n");
  return;
} 
