
/*
 * 2imageTo1.c
 *
 *   Read 2 byte images and make them into one.  
 *
 * jph 4/17/96
 *
 */

#include <stdio.h>
#include <math.h>
#include <fcntl.h>
#include "teager2D.h"

main(argc,argv)

  int   argc;
  char *argv[];

{


  BYTE   *x1;                /* input image 1 */
  BYTE   *x2;                /* input image 2 */
  BYTE   *y;                 /* output image */
  char   *fn1;               /* filename for x1 */
  char   *fn2;               /* filename for x2 */
  char   *outfn;             /* filename for output image */
  int     x1rows;            /* num rows in x1 */
  int     x1cols;            /* num cols in x1 */
  int     Nx1;               /* num pixels in x1 */
  int     x2rows;            /* num rows in x2 */
  int     x2cols;            /* num cols in x2 */
  int     Nx2;               /* num pixels in x2 */
  int     yrows;             /* row dim of output image */
  int     ycols;             /* col dim of output image */
  int     Ny;                /* num pixels in output image */
  int     MaxSize;           /* max num rows or cols in an image */
  int     gap;               /* nominal gap value */
  int     midcol;            /* middle col of output image */
  int     hgap1;             /* horiz gap to left of first image */
  int     hgap2;             /* horiz gap between images */
  int     x1startrow;        /* starting row for image 1 */
  int     x1startcol;        /* starting col for image 1 */
  int     x2startrow;        /* starting row for image 2 */
  int     x2startcol;        /* starting col for image 2 */
  int     i;                 /* counter */
  int     j;                 /* counter */
  int     ii;                /* counter */
  int     jj;                /* counter */
  int     bkgrnd;            /* background value */
 
 /*
  * Check for right number of command line arguments
  */
  if (argc !=  11) {
    printf("\n\n%s: Turn two byte images into one.",argv[0]);
    printf("\nusage: %s fn1 rows cols fn2 rows cols outfn rows cols bkgrnd\n",
           argv[0]);
    return;
  }

 /*
  * parse args
  */
  fn1 = argv[1];
  x1rows = atoi(argv[2]);
  x1cols = atoi(argv[3]);
  fn2 = argv[4];
  x2rows = atoi(argv[5]);
  x2cols = atoi(argv[6]);
  outfn = argv[7];
  yrows = atoi(argv[8]);
  ycols = atoi(argv[9]);
  bkgrnd = atoi(argv[10]);
  
  Nx1 = x1rows * x1cols;
  Nx2 = x2rows * x2cols;
  Ny = yrows * ycols;

 /*
  * Allocate
  */
  if ((x1 = (BYTE *)malloc(x1rows*x1cols*sizeof(BYTE))) == NULL) {
    printf("\n%s: free store exhausted.\n",argv[0]);
    exit(-1);
  }
  if ((x2 = (BYTE *)malloc(x2rows*x2cols*sizeof(BYTE))) == NULL) {
    printf("\n%s: free store exhausted.\n",argv[0]);
    exit(-1);
  }
  if ((y = (BYTE *)malloc(yrows*ycols*sizeof(BYTE))) == NULL) {
    printf("\n%s: free store exhausted.\n",argv[0]);
    exit(-1);
  }

 /*
  * Initialize output image
  */
  for (i=0; i < Ny; i++) {
    y[i] = (BYTE)bkgrnd;
  }

 /*
  * Compute starting row/col for each input imgage
  */
  midcol = ycols / 2;
  MaxSize = x1cols;
  if (x2cols > MaxSize) MaxSize = x2cols;
  if (x1rows > MaxSize) MaxSize = x1rows;
  if (x2rows > MaxSize) MaxSize = x2rows;

  gap = (ycols - x1cols - x2cols) / 10;

  hgap1 = gap + (MaxSize - x1cols) / 2;
  hgap2 = gap + (MaxSize - x2cols) / 2;

  x1startcol = midcol - hgap1 - x1cols;
  x2startcol = midcol + hgap2 + 1;

  x1startrow = (yrows - x1rows) / 2 + 1;
  x2startrow = (yrows - x2rows) / 2 + 1;
  
 /*
  * get input images
  */
  disk2byte(x1,x1rows,x1cols,fn1);
  disk2byte(x2,x2rows,x2cols,fn2);

 /*
  * Expand dynamic range
  */
  dbx(x1,x1rows,x1cols);
  dbx(x2,x2rows,x2cols);

 /*
  * Put first input image into output image
  */
  for (i=0,ii=x1startrow; i < x1rows; i++,ii++) {
    for (j=0,jj=x1startcol; j < x1cols; j++,jj++) {
      y[ii*ycols + jj] = x1[i*x1cols + j];
    }
  }

 /*
  * Put second input image into output image
  */
  for (i=0,ii=x2startrow; i < x2rows; i++,ii++) {
    for (j=0,jj=x2startcol; j < x2cols; j++,jj++) {
      y[ii*ycols + jj] = x2[i*x2cols + j];
    }
  }

 /*
  * write output image
  */
  byte2disk(y,yrows,ycols,outfn);

  printf("\n");
  return;
} 
