
/*
 * 3imageTo1.c
 *
 *   Read 3 byte images and make them into one.  
 *
 * jph 4/17/96
 *
 */

#include <stdio.h>
#include <math.h>
#include <fcntl.h>
#include "teager2D.h"

main(argc,argv)

  int   argc;
  char *argv[];

{


  BYTE   *x1;                /* input image 1 */
  BYTE   *x2;                /* input image 2 */
  BYTE   *x3;                /* input image 3 */
  BYTE   *y;                 /* output image */
  char   *fn1;               /* filename for x1 */
  char   *fn2;               /* filename for x2 */
  char   *fn3;               /* filename for x2 */
  char   *outfn;             /* filename for output image */
  int     x1rows;            /* num rows in x1 */
  int     x1cols;            /* num cols in x1 */
  int     Nx1;               /* num pixels in x1 */
  int     x2rows;            /* num rows in x2 */
  int     x2cols;            /* num cols in x2 */
  int     Nx2;               /* num pixels in x2 */
  int     x3rows;            /* num rows in x3 */
  int     x3cols;            /* num cols in x3 */
  int     Nx3;               /* num pixels in x3 */
  int     yrows;             /* row dim of output image */
  int     ycols;             /* col dim of output image */
  int     Ny;                /* num pixels in output image */
  int     MaxSize;           /* max num rows or cols in an image */
  int     gap;               /* nominal gap value */
  int     hgap1;             /* horiz gap from midcol to image 1 */
  int     hgap2;             /* horiz gap from midcol to image 2 */
  int     hgap3;             /* horiz gap from left edge to image 3 */
  int     vgap1;             /* vert gap from midrow to image 1 */
  int     vgap2;             /* vert gap from midrow to image 2 */
  int     vgap3;             /* vert gap from midrow to image 3 */
  int     x1startrow;        /* starting row for image 1 */
  int     x1startcol;        /* starting col for image 1 */
  int     x2startrow;        /* starting row for image 2 */
  int     x2startcol;        /* starting col for image 2 */
  int     x3startrow;        /* starting row for image 3 */
  int     x3startcol;        /* starting col for image 3 */
  int     midrow;            /* middle row of output image */
  int     midcol;            /* middle col of output image */
  int     i;                 /* counter */
  int     j;                 /* counter */
  int     ii;                /* counter */
  int     jj;                /* counter */
  int     bkgrnd;            /* background value */
 
 /*
  * Check for right number of command line arguments
  */
  if (argc !=  14) {
    printf("\n\n%s: Turn three byte images into one.",argv[0]);
    printf("\nusage: %s %s\n",argv[0],
           "fn1 rows cols fn2 rows cols fn3 rows cols outfn rows cols bkgrnd");
    return;
  }

 /*
  * parse args
  */
  fn1 = argv[1];
  x1rows = atoi(argv[2]);
  x1cols = atoi(argv[3]);
  fn2 = argv[4];
  x2rows = atoi(argv[5]);
  x2cols = atoi(argv[6]);
  fn3 = argv[7];
  x3rows = atoi(argv[8]);
  x3cols = atoi(argv[9]);
  outfn = argv[10];
  yrows = atoi(argv[11]);
  ycols = atoi(argv[12]);
  bkgrnd = atoi(argv[13]);
  
  Nx1 = x1rows * x1cols;
  Nx2 = x2rows * x2cols;
  Nx3 = x3rows * x3cols;
  Ny = yrows * ycols;

 /*
  * Allocate
  */
  if ((x1 = (BYTE *)malloc(Nx1*sizeof(BYTE))) == NULL) {
    printf("\n%s: free store exhausted.\n",argv[0]);
    exit(-1);
  }
  if ((x2 = (BYTE *)malloc(Nx2*sizeof(BYTE))) == NULL) {
    printf("\n%s: free store exhausted.\n",argv[0]);
    exit(-1);
  }
  if ((x3 = (BYTE *)malloc(Nx3*sizeof(BYTE))) == NULL) {
    printf("\n%s: free store exhausted.\n",argv[0]);
    exit(-1);
  }
  if ((y = (BYTE *)malloc(Ny*sizeof(BYTE))) == NULL) {
    printf("\n%s: free store exhausted.\n",argv[0]);
    exit(-1);
  }

 /*
  * Initialize output image
  */
  for (i=0; i < Ny; i++) {
    y[i] = (BYTE)bkgrnd;
  }

 /*
  * Compute starting row/col for each input imgage
  */
  midrow = yrows / 2;
  midcol = ycols / 2;
  MaxSize = x1cols;
  if (x2cols > MaxSize) MaxSize = x2cols;
  if (x3cols > MaxSize) MaxSize = x3cols;
  if (x1rows > MaxSize) MaxSize = x1rows;
  if (x2rows > MaxSize) MaxSize = x2rows;
  if (x3rows > MaxSize) MaxSize = x3rows;

  gap = MaxSize / 8;

  hgap1 = gap + (MaxSize - x1cols) / 2;
  hgap2 = gap + (MaxSize - x2cols) / 2;
  hgap3 = (ycols - x3cols) / 2;

  vgap1 = gap + (MaxSize - x1rows) / 2;
  vgap2 = gap + (MaxSize - x2rows) / 2;
  vgap3 = gap + (MaxSize - x3rows) / 2;

  x1startcol = midcol - hgap1 - x1cols;
  x2startcol = midcol + hgap2 + 1;
  x3startcol = hgap3 + 1;

  x1startrow = midrow - vgap1 - x1rows;
  x2startrow = midrow - vgap2 - x2rows;
  x3startrow = midrow + vgap3 + 1;

 /*
  * get input images
  */
  disk2byte(x1,x1rows,x1cols,fn1);
  disk2byte(x2,x2rows,x2cols,fn2);
  disk2byte(x3,x3rows,x3cols,fn3);

 /*
  * Expand dynamic range
  */
  dbx(x1,x1rows,x1cols);  
  dbx(x2,x2rows,x2cols);  
  dbx(x3,x3rows,x3cols);  

 /*
  * Put first input image into output image
  */
  for (i=0,ii=x1startrow; i < x1rows; i++,ii++) {
    for (j=0,jj=x1startcol; j < x1cols; j++,jj++) {
      y[ii*ycols + jj] = x1[i*x1cols + j];
    }
  }

 /*
  * Put second input image into output image
  */
  for (i=0,ii=x2startrow; i < x2rows; i++,ii++) {
    for (j=0,jj=x2startcol; j < x2cols; j++,jj++) {
      y[ii*ycols + jj] = x2[i*x2cols + j];
    }
  }

 /*
  * Put third input image into output image
  */
  for (i=0,ii=x3startrow; i < x3rows; i++,ii++) {
    for (j=0,jj=x3startcol; j < x3cols; j++,jj++) {
      y[ii*ycols + jj] = x3[i*x3cols + j];
    }
  }

 /*
  * write output image
  */
  byte2disk(y,yrows,ycols,outfn);

  printf("\n");
  return;
} 
