/*
 * BumpConvUnit.c
 *
 *   This program converts the units of both x and y for data files made by
 *   PB-2.c.  See book XI, pp. 94-101.  In the data files, the x-axis units
 *   are seconds and the y-axis units are radians/second.  This program
 *   converts the x-axis units to samples and the y-axis units to 
 *   cycles/sample.
 *
 * 7/2/97 jph
 *
 */

#ifndef BUMPCONVUNIT_C
#define BUMPCONVUNIT_C
#include <stdio.h>
#include <math.h>
#include <fcntl.h>
#include "teager2D.h"

main(argc,argv)

  int   argc;
  char *argv[];

{

  float *xIn;           /* input signal x-axis values (original units) */
  float *yIn;           /* input signal y-axis values (original units) */
  float *xOut;          /* output signal x-axis values (converted units) */
  float *yOut;          /* output signal y-axis values (converted units) */
  float  Kx;            /* conversion factor for x-axis data */
  float  Ky;            /* conversion factor for y-axis data */
  int    size;          /* number of data points */
  int    i;             /* counter */
  FILE  *infile;        /* input file pointer */
  char  *infn;          /* input filename */
  FILE  *outfile;       /* output file pointer */
  char  *outfn;         /* output filename */
 
 /*
  * Parse args
  */
  if (argc !=  4) {
    printf("\n\n%s: convert phase bump data units from radians to samples",
           argv[0]);
    printf("\n\nusage: %s size infile outfile\n\n",argv[0]);
    return;
  }
  size = atoi(argv[1]);
  infn = argv[2];
  outfn = argv[3];

 /*
  * Allocate arrays
  */
  if ((xIn  = (float *) malloc(size*sizeof(float))) == NULL) {
    printf("\n\n%s: free store exhausted.\n",argv[0]);
    exit(-1);
  }
  if ((yIn  = (float *) malloc(size*sizeof(float))) == NULL) {
    printf("\n\n%s: free store exhausted.\n",argv[0]);
    exit(-1);
  }
  if ((xOut = (float *) malloc(size*sizeof(float))) == NULL) {
    printf("\n\n%s: free store exhausted.\n",argv[0]);
    exit(-1);
  }
  if ((yOut = (float *) malloc(size*sizeof(float))) == NULL) {
    printf("\n\n%s: free store exhausted.\n",argv[0]);
    exit(-1);
  }
 
 /*
  * Read input file
  */
  if ((infile = fopen(infn,"r")) == NULL) {
    printf("\nCannot open file %s",infn);
    exit(0);
  }
  for (i=0; i<size; i++) {
    fscanf(infile,"%f\t%f\n",&xIn[i],&yIn[i]);
  }
  fclose(infile);

 /*
  * Convert units of data
  */
  Kx = (float)20.0;
  Ky = (float)1.0/((float)40.0 * (float)M_PI);
  for (i=0; i < size; i++) {
    xOut[i] = xIn[i] * Kx;
    yOut[i] = yIn[i] * Ky;
  }

 /*
  * Write output file
  */
  if ((outfile = fopen(outfn,"w")) == NULL) {
    printf("\nCannot open file %s",outfn);
    exit(0);
  }
  for (i=0; i < size; i++) {
    fprintf(outfile,"%f\t%f\n",xOut[i],yOut[i]);
  }
  fclose(outfile);

  return;
} 
#endif
