
/*
 * ExpSmTest.c
 *
 * Run exponential smoothing on a lucent data file, 13-week forecasts.
 *
 * jph 8/23/99
 *
 */

#ifndef EXPSMTEST
#define EXPSMTEST
#include <stdio.h>
#include <math.h>
#include <fcntl.h>
#include "teager2D.h"

/*
 * Function prototypes
 */
char RLReadLine();




main(argc,argv)

  int   argc;
  char *argv[];

{


  float     alpha;             /* exponential smoother parameter */
  float     One_m_alpha;       /* 1 - alpha */
  char     *InFn;              /* input filename */
  char     *OutFn;             /* output filename */
  FILE     *InFile;            /* input  file pointer */
  int       size;              /* number of samples in signal */
  char      achar;             /* temp char variable */
  char      InString[FNSIZE];  /* input string buffer */
  int      *Week;              /* array of week codes */
  int      *ActOrds;           /* actual orders data */
  int       i;                 /* loop counter */
  int       j;                 /* loop counter */
  int       OutSize;           /* size of output forecast array */
  float    *Fcst;              /* floating point forecasts */
  int      *iFcst;             /* integer forecasts */
  FILE     *OutFile;           /* output file pointer */
 
 /*
  * Parse command line arguments
  */
  if (argc !=  4) {
    printf("\nRun exponential smoothing on a lucent data file, 13 Wk Fcst.\n");
    printf("\n\nusage: %s alpha infile outfile\n",argv[0]);
    return;
  }
  alpha = atof(argv[1]);
  One_m_alpha = (float)1.0 - alpha;
  InFn = argv[2];
  OutFn = argv[3];

 /*
  * Open input file.
  */
  if ((InFile = fopen(InFn,"r")) == NULL) {
    printf("\n\n%s: unable to open %s\n",argv[0],InFn);
    exit(-1);
  }

 /*
  * Count lines in input file.
  */
  size = 0;
  while ((achar = RLReadLine(InFile,InString)) != EOF) {
    size++;
  }

 /*
  * Allocate input arrays.
  */
  if((Week  = (int *) malloc(size*sizeof(int))) == NULL) {
    printf("\n%s: free store exhausted.\n",argv[0]);
    exit(-1);
  }
  if((ActOrds  = (int *) malloc(size*sizeof(int))) == NULL) {
    printf("\n%s: free store exhausted.\n",argv[0]);
    exit(-1);
  }

 /*
  * Read the input file
  */
  rewind(InFile);
  for (i=0; i<size; i++) {
    fscanf(InFile,"%d\t%d\n",Week+i,ActOrds+i);
  }
  fclose(InFile);

 /*
  * Allocate output arrays.
  */
  OutSize = size + 13;
  if((Fcst  = (float *) malloc(OutSize*sizeof(float))) == NULL) {
    printf("\n%s: free store exhausted.\n",argv[0]);
    exit(-1);
  }
  if((iFcst  = (int *) malloc(OutSize*sizeof(int))) == NULL) {
    printf("\n%s: free store exhausted.\n",argv[0]);
    exit(-1);
  }

 /*
  * Make forecasts
  */
  for (j=0; j < 13; j++) {
    Fcst[j] = (float)-1.0;
  }
  Fcst[13] = (float)ActOrds[0] * alpha;

  for (i=1,j++; i < size; i++,j++) {
    Fcst[j] = One_m_alpha * Fcst[j-1] + alpha * (float)ActOrds[i];
  }

 /*
  * Convert forecasts to int
  */
  for (i=0; i < 13; i++) {
    iFcst[i] = -1;
  }
  for (i=13; i < OutSize; i++) {
    iFcst[i] = (int)(Fcst[i] + 0.5);
  }

 /*
  * Write the output file 
  */
  if ((OutFile = fopen(OutFn,"w")) == NULL) {
    printf("\n%s: cannot open file %s\n",argv[0],OutFn);
    exit(-1);
  }
  for (i=0; i < size; i++) {
    fprintf(OutFile,"%d\t%d\n",Week[i],iFcst[i]);
  }
  for ( ; i < OutSize; i++) {
    fprintf(OutFile,"%d\t%d\n",++Week[size-1],iFcst[i]);
  }
  fclose(OutFile);

  return;
} 
/*--------------------------------------------------------------------------*/
/*
 * RLReadLine.c
 *
 *  Read one line of a Lucent data file.
 *
 *  The first character is read with getc().
 *  If this is not EOF, the rest of the line is read into InString[1].
 *  The first character is then copied into InString[0].
 *  The first character is returned for easy detection of EOF.
 *
 *  11/13/97 jph
 *
 */

char RLReadLine(InFile,InString)

  FILE         *InFile;             /* input data file ptr */
  char          InString[FNSIZE];   /* input string buffer */

{

  char          Ret;                /* return value */

  Ret = getc(InFile);
  if (Ret != EOF) {
    fscanf(InFile,"%[^\n]\n",&InString[1]);
  }
  InString[0] = Ret;

  return(Ret);
} /*--------------------------- RLReadLine ---------------------------------*/
#endif
