
/*
 * IBWsep.c
 *
 *  Computes the ratio of the magnitude instantaneous frequency separation to 
 *  the magnitude of the IBW for two image components.  This ratio should
 *  be large if the multicomponent interpretation is valid.
 *
 *  usage:  IBWsep size A1fn U1fn V1fn A2fn U2fn V2fn outfn1 outfn2
 * 
 * 8/23/96 jph
 *
 */

#ifndef IBWSEP_C
#define IBWSEP_C
#include <stdio.h>
#include <math.h>
#include <fcntl.h>
#include "teager2D.h"

main(argc,argv)

  int   argc;
  char *argv[];

{


  float  *a1;          /* amplitude of first component */
  float  *a2;          /* amplitude of second component*/
  float  *u1;          /* horiz freq of first component */
  float  *u2;          /* horiz freq of second component */
  float  *v1;          /* vert freq of first component */
  float  *v2;          /* vert freq of second component */
  double *aa1;         /* double version of a1 */
  double *aa2;         /* double version of a2 */
  double *ddja1;       /* d/dj a1 */
  double *ddia1;       /* d/di a1 */
  double *ddja2;       /* d/d2 a1 */
  double *ddia2;       /* d/d2 a1 */
  double *Magfdiff;    /* magnitude frequency separation of components */
  float  *y1;          /* output for component 1 */
  float  *y2;          /* output for component 2 */
  int     i;           /* counter */
  int     fd;          /* file descriptor */
  int     size;        /* row , col dim */
  int     N;           /* size * size */
 
 /*
  * check for correct number of command line args
  */
  if (argc !=  10) {
    printf("\n\n%s: Compare IBW and freq seq of 2 components",argv[0]);
    printf("\nusage: %s size A1fn U1fn V1fn A2fn U2fn V2fn outfn1 outfn2\n",
           argv[0]);
    return;
  }

 /*
  * parse arguments, read input files, and allocate working arrays
  */
  size = atoi(argv[1]);
  N = size * size;

  a1 = (float *) malloc(size*size*sizeof(float));
  a2 = (float *) malloc(size*size*sizeof(float));
  u1 = (float *) malloc(size*size*sizeof(float));
  u2 = (float *) malloc(size*size*sizeof(float));
  v1 = (float *) malloc(size*size*sizeof(float));
  v2 = (float *) malloc(size*size*sizeof(float));
  y1 = (float *) malloc(size*size*sizeof(float));
  y2 = (float *) malloc(size*size*sizeof(float));
  Magfdiff = (double *) malloc(size*size*sizeof(float));

  disk2f(a1,size,size,argv[2]);
  disk2f(a2,size,size,argv[5]);
  disk2f(u1,size,size,argv[3]);
  disk2f(u2,size,size,argv[6]);
  disk2f(v1,size,size,argv[4]);
  disk2f(v2,size,size,argv[7]);

 /*
  * compute IBW of first component
  */
  aa1 = f2d(a1,size,size);
  ddja1 = d_d_dCol(aa1,size,size);
  ddia1 = d_d_dRow(aa1,size,size);
  for (i=0; i < N; i++) {
    if (aa1[i] != (double)0.0) {
      aa1[i] = dNorm(ddia1[i], ddja1[i]) / aa1[i];
    }
  }

 /*
  * compute IBW of second component
  */
  aa2 = f2d(a2,size,size);
  ddja2 = d_d_dCol(aa2,size,size);
  ddia2 = d_d_dRow(aa2,size,size);
  for (i=0; i < N; i++) {
    if (aa2[i] != (double)0.0) {
      aa2[i] = dNorm(ddia2[i], ddja2[i]) / aa2[i];
    }
  }

 /*
  * compute magnitude frequency separation of components
  */
  for (i=0; i < N; i++) {
    Magfdiff[i] = dNorm((double)(u1[i] - u2[i]),(double)(v1[i] - v2[i]));
  }

 /*
  * compute ratio of magnitude freq separation to IBW
  */
  for (i=0; i < N; i++) {
    if (aa1[i] != (double)0.0) {
      aa1[i] = Magfdiff[i] / aa1[i];
    }
    if (aa2[i] != (double)0.0) {
      aa2[i] = Magfdiff[i] / aa2[i];
    }
  }

 /*
  * convert output to float
  */
  y1 = d2f(aa1,size,size);
  y2 = d2f(aa2,size,size);

 /*
  * write output files
  */
  f2disk(y1,size,size,argv[8]);
  f2disk(y2,size,size,argv[9]);

  return;
} 
#endif
