
/*
 * Kgain2ascii.c
 *
 * converts float Kalman gain sequence to ascii suitable for use with xvgr
 *
 * jph 2/21/96
 */

#ifndef KGAIN2ASCII_C
#define KGAIN2ASCII_C
#include <stdio.h>
#include <math.h>
#include <fcntl.h>
#include "teager2D.h"

main(argc,argv)

  int   argc;
  char *argv[];

{


  float       fin;      /* an input float */
  float      *gain;     /* float gain sequence */
  int         size;     /* length of gain sequence */
  int         ifd;      /* input file descriptor */
  FILE       *ofpin;    /* output file pointer */
  int         i;        /* counter */
 
 /*
  * Check for right number of command line arguments
  */
  if (argc !=  3) {
    printf("\n\nusage: %s gainfile outfile \n",argv[0]);
    return;
  }
  
 /*
  * Open gain file
  */
  if ((ifd = open(argv[1], O_RDONLY)) == -1) {
    printf("\n%s: could not open file %s.\n",argv[0],argv[1]);
    exit(-1);
  }

 /*
  * Read number of gains from the file and report
  */
  if (read(ifd,&fin,sizeof(float)) != sizeof(float)) {
    printf("\n%s: could not read number of gains from %s.\n",argv[0],argv[1]);
    exit(-1);
  }
  size = (int)fin;
  printf("\n%s: The number of gains is %d.\n",argv[0],size);

 /*
  * Allocate gain sequence
  */ 
  if ((gain = (float *)malloc(size * sizeof(float))) == NULL) {
    printf("\n%s: Free Store Exhausted.\n",argv[0]);
    exit(-1);
  }

 /*
  * Read gains
  */
  if (read(ifd,gain,size*sizeof(float)) != size*sizeof(float)) {
    printf("\n%s: Complete disk read of %s did not succeed %s.\n",
            argv[0],argv[1]);
    exit(-1);
  }

 /*
  * Close gain file
  */
  if (close(ifd) == -1) {
    printf("\n%s: Error closing %s.\n",argv[0],argv[1]);
    exit(-1);
  }

 /*
  * Open output file
  */
  if ((ofpin = fopen(argv[2],"w")) == NULL) {
    printf("\nCannot open file %s",argv[2]);
    exit(0);
  }
  for (i=0; i<size; i++) {
    fprintf(ofpin,"%d\t%f\n",i,gain[i]);
  }
  fclose(ofpin);
  return;
} 
#endif
