
/*
 * LWk2Mnth.c
 *
 *  Read Lucent data file with weekly circuit pack data.  Convert to
 *  monthly data.
 *
 *
 * jph 8/3/99
 *
 */

#ifndef LWK2MNTH
#define LWK2MNTH
#include <stdio.h>
#include <math.h>
#include <fcntl.h>
#include "teager2D.h"

main(argc,argv)

  int   argc;
  char *argv[];

{


  int       size;           /* number of lines in input file */
  int       Line;           /* line number of input file being processed */
  char     *infn;           /* input filename */
  char     *outfn;          /* output filename */
  int       junk;           /* dummy variable to hold comcode */
  int       AtWeek;         /* At Week specifier */
  int       WkOrd;          /* Number of orders for At Week */
  int       YearIn;         /* AtWeek year for last line read */
  int       MonthIn;        /* AtWeek month for last line read */
  int       WeekIn;         /* AtWeek week for last line read */
  int       Year;           /* Year of the month currently being processed */
  int       Month;          /* Month currently being processed */
  int       MonthOrd;       /* Orders for month currently being processed */
  BYTE      FirstWrite;     /* True after first record written to output file */
  FILE     *infile;         /* input  file pointer */
  FILE     *outfile;        /* output file pointer */
 
 /*
  * Parse command line arguments
  */
  if (argc !=  4) {
    printf("\n\nusage: %s size infile outfile \n",argv[0]);
    printf("\n  Convert weekly lucent ckt pack data to monthly.\n");
    return;
  }
  size = atoi(argv[1]);
  infn = argv[2];
  outfn = argv[3];

 /*
  * Open input and output files
  */
  if ((infile = fopen(infn,"r")) == NULL) {
    printf("\n%s: cannot open file %s\n",argv[0],infn);
    exit(-1);
  }
  if ((outfile = fopen(outfn,"w")) == NULL) {
    printf("\n%s: cannot open file %s\n",argv[0],outfn);
    exit(-1);
  }

 /*
  * Read the first record of the input file.
  */
  FirstWrite = 0;
  fscanf(infile,"%d %d %d\n",&junk,&AtWeek,&WkOrd);
  Year = YearIn = AtWeek / 1000;
  Month = MonthIn = (AtWeek / 10) % 100;
  WeekIn = AtWeek % 10;
  MonthOrd = WkOrd;
  if (MonthOrd < 0) {
    MonthOrd = 0;
  }

/*----------------------------------
    printf("\nYearIn=%d, MonthIn=%d, WeekIn=%d, WkOrd=%d",
           YearIn,MonthIn,WeekIn,WkOrd);
----------------------------------*/


 /*
  * Loop over records in the input file.  Aggregate weekly data to monthly.
  */
  for (Line=1; Line < size; Line++) {
    fscanf(infile,"%d %d %d\n",&junk,&AtWeek,&WkOrd);
    YearIn = AtWeek / 1000;
    MonthIn = (AtWeek / 10) % 100;
    WeekIn = AtWeek % 10;

/*----------------------------------
    printf("\nYearIn=%d, MonthIn=%d, WeekIn=%d, WkOrd=%d",
           YearIn,MonthIn,WeekIn,WkOrd);
    if (Line > 20) {
      printf("\n");
      exit(0);
    }
----------------------------------*/

    if ( (MonthIn == Month) && (YearIn == Year) ) {
      if (WkOrd > 0) {
        MonthOrd += WkOrd;
      }
    } else {  /* just read a new month; write old month to output file */
      if (!FirstWrite) {
        fprintf(outfile,"%d\t%d\t%d\n",MonthOrd,Year,Month);

/*----------------------------------
        printf("\n%d\t%d\t%d\n",MonthOrd,Year,Month);
----------------------------------*/

        FirstWrite = 1;
      } else {
        fprintf(outfile,"%d\n",MonthOrd);

/*----------------------------------
        printf("\n%d\n",MonthOrd);
----------------------------------*/

      }
      Year = YearIn;
      Month = MonthIn;
      MonthOrd = WkOrd;
      if (MonthOrd < 0) {
        MonthOrd = 0;
      }
    }
  }

  fprintf(outfile,"%d\n",MonthOrd);

 /*
  * Close files
  */
  fclose(infile);
  fclose(outfile);

  return;
} 
#endif
