
/*
 * MkIBW.c
 *
 *  Computes the instantaneous bandwidth of an amplitude image.
 *    - input a float amplitude image.
 *    - output a float image of the instantaneous bandwidth,
 *        | del A |
 *        ---------
 *          | A |
 *
 *  usage:  MkIBW size infile outfile
 * 
 * 8/16/94 jph
 *
 */

#ifndef MkIBW
#define MkIBW
#include <stdio.h>
#include <math.h>
#include <fcntl.h>
#include "teager2D.h"

main(argc,argv)

  int   argc;
  char *argv[];

{


  float  *x;           /* image to be read */
  double *xx;          /* double version of x */
  double *ddjxx;       /* d/dj xx */
  double *ddixx;       /* d/di xx */
  float  *y;           /* output image */
  int     i;           /* counter */
  int     fd;          /* file descriptor */
  int     size;        /* row , col dim */
  int     N;           /* size * size */
 
  if (argc !=  4) {
    printf("\n\n%s: Compute Instantaneous Bandwidth Image",argv[0]);
    printf("\nusage: %s size infile outfile \n",argv[0]);
    return;
  }

  size = atoi(argv[1]);
  N = size * size;
  x = (float *) malloc(size*size*sizeof(float));
  disk2f(x,size,size,argv[2]);
  xx = f2d(x,size,size);
  ddjxx = d_d_dCol(xx,size,size);
  ddixx = d_d_dRow(xx,size,size);
  for (i=0; i < N; i++) {
    if (xx[i] != (double)0.0) {
      xx[i] = dNorm(ddixx[i], ddjxx[i]) / xx[i];
    }
  }
  y = d2f(xx,size,size);
  f2disk(y,size,size,argv[3]);

  return;
} 
#endif
