
/*
 * aDropPts.c
 *
 * Reads an ascii signal, subsamples without changing the domain,
 * and writes the subsampled signal back out to an ascii file suitable
 * for xvgr.
 *
 * jph 4/21/95
 *
 */

#ifndef ADROPPTS
#define ADROPPTS
#include <stdio.h>
#include <math.h>
#include <fcntl.h>
#include "teager2D.h"

main(argc,argv)

  int   argc;
  char *argv[];

{


  int       size;           /* number of samples in signal */
  int       i;              /* loop counter */
  int       outsize;        /* number of samples in subsampled signal */
  int       outidx;         /* cursor into output signal */
  int       lastidx;        /* index of last input sample kept in output */
  double    interval;       /* subsampling interval */
  double   *x;              /* input signal domain */
  double   *y;              /* input signal values */
  double   *sx;             /* subsampled signal domain */
  double   *sy;             /* subsampled signal values */
  FILE     *infile;         /* input  file pointer */
  FILE     *outfile;        /* output file pointer */
 
 /*
  * Parse command line arguments
  */
  if (argc !=  5) {
    printf("\n\nusage: %s size interval infile outfile \n",argv[0]);
    printf("\n  Subsample an ascii signal w/out changing the domain.\n");
    return;
  }
  size = atoi(argv[1]);
  interval = atof(argv[2]);

 /*
  * Allocate working arrays 
  */
  if((x  = (double *) malloc(size*sizeof(double))) == NULL) {
    printf("\n%s: free store exhausted.\n",argv[0]);
    exit(-1);
  }
  if((y  = (double *) malloc(size*sizeof(double))) == NULL) {
    printf("\n%s: free store exhausted.\n",argv[0]);
    exit(-1);
  }
  if((sx = (double *) malloc(size*sizeof(double))) == NULL) {
    printf("\n%s: free store exhausted.\n",argv[0]);
    exit(-1);
  }
  if((sy = (double *) malloc(size*sizeof(double))) == NULL) {
    printf("\n%s: free store exhausted.\n",argv[0]);
    exit(-1);
  }

 /*
  * Read the input file
  */
  if ((infile = fopen(argv[3],"r")) == NULL) {
    printf("\n%s: cannot open file %s\n",argv[0],argv[3]);
    exit(-1);
  }
  for (i=0; i<size; i++) {
    fscanf(infile,"%lf\t%lf\n",x+i,y+i);
  }
  fclose(infile);

 /*
  * Subsample
  */
  for (i=outsize=lastidx=0,outidx=1,sx[0]=x[0],sy[0]=y[0]; i<size; outidx++) {
    for (i=lastidx+1; i < size && fabs(x[i] - x[lastidx]) < interval; i++) {
      ;
    }
    sx[outidx] = x[i];
    sy[outidx] = y[i];
    lastidx = i;
  }
  outsize = outidx-1;

 /*
  * Write the output file 
  */
  if ((outfile = fopen(argv[4],"w")) == NULL) {
    printf("\n%s: cannot open file %s\n",argv[0],argv[4]);
    exit(-1);
  }
  for (i=0; i<outsize; i++) {
    fprintf(outfile,"%lf\t%lf\n",sx[i],sy[i]);
  }
  fclose(outfile);

  return;
} 
#endif
