
/*
 * bmax2mean.c
 *
 *   In a byte image, 
 *     1. find the max and mean grey levels
 *     2. change every pixel >= max-5 to mean.
 *
 *   This is a specialized program for fingerprint work to change the latent
 *   region from ~255 to the mean of the image.
 *
 * 6/18/99 jph
 *
 */

#include <stdio.h>
#include <math.h>
#include <fcntl.h>
#include "teager2D.h"

main(argc,argv)

  int   argc;
  char *argv[];

{


  int    size;              /* row , col dim */
  int    i;                 /* loop counter */
  char  *infile;            /* input filename */
  char  *outfile;           /* output filename */
  BYTE  *x;                 /* input image  */
  BYTE   min;               /* minimum grey level of input image */
  BYTE   med;               /* median grey level of input image */
  BYTE   mean;              /* mean grey level of input image */
  BYTE   max;               /* maximum grey level of input image */
  double dmean;             /* mean grey level of input image */
  double sigma;             /* std deviation of input image */

 /*
  * Parse args
  */
  if (argc !=  4) {
    printf("\n\n%s: change ~max grey level to mean in a byte image.",argv[0]);
    printf("\nusage: %s size infile outfile\n",argv[0]);
    return;
  }
  size = atoi(argv[1]);
  infile = argv[2];
  outfile = argv[3];

 /*
  * Allocate image and read file
  */
  x = (BYTE *) malloc(size*size*sizeof(BYTE));
  disk2byte(x,size,size,infile);

 /*
  * Find max and mean
  */
  sstat(x,size,size,&min,&med,&max,&dmean,&sigma);
  mean = (BYTE)(dmean + (double)0.5);

 /*
  * Change all occurences of max to mean
  */
  for (i=0; i < size*size; i++) {
    if (x[i] >= max-5) {
      x[i] = mean;
    }
  }

 /*
  * Write output file
  */
  byte2disk(x,size,size,outfile);

  return;
} 
