
/*
 * bmax2meanxy.c
 *
 *   In a byte image, 
 *     0. Do *not* assume image is square
 *     1. find the max and mean grey levels
 *     2. change every pixel >= max-5 to mean.
 *     3. pad to square size that is a power of two filling with mean.
 *
 *   This is a specialized program for fingerprint work to change the latent
 *   region from ~255 to the mean of the image.
 *
 * 6/29/99 jph
 *
 */

#include <stdio.h>
#include <math.h>
#include <fcntl.h>
#include "teager2D.h"

main(argc,argv)

  int   argc;
  char *argv[];

{


  int    rows;              /* number of rows in input image */
  int    cols;              /* number of cols in input image */
  int    size;              /* max(rows,cols) */
  int    outrows;           /* number of rows in output image */
  int    outcols;           /* number of cols in output image */
  int    i;                 /* loop counter for rows */
  int    j;                 /* loop counter for cols */
  int    k;                 /* loop counter for input image */
  int    l;                 /* loop counter for output image */
  char  *infile;            /* input filename */
  char  *outfile;           /* output filename */
  BYTE  *x;                 /* input image  */
  BYTE  *y;                 /* output image  */
  BYTE   min;               /* minimum grey level of input image */
  BYTE   med;               /* median grey level of input image */
  BYTE   mean;              /* mean grey level of input image */
  BYTE   max;               /* maximum grey level of input image */
  double dmean;             /* mean grey level of input image */
  double sigma;             /* std deviation of input image */

 /*
  * Parse args
  */
  if (argc !=  5) {
    printf("\n\n%s: change ~max to mean in a byte image and make square."
           ,argv[0]);
    printf("\nusage: %s rows cols infile outfile\n",argv[0]);
    return;
  }
  rows = atoi(argv[1]);
  cols = atoi(argv[2]);
  infile = argv[3];
  outfile = argv[4];
  if (rows > cols) {
    size = rows;
  } else {
    size = cols;
  }
  outrows = outcols = 
    (int)pow((double)2.0,ceil(log((double)size)/log((double)2.0)));

 /*
  * Allocate input image and read file
  */
  x = (BYTE *) malloc(rows*cols*sizeof(BYTE));
  disk2byte(x,rows,cols,infile);

 /*
  * Find max and mean
  */
  sstat(x,rows,cols,&min,&med,&max,&dmean,&sigma);
  mean = (BYTE)(dmean + (double)0.5);

 /*
  * Change all occurences of max to mean
  */
  for (i=0; i < rows*cols; i++) {
    if (x[i] >= max-5) {
      x[i] = mean;
    }
  }

 /*
  * Allocate output image and fill
  */
  y = (BYTE *) malloc(outrows*outcols*sizeof(BYTE));
  for (i=k=l=0; i < rows; i++) {
    for (j=0; j < cols; j++,k++,l++) {
      y[l] = x[k];
    }
    for ( ; j < outcols; j++,l++) {
      y[l] = mean;
    }
  }
  for ( ; i < outrows; i++) {
    for (j=0; j < outcols; j++,l++) {
      y[l] = mean;
    }
  }

 /*
  * Write output file
  */
  byte2disk(y,outrows,outcols,outfile);

  return;
} 
