/*
 * bthresh: 
 *   
 *   Turn a byte image into a binary image by thresholding at a specified
 *   value.  For pixels that exceed threshold, the output is 255.  Otherwise
 *   the output is 0.  The output uses one byte per pixel.
 *
 *   The input image must be square.
 *
 *   The following definition for "BYTE" is in the include file teager2D.h:
 *
 *        #define BYTE unsigned char
 *
 * 8/23/2000 jph
 *
 */

#include <stdio.h>

#include "teager2D.h"


main(argc,argv) 

  int     argc;
  char   *argv[];
{

  int      i;                     /* counter */
  int      size;                  /* num rows/cols in image */
  BYTE     thresh;                /* threshold value to be used */
  BYTE    *x;                     /* input image */
  BYTE    *y;                     /* output image */
  char    *infile;                /* input filename */
  char    *outfile;               /* output filename */

 /*
  * Check for proper invocation, parse args
  */  
  if (argc != 5) {
    printf(
   "\n%s: converts sizeXsize byte image to a binary byte image by thesholding.",
       argv[0]);
    printf("\nUsage: %s size thresh infile outfile\n",argv[0]);
    exit(0);
  }
  size = atoi(argv[1]);
  thresh = (BYTE)atoi(argv[2]);
  infile = argv[3];
  outfile = argv[4];

 /*
  * Allocate image arrays
  */
  if ((x = (BYTE *)malloc(size*size*sizeof(BYTE))) == NULL) {
    printf("\n%s: free store exhausted.\n",argv[0]);
    exit(-1);
  }
  if ((y = (BYTE *)malloc(size*size*sizeof(BYTE))) == NULL) {
    printf("\n%s: free store exhausted.\n",argv[0]);
    exit(-1);
  }

 /*
  * Read the input image
  */
  disk2byte(x,size,size,infile);

 /*
  * Threshold the image
  */
  for (i=0; i<size*size; i++) {
    if (x[i] < thresh) {
      y[i] = 0x00;
    }
    else {
      y[i] = 0xff;
    }
  }

 /*
  * Write the output image
  */
  byte2disk(y,size,size,outfile);

  return;
}
