/*
 * dsply40.c
 *
 *  program inputs the complex responses of 40 gabor filters to an input
 *  image and makes a 512x512 output image showing all of the responses
 *  surrounding the original image.
 *
 *  It is assumed that the original image is declared unsigned char[256][256]
 *  and that all of the response images are defined float[256][256].
 *
 *  the command line argument gives the base name of the file
 *  to be operated on.  For example
 *
 *  dsply40 myfile
 *
 *  will assume the original byte data is in myfile.data, that the
 *  real gabor filter responses are in myfile_realXX.data, and that
 *  the imaginary gabor filter responses are in myfile_imagXX.data.
 *
 *  this program is quick and dirty.
 *
 *  1 July 1992  joe havlicek
 */

#ifndef DSPLY40_C
#define DSPLY40_C
#include <stdio.h>
#include <math.h>
#include <fcntl.h>
#include <stdlib.h>

#define FNSIZE 160

int byte2monitor();
int byte2disk();

main(argc,argv)

  int   argc;
  char *argv[];

{

 /*
  * local variable frame
  */
  int start_row, start_col;
  int filecount;
  int i,j,row,col;
  int src_row,src_col;
  float maxf, minf;
  float fact;
  char cmd[FNSIZE];
  char fn[FNSIZE];
  int choice;
  int fd;
  float A[256][256],B[256][256];
  float AA[64][64];
  float fx[504][504];
  unsigned char im256[256][256];
  unsigned char im128[128][128];
  unsigned char result[512][512];

  

  system("clear");
  printf("\n\nDoing %s.data\n",argv[1]);
  do {
    printf("\nCHOOSE RESPONSE DATA TO BE DISPLAYED IN COMPOSITE:");
    printf("\n1.  Magnitude response");
    printf("\n2.  Re[response]");
    printf("\n3.  Im[response]\n");
    scanf("%s",cmd);
    choice = atoi(cmd);
  }while((choice<1) || (choice>3));
  printf("\nMaking Composite.....< it will take a while >.....\n\n");

  maxf = minf = 0.0;
  filecount = 0;

 /*
  * do the first two rows of images in the composite
  */
  for (start_row=0; start_row<73; start_row+=72)
    for (start_col=0; start_col<483; start_col+=72) { 
      if (choice != 3) {
        sprintf(fn,"%s_real%02d.data",argv[1],filecount);
        if ((fd = open(fn, O_RDONLY))==-1) {
          printf("\nCould Not Open File %s\n",fn);
          return;
        }
        else printf("\n%s",fn);
        read(fd,A,256*256*sizeof(float));
        close(fd);
        if (choice==1) {
          sprintf(fn,"%s_imag%02d.data",argv[1],filecount);
          if ((fd = open(fn, O_RDONLY))==-1) {
            printf("\nCould Not Open File %s\n",fn);
            return;
          }
          else printf("\n%s",fn);
          read(fd,B,256*256*sizeof(float));
          close(fd);
          for (i=0; i<256; i++)
            for (j=0; j<256; j++)
              A[i][j] = (float) pow(
                        pow((double)(A[i][j]),(double)2.0) +
                        pow((double)(B[i][j]),(double)2.0),(double)0.5);
        }
      }
      else {
        sprintf(fn,"%s_imag%02d.data",argv[1],filecount);
        if ((fd = open(fn, O_RDONLY))==-1) {
          printf("\nCould Not Open File %s\n",fn);
          return;
        }
        else printf("\n%s",fn);
        read(fd,A,256*256*sizeof(float));
        close(fd);
      }
      filecount++;
      for (src_row=row=0; src_row<256; src_row+=4,row++)
        for (src_col=col=0; src_col<256; src_col+=4,col++) {
          AA[row][col] = 0.0;
          for (i=0; i<4; i++)
            for (j=0; j<4; j++)
              AA[row][col] += A[src_row+i][src_col+j] / 16.0;
      }
      for (i=0; i<64; i++)
        for (j=0; j<64; j++)
          if (AA[i][j] > maxf) maxf = AA[i][j];
          else if (AA[i][j] < minf) minf = AA[i][j];
      for (i=0; i<4; i++)
        for (j=0; j<72; j++) 
          fx[start_row+i][start_col+j] = fx[start_row+71-i][start_col+j] =
          fx[start_row+j][start_col+i] = fx[start_row+j][start_col+71-i] = 0.0;
      for (i=0; i<64; i++)
        for (j=0; j<64; j++)
          fx[start_row+i+4][start_col+j+4] = AA[i][j];
  }

 /*
  * do the middle three rows of images in the composite
  */
  for (start_row=144; start_row<360; start_row+=72) {
    for (start_col=0; start_col<144; start_col+=72) {
      if (choice != 3) {
        sprintf(fn,"%s_real%02d.data",argv[1],filecount);
        if ((fd = open(fn, O_RDONLY))==-1) {
          printf("\nCould Not Open File %s\n",fn);
          return;
        }
        else printf("\n%s",fn);
        read(fd,A,256*256*sizeof(float));
        close(fd);
        if (choice==1) {
          sprintf(fn,"%s_imag%02d.data",argv[1],filecount);
          if ((fd = open(fn, O_RDONLY))==-1) {
            printf("\nCould Not Open File %s\n",fn);
            return;
          }
          else printf("\n%s",fn);
          read(fd,B,256*256*sizeof(float));
          close(fd);
          for (i=0; i<256; i++)
            for (j=0; j<256; j++)
              A[i][j] = (float) pow(
                        pow((double)(A[i][j]),(double)2.0) +
                        pow((double)(B[i][j]),(double)2.0),(double)0.5);
        }
      }
      else {
        sprintf(fn,"%s_imag%02d.data",argv[1],filecount);
        if ((fd = open(fn, O_RDONLY))==-1) {
          printf("\nCould Not Open File %s\n",fn);
          return;
        }
        else printf("\n%s",fn);
        read(fd,A,256*256*sizeof(float));
        close(fd);
      }
      filecount++;
      for (src_row=row=0; src_row<256; src_row+=4,row++)
        for (src_col=col=0; src_col<256; src_col+=4,col++) {
          AA[row][col] = 0.0;
          for (i=0; i<4; i++)
            for (j=0; j<4; j++)
              AA[row][col] += A[src_row+i][src_col+j] / 16.0;
      }
      for (i=0; i<64; i++)
        for (j=0; j<64; j++)
          if (AA[i][j] > maxf) maxf = AA[i][j];
          else if (AA[i][j] < minf) minf = AA[i][j];
      for (i=0; i<4; i++)
        for (j=0; j<72; j++)
          fx[start_row+i][start_col+j] = fx[start_row+71-i][start_col+j] =
          fx[start_row+j][start_col+i] = fx[start_row+j][start_col+71-i] = 0.0;
      for (i=0; i<64; i++)
        for (j=0; j<64; j++)
          fx[start_row+i+4][start_col+j+4] = AA[i][j];
    }
    for (start_col=360; start_col<433; start_col+=72) {
      if (choice != 3) {
        sprintf(fn,"%s_real%02d.data",argv[1],filecount);
        if ((fd = open(fn, O_RDONLY))==-1) {
          printf("\nCould Not Open File %s\n",fn);
          return;
        }
        else printf("\n%s",fn);
        read(fd,A,256*256*sizeof(float));
        close(fd);
        if (choice==1) {
          sprintf(fn,"%s_imag%02d.data",argv[1],filecount);
          if ((fd = open(fn, O_RDONLY))==-1) {
            printf("\nCould Not Open File %s\n",fn);
            return;
          }
          else printf("\n%s",fn);
          read(fd,B,256*256*sizeof(float));
          close(fd);
          for (i=0; i<256; i++)
            for (j=0; j<256; j++)
              A[i][j] = (float) pow(
                        pow((double)(A[i][j]),(double)2.0) +
                        pow((double)(B[i][j]),(double)2.0),(double)0.5);
        }
      }
      else {
        sprintf(fn,"%s_imag%02d.data",argv[1],filecount);
        if ((fd = open(fn, O_RDONLY))==-1) {
          printf("\nCould Not Open File %s\n",fn);
          return;
        }
        else printf("\n%s",fn);
        read(fd,A,256*256*sizeof(float));
        close(fd);
      }
      filecount++;
      for (src_row=row=0; src_row<256; src_row+=4,row++)
        for (src_col=col=0; src_col<256; src_col+=4,col++) {
          AA[row][col] = 0.0;
          for (i=0; i<4; i++)
            for (j=0; j<4; j++)
              AA[row][col] += A[src_row+i][src_col+j] / 16.0;
      }
      for (i=0; i<64; i++)
        for (j=0; j<64; j++)
          if (AA[i][j] > maxf) maxf = AA[i][j];
          else if (AA[i][j] < minf) minf = AA[i][j];
      for (i=0; i<4; i++)
        for (j=0; j<72; j++)
          fx[start_row+i][start_col+j] = fx[start_row+71-i][start_col+j] =
          fx[start_row+j][start_col+i] = fx[start_row+j][start_col+71-i] = 0.0;
      for (i=0; i<64; i++)
        for (j=0; j<64; j++)
          fx[start_row+i+4][start_col+j+4] = AA[i][j];
    }
  }
 /*
  * do the last two rows of the composite image
  */
  for (start_row=360; start_row<433; start_row+=72)
    for (start_col=0; start_col<483; start_col+=72) {
      if (choice != 3) {
        sprintf(fn,"%s_real%02d.data",argv[1],filecount);
        if ((fd = open(fn, O_RDONLY))==-1) {
          printf("\nCould Not Open File %s\n",fn);
          return;
        }
        else printf("\n%s",fn);
        read(fd,A,256*256*sizeof(float));
        close(fd);
        if (choice==1) {
          sprintf(fn,"%s_imag%02d.data",argv[1],filecount);
          if ((fd = open(fn, O_RDONLY))==-1) {
            printf("\nCould Not Open File %s\n",fn);
            return;
          }
          else printf("\n%s",fn);
          read(fd,B,256*256*sizeof(float));
          close(fd);
          for (i=0; i<256; i++)
            for (j=0; j<256; j++)
              A[i][j] = (float) pow(
                        pow((double)(A[i][j]),(double)2.0) +
                        pow((double)(B[i][j]),(double)2.0),(double)0.5);
        }
      }
      else {
        sprintf(fn,"%s_imag%02d.data",argv[1],filecount);
        if ((fd = open(fn, O_RDONLY))==-1) {
          printf("\nCould Not Open File %s\n",fn);
          return;
        }
        else printf("\n%s",fn);
        read(fd,A,256*256*sizeof(float));
        close(fd);
      }
      filecount++;
      for (src_row=row=0; src_row<256; src_row+=4,row++)
        for (src_col=col=0; src_col<256; src_col+=4,col++) {
          AA[row][col] = 0.0;
          for (i=0; i<4; i++)
            for (j=0; j<4; j++)
              AA[row][col] += A[src_row+i][src_col+j] / 16.0;
      }
      for (i=0; i<64; i++)
        for (j=0; j<64; j++)
          if (AA[i][j] > maxf) maxf = AA[i][j];
          else if (AA[i][j] < minf) minf = AA[i][j];
      for (i=0; i<4; i++)
        for (j=0; j<72; j++)
          fx[start_row+i][start_col+j] = fx[start_row+71-i][start_col+j] =
          fx[start_row+j][start_col+i] = fx[start_row+j][start_col+71-i] = 0.0;
      for (i=0; i<64; i++)
        for (j=0; j<64; j++)
          fx[start_row+i+4][start_col+j+4] = AA[i][j];
    }

 /*
  * go back and set the borders of the response images to minf
  */
  for (start_row=0; start_row<73; start_row+=72)
    for (start_col=0; start_col<483; start_col+=72) 
      for (i=0; i<4; i++)
        for (j=0; j<72; j++)
          fx[start_row+i][start_col+j] = fx[start_row+71-i][start_col+j] =
          fx[start_row+j][start_col+i] = fx[start_row+j][start_col+71-i] = minf;
  for (start_row=144; start_row<360; start_row+=72) {
    for (start_col=0; start_col<144; start_col+=72) 
      for (i=0; i<4; i++)
        for (j=0; j<72; j++)
          fx[start_row+i][start_col+j] = fx[start_row+71-i][start_col+j] =
          fx[start_row+j][start_col+i] = fx[start_row+j][start_col+71-i] = minf;
    for (start_col=360; start_col<433; start_col+=72) 
      for (i=0; i<4; i++)
        for (j=0; j<72; j++)
          fx[start_row+i][start_col+j] = fx[start_row+71-i][start_col+j] =
          fx[start_row+j][start_col+i] = fx[start_row+j][start_col+71-i] = minf;
  }
  for (start_row=360; start_row<433; start_row+=72)
    for (start_col=0; start_col<483; start_col+=72) 
      for (i=0; i<4; i++)
        for (j=0; j<72; j++)
          fx[start_row+i][start_col+j] = fx[start_row+71-i][start_col+j] =
          fx[start_row+j][start_col+i] = fx[start_row+j][start_col+71-i] = minf;

 /*
  * fill in the center of the composite image with minf
  */
  for (i=144; i<360; i++)
    for (j=144; j<360; j++)
      fx[i][j] = minf;

 /*
  * normalize the composite image to be in the range [0,255]
  */
  fact = 255.0 / (maxf - minf);
  for (i=0; i<504; i++)
    for (j=0; j<504; j++)
      fx[i][j] = fact * (fx[i][j] - minf);


 /*
  * fill in the borders of the result image with zeros
  */
  for (i=0; i<4; i++)
    for (j=0; j<512; j++) 
      result[i][j] = result[j][i] = result[512-i][j] = result[j][512-i] = 0;

 /*
  * copy the floating composite image into the byte result image
  */
  for (i=0; i<504; i++)
    for (j=0; j<504; j++)
      result[i+4][j+4] = (unsigned char) fx[i][j];

 /*
  * copy the original image into the center of the composite
  */
  sprintf(fn,"%s.data",argv[1]);
  if ((fd = open(fn, O_RDONLY))==-1) {
    printf("\nCould Not Open File %s\n",fn);
    return;
  }
  else printf("\n%s",fn);
  read(fd,im256,256*256*sizeof(unsigned char));
  close(fd);
  for (i=0; i<256; i+=2)
    for (j=0; j<256; j+=2)
      im128[(int)i/2][(int)j/2] = (unsigned char)(
                       (double)((int)im256[i][j]+(int)im256[i][j+1]
                        +(int)im256[i+1][j]+(int)im256[i+1][j+1]) 
                        / (double)4.0);
  for (i=0; i<128; i++)
    for (j=0; j<128; j++)
      result[i+192][j+192] = im128[i][j];

 /*
  * write output to the requested destination (s)
  */
  do {
    do {
      system("clear");
      printf("\nCHOOSE OUTPUT DESTINATION:");
      printf("\n1.  DataCube");
      printf("\n2.  Binary File ( 512x512 byte array )");
      printf("\n3.  PostScript File");
      printf("\n4.  Quit\n\n");
      scanf("%s",cmd);
      choice = atoi(cmd);
    }while((choice < 1) || (choice > 4));

   /*
    * carry out choice
    */
    switch(choice) {

      case 1:  bytedisplay(result,512,512);
               break;
      case 2:  printf("\n\nFilename ? ");
               scanf("%s",fn);
               byte2disk(result,512,512,fn);
               break;
      case 3:  printf("\n\nFilename ? ");
               scanf("%s",fn);
               byte2disk(result,512,512,"___dsply40_tmp_file");
               sprintf(cmd,"ph -f %s ___dsply40_tmp_file",fn);
               system(cmd);
               system("rm ___dsply40_tmp_file");
               break; 
      default: break;
    }

  }while(choice != 4);

}
#endif 
