/*
 * fAUV2xyz.c
 *
 *   read float images for amplitude and horiz & vert components of 
 *   instantaneous frequency.  Output single float image as (x,y,z) triples.
 *   This can be read in by xprism3 to generate a scatter plot.  x=U, y=V, a=Z.
 *
 * jph 1/11/99
 */

#ifndef FAUV2XYZ_C
#define FAUV2XYZ_C
#include "teager2D.h"

main(argc,argv)

   int   argc;
  char *argv[];

{

  float *A;              /* AM function */
  float *U;              /* horizontal component of frequency vector */
  float *V;              /* vertical   component of frequency vector */
  float *AllThree;       /* array of A,U,V triples */
  char  *Afile;          /* Input file name for A estimates */
  char  *Ufile;          /* Input file name for U estimates */
  char  *Vfile;          /* Input file name for V estimates */
  char  *OutFile;        /* Output file name */
  int    size;           /* row , col dim */
  int    i;              /* loop counter on rows and cols */
  int    j;              /* cursor into output array */

 /*
  * Parse args
  */
  if (argc != 6) {
    printf(
    "\n\n%s: Read A,U,V files and output X=U,Y=V,Z=A for xprism3 scatter plot.",
      argv[0]);
    printf("\nusage: %s size Afile Ufile Vfile outfile \n",argv[0]);
    exit(0);
  }
  size = atoi(argv[1]);
  Afile = argv[2];
  Ufile = argv[3];
  Vfile = argv[4];
  OutFile = argv[5];

 /*
  * Allocate arrays
  */
  A = (float*) malloc(size*size*sizeof(float));
  U = (float*) malloc(size*size*sizeof(float));
  V = (float*) malloc(size*size*sizeof(float));
  AllThree = (float*) malloc(3*size*size*sizeof(float));

 /*
  * Read input data
  */
  disk2f(A,size,size,Afile);
  disk2f(U,size,size,Ufile);
  disk2f(V,size,size,Vfile);

 /*
  * Make output array and write
  */

  for (i=j=0; i < size*size; i++) {
    AllThree[j++] = U[i];
    AllThree[j++] = V[i];
    AllThree[j++] = A[i];
  }
  f2disk(AllThree,3*size*size,1,OutFile);

  return;
}
#endif
