
/*
 * fAve.c
 *
 *  Perform noncircular MxM average filter on float image.
 *  In incomplete (edge) neighborhoods, the average is taken over the
 *  elements that are present.
 *
 */

#ifndef FAVE_C
#define FAVE_C
#include <stdio.h>
#include <math.h>
#include <fcntl.h>
#include "teager2D.h"

main(argc,argv)

  int   argc;
  char *argv[];

{


  int          size;         /* row , col dim */
  char        *infile;       /* input filename */
  char        *outfile;      /* output filename */
  int          M;            /* window size */
  int          Mo2;          /* window half-size */
  float       *x;            /* input image */
  float       *y;            /* output image */
  int          row,col;      /* loop counter */
  int          Row,Col;      /* loop counter */
  int          N;            /* Num pixels in window */
  float        Wsum;         /* sum of pixels in window */
 
  if (argc !=  5) {
    printf("\n\n%s: Perform MxM noncircular averaging on float image.",
           argv[0]);
    printf("\nusage: %s size infile M outfile \n",argv[0]);
    return;
  }

 /*
  * Parse command line parameters
  */
  size  = atoi(argv[1]);
  infile = argv[2];
  M = atoi(argv[3]);
  outfile = argv[4];

  if (!(M%2)) {
    printf("\n%s: window size must be odd. %d entered.\n\n",argv[0],M);
    exit(-1);
  }

  Mo2 = M >> 1;  
 
 /*
  * Read input
  */
  x = (float *) malloc(size*size*sizeof(float));
  disk2f(x,size,size,infile);

 /*
  * Allocate output
  */
  y = (float *) malloc(size*size*sizeof(float));

 /*
  * Loop on pixels
  */
  for (row=0; row < size; row++) {
    for (col=0; col < size; col++) {
      N = 0;
      Wsum = (float)0.0;
      for (Row=row-Mo2; Row <= row+Mo2; Row++) {
        for (Col=col-Mo2; Col <= col+Mo2; Col++) {
          if ( (Row >= 0) && (Row < size) && (Col >= 0) && (Col < size) ) {
            N++;
            Wsum += x[Row*size + Col];
          }
        }
      }
      y[row*size + col] = Wsum / (float)N;
    }
  }

 /*
  * Write output
  */
  f2disk(y,size,size,outfile);

  return;
} 
#endif
