/*
 * fDom2FFT.c
 *
 *   Make a byte image of the dominant 2 FFT components of a float image.
 * 
 * 4/17/96 jph
 *
 */

#include <stdio.h>
#include <math.h>
#include <fcntl.h>
#include <string.h>

#include "teager2D.h"

main(argc,argv) 
 
  int   argc;
  char *argv[];
 
{

  int       size;              /* row/col dim of image */
  int       N;                 /* size * size */
  int       i;                 /* loop counter */
  int       j;                 /* loop counter */
  int       k;                 /* loop counter */
  float    *xReal;             /* Real part of image */
  float    *xImag;             /* Imag part of image */
  float    *XReal;             /* Real part of FFT */
  float    *XImag;             /* Imag part of FFT */
  float    *logMagFFT;         /* log[1 + Mag FFT] */
  float     BigVal1;           /* value of biggest FFT coefficient */
  int       BigRow1;           /* row of biggest FFT coefficient */
  int       BigCol1;           /* col of biggest FFT coefficient */
  float     BigVal2;           /* value of 2nd biggest FFT coefficient */
  int       BigRow2;           /* row of 2nd biggest FFT coefficient */
  int       BigCol2;           /* col of 2nd biggest FFT coefficient */
  float    *D;                 /* dominant fft component */
  double    u1;                /* horiz freq of dominant fft component */
  double    v1;                /* vert  freq of dominant fft component */
  double    u2;                /* horiz freq of 2nd dominant fft component */
  double    v2;                /* vert  freq of 2nd dominant fft component */
  char     *fname;             /* filename string */
  BYTE     *y;                 /* byte output image */

  if (argc != 4) {
    printf("\n%s: write dominant fft component of a REAL float image to file."
        ,argv[0]);
    printf("\nusage: %s size RealInFile OutFile\n",argv[0]);
    exit(0);
  }
  size   = atoi(argv[1]);
  fname = argv[3];
  N = size * size;

 /*
  * Allocate working arrays
  */
  if ((xReal = (float *)malloc(N * sizeof(float))) == NULL) {
    printf("\n%s: free store exhausted.\n",argv[0]);
    exit(-1);
  }
  if ((xImag = (float *)malloc(N * sizeof(float))) == NULL) {
    printf("\n%s: free store exhausted.\n",argv[0]);
    exit(-1);
  }
  if ((XReal = (float *)malloc(N * sizeof(float))) == NULL) {
    printf("\n%s: free store exhausted.\n",argv[0]);
    exit(-1);
  }
  if ((XImag = (float *)malloc(N * sizeof(float))) == NULL) {
    printf("\n%s: free store exhausted.\n",argv[0]);
    exit(-1);
  }
  if ((logMagFFT = (float *)malloc(N * sizeof(float))) == NULL) {
    printf("\n%s: free store exhausted.\n",argv[0]);
    exit(-1);
  }

 /*
  * Read Image
  */
  disk2f(xReal,size,size,argv[2]);
  for (i=0; i < size*size; i++) {
    xImag[i] = (float)0.0;
  }

 /*
  * Compute log[1 + Mag FFT]
  */
  fft2d(xReal,xImag,XReal,XImag,size,1);
  for (i=0; i < N; i++) {
    logMagFFT[i] = (float)log((double)1.0 + (double)fNorm(XReal[i],XImag[i]));
  }

 /*
  * Find the dominant component 
  */
  BigVal1 = BigVal2 = (float)0.0;
  BigRow1 = BigCol1 = BigRow2 = BigCol2 = 0;
  for (k=i=0; i < size; i++) {
    for (j=0; j < size; j++,k++) {
      if (logMagFFT[k] > BigVal1) {
        BigVal1 = logMagFFT[k];
        BigRow1 = i;
        BigCol1 = j;
      } else {
        if (logMagFFT[k] > BigVal2) {
          BigVal2 = logMagFFT[k];
          BigRow2 = i;
          BigCol2 = j;
        }
      }
    }
  }

 /*
  * construct the dominant 2 fft components
  */
  u1 = (double)2.0*(double)M_PI * (double)(BigCol1 - (size >> 1))/(double)size;
  v1 = (double)2.0*(double)M_PI * (double)(BigRow1 - (size >> 1))/(double)size;
  u2 = (double)2.0*(double)M_PI * (double)(BigCol2 - (size >> 1))/(double)size;
  v2 = (double)2.0*(double)M_PI * (double)(BigRow2 - (size >> 1))/(double)size;

  if ((D = (float *)malloc(size*size*sizeof(float))) == NULL) {
    printf("\n%s: free store exhausted.\n",argv[0]);
    exit(-1);
  }
  for (k=i=0; i < size; i++) {
    for (j=0; j < size; j++,k++) {
      D[k] = 
        (float)cos(u1*(double)j + v1*(double)i) * XReal[BigRow1*size + BigCol1]
      - (float)sin(u1*(double)j + v1*(double)i) * XImag[BigRow1*size + BigCol1]
      + (float)cos(u2*(double)j + v2*(double)i) * XReal[BigRow2*size + BigCol2]
      - (float)sin(u2*(double)j + v2*(double)i) * XImag[BigRow2*size + BigCol2];
    }
  }

 /*
  * Save dominant FFT component to byte file
  */
  y = f_dbx(D,size,size);
  byte2disk(y,size,size,fname);

  printf("\n\n");
  exit(0);

}

