/*
 * fDomFFT.c
 *
 *   Make a byte image of the dominant FFT component of a float image.
 * 
 * 4/17/96 jph
 *
 */

#include <stdio.h>
#include <math.h>
#include <fcntl.h>
#include <string.h>

#include "teager2D.h"

main(argc,argv) 
 
  int   argc;
  char *argv[];
 
{

  int       size;              /* row/col dim of image */
  int       N;                 /* size * size */
  int       i;                 /* loop counter */
  int       j;                 /* loop counter */
  int       k;                 /* loop counter */
  float    *xReal;             /* Real part of image */
  float    *xImag;             /* Imag part of image */
  float    *XReal;             /* Real part of FFT */
  float    *XImag;             /* Imag part of FFT */
  float    *logMagFFT;         /* log[1 + Mag FFT] */
  float     BigVal;            /* value of biggest FFT coefficient */
  int       BigRow;            /* row of biggest FFT coefficient */
  int       BigCol;            /* col of biggest FFT coefficient */
  float    *D;                 /* dominant fft component */
  double    u;                 /* horiz freq of dominant fft component */
  double    v;                 /* vert  freq of dominant fft component */
  char     *fname;             /* filename string */
  BYTE     *y;                 /* byte output image */

  if (argc != 4) {
    printf("\n%s: write dominant fft component of a REAL float image to file."
        ,argv[0]);
    printf("\nusage: %s size RealInFile OutFile\n",argv[0]);
    exit(0);
  }
  size   = atoi(argv[1]);
  fname = argv[3];
  N = size * size;

 /*
  * Allocate working arrays
  */
  if ((xReal = (float *)malloc(N * sizeof(float))) == NULL) {
    printf("\n%s: free store exhausted.\n",argv[0]);
    exit(-1);
  }
  if ((xImag = (float *)malloc(N * sizeof(float))) == NULL) {
    printf("\n%s: free store exhausted.\n",argv[0]);
    exit(-1);
  }
  if ((XReal = (float *)malloc(N * sizeof(float))) == NULL) {
    printf("\n%s: free store exhausted.\n",argv[0]);
    exit(-1);
  }
  if ((XImag = (float *)malloc(N * sizeof(float))) == NULL) {
    printf("\n%s: free store exhausted.\n",argv[0]);
    exit(-1);
  }
  if ((logMagFFT = (float *)malloc(N * sizeof(float))) == NULL) {
    printf("\n%s: free store exhausted.\n",argv[0]);
    exit(-1);
  }

 /*
  * Read Image
  */
  disk2f(xReal,size,size,argv[2]);
  for (i=0; i < size*size; i++) {
    xImag[i] = (float)0.0;
  }

 /*
  * Compute log[1 + Mag FFT]
  */
  fft2d(xReal,xImag,XReal,XImag,size,1);
  for (i=0; i < N; i++) {
    logMagFFT[i] = (float)log((double)1.0 + (double)fNorm(XReal[i],XImag[i]));
  }

 /*
  * Find the dominant component 
  */
  BigVal = (float)0.0;
  BigRow = BigCol = 0;
  for (k=i=0; i < size; i++) {
    for (j=0; j < size; j++,k++) {
      if (logMagFFT[k] > BigVal) {
        BigVal = logMagFFT[k];
        BigRow = i;
        BigCol = j;
      }
    }
  }

 /*
  * construct the dominant fft component
  */
  u = (double)2.0*(double)M_PI * (double)(BigCol - (size >> 1))/(double)size;
  v = (double)2.0*(double)M_PI * (double)(BigRow - (size >> 1))/(double)size;

  if ((D = (float *)malloc(size*size*sizeof(float))) == NULL) {
    printf("\n%s: free store exhausted.\n",argv[0]);
    exit(-1);
  }
  for (k=i=0; i < size; i++) {
    for (j=0; j < size; j++,k++) {
/*      D[k] = (float)cos(u*(double)j + v*(double)i);*/
      D[k] = (float)cos(u*(double)j + v*(double)i) * XReal[BigRow*size + BigCol]
         - (float)sin(u*(double)j + v*(double)i) * XImag[BigRow*size + BigCol];
    }
  }

 /*
  * Save dominant FFT component to byte file
  */
  y = f_dbx(D,size,size);
  byte2disk(y,size,size,fname);

  printf("\n\n");
  exit(0);

}

