/*
 * fEnlarge.c
 *
 *   Input float image is insize X insize.  Increase by largest possible factor
 *   such that <= outsize X outsize.  Zero pad to padsize X padsize.
 *   Output image is BYTE.
 * 
 *   10/3/95 jph
 *
 */

#include <stdio.h>
#include "teager2D.h"

main(argc,argv)

   int   argc;
  char *argv[];

{

  int     insize;        /* row,col dim of input image */
  int     outsize;       /* desired row,col dim of enlarged image */
  int     padsize;       /* desired row,col dim of padded image */
  int     k;             /* pixel counter */
  int     E;             /* enlargement by a factor of E in each direction */
  int     newsize;       /* row,col dim of enlarged, unpadded image */
  int     row,col;       /* counters */
  int     r,c;           /* counters */
  int     startrow;      /* row 0 of xBig goes in row startrow of y */
  int     startcol;      /* col 0 of xBig goes in col startcol of y */
  float  *xfloat;        /* float version of input image */
  BYTE   *x;             /* BYTE version of input image */
  BYTE   *y;             /* output BYTE image */
  BYTE   *xBig;          /* enlarged, unpadded image, BYTE */

 /*
  * Check for right number of command line arguments
  */
  if (argc != 6) {
    printf("\n\n%s: Increase size of float image and pad with zeros.",argv[0]);
    printf("\n             - Increase size of image by largest factor");
    printf("\n               possible so that newsize <= outsize.");
    printf("\n             - Zero pad to padsize X padsize.");
    printf("\n             - Output image is BYTE.\n");
    printf("\nusage: %s infile insize outsize padsize outfile \n",argv[0]);
    exit(0);
  }

 /*
  * Parse command line parameters and check for validity
  */
  insize = atoi(argv[2]);
  outsize = atoi(argv[3]);
  if (insize > outsize) {
    printf("\n\n%s: insize (%d) must be <= outsize (%d).\n\n",argv[0],insize,
                                                              outsize);
    exit(0);
  }
  padsize = atoi(argv[4]);
  if (outsize > padsize) {
    printf("\n\n%s: outsize (%d) must be <= padsize (%d).\n\n",argv[0],outsize,
                                                              padsize);
    exit(0);
  }

 /*
  * Allocate input & output images, read input image
  */
  xfloat = (float*) malloc(insize*insize*sizeof(float));
  y = (BYTE*) malloc(padsize*padsize*sizeof(BYTE));
  disk2f(xfloat,insize,insize,argv[1]);
  x = f_dbx(xfloat,insize,insize); 
  free(xfloat);

 /*
  * Zero the output image 
  */
  for (k=0; k < padsize*padsize; k++) {
    y[k] = 0;
  }

 /*
  * Enlarge 
  */
  E = (int)floor((double)outsize/(double)insize);
  if (E==1) {
    newsize = insize;
    xBig = x;
  }
  else {
    newsize = E * insize;
    xBig = (BYTE *)malloc(newsize*newsize*sizeof(BYTE));
    for (k=row=0; row < newsize; row+=E) {
      for (col=0; col < newsize; col+=E,k++) {
        for (r=0; r < E; r++) {
          for (c=0; c < E; c++) {
            xBig[(row+r)*newsize + col + c] = x[k];
          }
        }
      }
    }
  } 

 /*
  * Copy enlarged image into center of output image
  */
  startrow = startcol = (padsize - newsize) >> 1;
  for (r=0,row=startrow; r < newsize; row++,r++) {
    for (c=0,col=startcol; c < newsize; col++,c++) {
      y[row*padsize + col] = xBig[r*newsize+c];
    }
  }

 /*
  * Write enlarged, padded image to disk
  */
  byte2disk(y,padsize,padsize,argv[5]);

  return;
}
