/*
 * fLoG.c
 *
 *  Lapalcian-of-Gaussian edge detection filter for floating point images.
 * 
 * 8/30/96 jph
 *
 */

#include <stdio.h>
#include <math.h>
#include <fcntl.h>
#include <string.h>

#include "teager2D.h"

main(argc,argv) 
 
  int   argc;
  char *argv[];
 
{

  int       size;              /* row/col dim of image */
  int       so2;               /* size over 2 */
  int       N;                 /* size * size */
  char     *infn;              /* input filename */
  char     *outfn;             /* edge map output filename */
  char     *LoGfn;             /* LoG filter output filename */
  char     *Delfn;             /* LoG filter output filename */
  float     sigma;             /* Gaussian smoother space constant */
  float     sigsq;             /* sigma * sigma */
  float     thresh;            /* threshold for "significant" edges */
  float     Rsq;               /* squared radial distance from image center */
  float     Rsqosigsq;         /* Rsq / sigsq */
  float     Delx;              /* horiz component of gradient of y */
  float     Dely;              /* vert  component of gradient of y */
  float    *DelMag;            /* gradient magnitude of y */
  float     ftemp;             /* temporary float variable */
  float    *gReal;             /* real part of filter function */
  float    *gImag;             /* imag part of filter function */
  float    *GReal;             /* real part of FFT of filter function */
  float    *GImag;             /* imag part of FFT of filter function */
  float    *xReal;             /* Real part of input image */
  float    *xImag;             /* Imag part of input image */
  float    *XReal;             /* Real part of FFT of input image */
  float    *XImag;             /* Imag part of FFT of input image */
  float    *yReal;             /* Real part of output image */
  float    *yImag;             /* Imag part of output image */
  float    *YReal;             /* Real part of FFT of output image */
  float    *YImag;             /* Imag part of FFT of output image */
  BYTE     *ZC;                /* zero crossing edge map */
  int       i;                 /* loop counter */
  int       row;               /* image row counter */
  int       col;               /* image col counter */

#ifdef DEBUG
 /*
  * Link to PA so we can use it in the debugger
  */
  if (!argc) {
    PA((void *)xReal, 0, -1, 'b');
    PRINTA((void *)xReal,0,-1,'b',"delete.me");
    WRITEA((void *)xReal,0,-1,'b',"delete.me");
    image2disk((void *)xReal,1,1,'*',"delete.me");
    SpecMag2disk(xReal,"delete.me");
    PC(xReal,0,0,-1,'b',0);
    PRINTC(xReal,0,0,-1,'b',0,"delete.me");
    WRITEC(xReal,0,0,-1,'b',0,"delete.me");
  }
#endif

 /*
  * parse args
  */
  if (argc != 8) {
    printf("\n%s: LoG edge detection for float images.",argv[0]);
    printf("\n    No thresholding performed if thresh=0.");
    printf("\n    LoGfile='none' for no write.");
    printf("\n    Delfile='none' for no write.");
    printf("\n\nusage: %s size infile sigma thresh edgefile LoGfile Delfile\n",
           argv[0]);
    exit(0);
  }
  size   = atoi(argv[1]);
  so2 = size >> 1;
  N = size * size;
  infn = argv[2];
  sigma = atof(argv[3]);
  sigsq = sigma * sigma;
  thresh = atof(argv[4]);
  outfn = argv[5];
  LoGfn = argv[6];
  Delfn = argv[7];

 /*
  * Allocate working arrays
  */
  if ((xReal = (float *)malloc(N * sizeof(float))) == NULL) {
    printf("\n%s: free store exhausted.\n");
    exit(-1);
  }
  if ((xImag = (float *)malloc(N * sizeof(float))) == NULL) {
    printf("\n%s: free store exhausted.\n");
    exit(-1);
  }
  if ((XReal = (float *)malloc(N * sizeof(float))) == NULL) {
    printf("\n%s: free store exhausted.\n");
    exit(-1);
  }
  if ((XImag = (float *)malloc(N * sizeof(float))) == NULL) {
    printf("\n%s: free store exhausted.\n");
    exit(-1);
  }
  if ((yReal = (float *)malloc(N * sizeof(float))) == NULL) {
    printf("\n%s: free store exhausted.\n");
    exit(-1);
  }
  if ((yImag = (float *)malloc(N * sizeof(float))) == NULL) {
    printf("\n%s: free store exhausted.\n");
    exit(-1);
  }
  if ((YReal = (float *)malloc(N * sizeof(float))) == NULL) {
    printf("\n%s: free store exhausted.\n");
    exit(-1);
  }
  if ((YImag = (float *)malloc(N * sizeof(float))) == NULL) {
    printf("\n%s: free store exhausted.\n");
    exit(-1);
  }
  if ((gReal = (float *)malloc(N * sizeof(float))) == NULL) {
    printf("\n%s: free store exhausted.\n");
    exit(-1);
  }
  if ((gImag = (float *)malloc(N * sizeof(float))) == NULL) {
    printf("\n%s: free store exhausted.\n");
    exit(-1);
  }
  if ((GReal = (float *)malloc(N * sizeof(float))) == NULL) {
    printf("\n%s: free store exhausted.\n");
    exit(-1);
  }
  if ((GImag = (float *)malloc(N * sizeof(float))) == NULL) {
    printf("\n%s: free store exhausted.\n");
    exit(-1);
  }
  if ((DelMag = (float *)malloc(N * sizeof(float))) == NULL) {
    printf("\n%s: free store exhausted.\n");
    exit(-1);
  }
  if ((ZC = (BYTE *)malloc(N * sizeof(BYTE))) == NULL) {
    printf("\n%s: free store exhausted.\n");
    exit(-1);
  }
  for (i=0; i < N; i++) {
    yReal[i] = yImag[i] = YReal[i] = YImag[i] = xImag[i] = (float)0.0;
  }

 /*
  * Read Image
  */
  disk2f(xReal,size,size,infn);

 /*
  * Compute FFT
  */
  fft2d(xReal,xImag,XReal,XImag,size,1);

 /*
  * Compute image-centered filter function
  */
  for (row=0; row < size; row++) {
    for (col=0; col < size; col++) {
      Rsq = (float)((row-so2)*(row-so2) + (col-so2)*(col-so2));
      Rsqosigsq = Rsq / sigsq;
      gReal[row*size+col] = ((float)2.0 - Rsqosigsq) 
                            * (float)exp((double)-0.5*(double)Rsqosigsq);
      gImag[row*size+col] = (float)0.0;
    }
  }

 /*
  * transpose filter to "four corners" for zero phase
  */
  for (row=0; row < so2; row++) {
    for (col=0; col < so2; col++) {
      ftemp = gReal[row*size + col];
      gReal[row*size + col] = gReal[(row+so2)*size + col+so2];
      gReal[(row+so2)*size + col+so2] = ftemp;
    }
    for (col=so2; col < size; col++) {
      ftemp = gReal[row*size + col];
      gReal[row*size + col] = gReal[(row+so2)*size + col - so2];
      gReal[(row+so2)*size + col - so2] = ftemp;
    }
  }
  fft2d(gReal,gImag,GReal,GImag,size,1);

 /*
  * Perform the LoG filtering
  */
  for (i=0; i < N; i++) {
    fCMult(&YReal[i],&YImag[i],GReal[i],GImag[i],XReal[i],XImag[i]);
  }
  fft2d(yReal,yImag,YReal,YImag,size,-1);

 /*
  * Find ZC's
  */
  for (row=0; row < size; row++) {
    for (col=0; col < size; col++) {
      i = row*size + col;
      ZC[i] = 0;
     /*
      * Check for horizontal edge
      */
      if (col < size-1) {
        if ((yReal[i] < (float)0.0) && (yReal[i+1] > (float)0.0)) {
          ZC[i] = 255;
        } else {
          if ((yReal[i] > (float)0.0) && (yReal[i+1] < (float)0.0)) {
            ZC[i] = 255;
          } else {
            if (col > 0) {
              if ((yReal[i-1] < (float)0.0) && (yReal[i] == (float)0.0) 
                                        && (yReal[i+1] > (float)0.0)) {
                ZC[i] = 255;
              } else {
                if ((yReal[i-1] > (float)0.0) && (yReal[i] == (float)0.0)
                                          && (yReal[i+1] < (float)0.0)) {
                  ZC[i] = 255;
                }
              }
            }
          }
        }
      }
     /*
      * Check for vertical edge
      */
      if (row < size-1) {
        if ((yReal[i] < (float)0.0) && (yReal[i+size] > (float)0.0)) {
          ZC[i] = 255;
        } else {
          if ((yReal[i] > (float)0.0) && (yReal[i+size] < (float)0.0)) {
            ZC[i] = 255;
          } else {
            if (row > 0) {
              if ((yReal[i-size] < (float)0.0) && (yReal[i] == (float)0.0) 
                                           && (yReal[i+size] > (float)0.0)) {
                ZC[i] = 255;
              } else {
                if ((yReal[i-size] > (float)0.0) && (yReal[i] == (float)0.0)
                                             && (yReal[i+size] < (float)0.0)) {
                  ZC[i] = 255;
                }
              }
            }
          }
        }
      }
    }     /* for col */
  }       /* for row */

 /*
  * Threshold ZC's if thresh != 0
  */
  if (thresh > (float)0.0) {
    for (row=0; row < size; row++) {
      for (col=0; col < size; col++) {
        i = row*size + col;
        if (ZC[i] == 255) {
          if (col==0) {
            Delx = yReal[i+1] - yReal[i];
          } else {
            if (col==size-1) {
              Delx = yReal[i] - yReal[i-1];
            } else {
              Delx = (float)0.5 * (yReal[i+1] - yReal[i-1]);
            }
          }
          if (row==0) {
            Dely = yReal[i+size] - yReal[i];
          } else {
            if (row==size-1) {
              Dely = yReal[i] - yReal[i-size];
            } else {
              Dely = (float)0.5 * (yReal[i+size] - yReal[i-size]);
            }
          }
          DelMag[i] = fNorm(Delx,Dely);
          if (DelMag[i] < thresh) {
            ZC[i] = 0;
          }
        }    /* if ZC[i] == 255 */
      }      /* for col */
    }        /* for row */
  }          /* if thresholding */

 /*
  * Write edge map output file
  */
  byte2disk(ZC,size,size,outfn);

 /*
  * Write LoG filter response file
  */
  if (strcmp(LoGfn,"none")) {
    f2disk(yReal,size,size,LoGfn);
  }

 /*
  * Write gradient magnitude file
  */
  if (strcmp(Delfn,"none")) {
    f2disk(DelMag,size,size,Delfn);
  }

  printf("\n\n");
  exit(0);

}

