/*
 * fOvrlyMask.c 
 *
 *   Read a float image and a BYTE mask.  The mask should take only two
 *   values: 0 and 255.  Compress the dynamic range of the float image
 *   to seven bits.  Where the mask is zero, the output image is 128 + the
 *   compressed input image value.  Where the mask is 255, the output image is
 *   zero.
 */

#include <stdio.h>
#include "teager2D.h"

main(argc,argv)

   int   argc;
  char *argv[];

{

  float *x;              /* float input image */
  BYTE  *m;              /* byte mask image */
  BYTE  *y;              /* Result image */
  int    size;           /* row , col dim */
  int    N;              /* size * size */
  int    i;              /* loop counter */
  char  *xfn;            /* filename for input float image */
  char  *mfn;            /* filename for input BYTE mask */
  char  *yfn;            /* filename for BYTE output image */

 /*
  * parse args
  */
  if (argc != 5) {
    printf("\n\n%s: overlay mask image on float image. Result is BYTE.",
           argv[0]);
    printf("\nusage: %s size floatfile maskfile outfile \n",argv[0]);
    exit(0);
  }
  size = atoi(argv[1]);
  N = size * size;
  xfn = argv[2];
  mfn = argv[3];
  yfn = argv[4];

 /*
  * Allocate arrays and read
  */
  x = (float*) malloc(size*size*sizeof(float));
  m = (BYTE*)  malloc(size*size*sizeof(BYTE));

  disk2f(x,size,size,xfn);
  disk2byte(m,size,size,mfn);

 /*
  * Convert x to BYTE and halve dynamic range.  Overlay mask.
  */
  y = f_dbx(x,size,size);
  for (i=0; i < N; i++) {
    if (m[i] > 128) {
      y[i] = 255;
    } else {
      y[i] = (BYTE)((float)y[i]/(float)1.25);
    }
  }
    
  byte2disk(y,size,size,yfn);

  return;
}
