/*
 * fSaveFFTR.c
 *
 *   Make a byte image of the log magnitude spectrum of a REAL float image,
 *   display on the screen, and save to a disk file.  The FFT of the image is 
 *   computed, *   log[1 + Mag FFT] is computed, converted to a byte image, 
 *   and displayed.  User is prompted for file name to save spectrum image
 *   to.
 * 
 * 11/2/95 jph
 *
 */

#include <stdio.h>
#include <math.h>
#include <fcntl.h>
#include <string.h>

#include "teager2D.h"

main(argc,argv) 
 
  int   argc;
  char *argv[];
 
{

  int       size;              /* row/col dim of image */
  int       N;                 /* size * size */
  int       i;                 /* loop counter */
  float    *xReal;             /* Real part of image */
  float    *xImag;             /* Imag part of image */
  float    *XReal;             /* Real part of FFT */
  float    *XImag;             /* Imag part of FFT */
  float    *logMagFFT;         /* log[1 + Mag FFT] */
  char      cmd[256];          /* command string */
  char      fname[256];        /* filename string */

  if (argc != 3) {
    printf("\n%s: save log[1 + Mag FFT] of a REAL float image to file."
        ,argv[0]);
    printf("\nusage: %s size RealFile\n",argv[0]);
    exit(0);
  }
  size   = atoi(argv[1]);
  N = size * size;

 /*
  * Allocate working arrays
  */
  if ((xReal = (float *)malloc(N * sizeof(float))) == NULL) {
    printf("\n%s: free store exhausted.\n");
    exit(-1);
  }
  if ((xImag = (float *)malloc(N * sizeof(float))) == NULL) {
    printf("\n%s: free store exhausted.\n");
    exit(-1);
  }
  if ((XReal = (float *)malloc(N * sizeof(float))) == NULL) {
    printf("\n%s: free store exhausted.\n");
    exit(-1);
  }
  if ((XImag = (float *)malloc(N * sizeof(float))) == NULL) {
    printf("\n%s: free store exhausted.\n");
    exit(-1);
  }
  if ((logMagFFT = (float *)malloc(N * sizeof(float))) == NULL) {
    printf("\n%s: free store exhausted.\n");
    exit(-1);
  }

 /*
  * Read Image
  */
  disk2f(xReal,size,size,argv[2]);
  for (i=0; i < size*size; i++) {
    xImag[i] = (float)0.0;
  }

 /*
  * Compute log[1 + Mag FFT]
  */
  fft2d(xReal,xImag,XReal,XImag,size,1);
  for (i=0; i < N; i++) {
    logMagFFT[i] = (float)log((double)1.0 + (double)fNorm(XReal[i],XImag[i]));
  }

 /*
  * Display
  */
  f2disk(logMagFFT,size,size,"tmplog_FFT");
  sprintf(cmd,"fgrey %d tmplog_FFT",size);
  system(cmd);

 /*
  * Prompt for filename and save FFT image, if desired.
  */
  printf("\n\nFilename to save FFT image to ? (none for no write) ");
  scanf("%s",fname);
  if (strcmp(fname,"none")) {
    sprintf(cmd,"f2b %d tmplog_FFT %s",size,fname);
    system(cmd);
  }
  system("rm tmplog_FFT");

  printf("\n\n");
  exit(0);

}

