
/*
 * fSigCmprs.c
 *
 * Scaled log compression of outliers in image.
 *
 * USE OF fmed3x3 prior to this program is HIGHLY recommended.
 *
 * The threshold is set to "sigmas" times the std dev of the image.
 *
 * If a pixel deviates from the mean by more than thresh, then the ammount
 * by which it exceeds thresh will be logarithmically scaled.
 *
 * The log of the deviation is further scaled so that the maximum deviation from
 * the mean in the image is "sigmas" + 1 standard deviations.
 *
 * jph 4/5/96
 *
 */

#include <stdio.h>
#include <math.h>
#include <fcntl.h>
#include "teager2D.h"

main(argc,argv)

  int   argc;
  char *argv[];

{


  float  *x;                 /* input image  */
  int     size;              /* row , col dim */
  int     k;
  float   min,med,max;
  float   mean,sigma;
  float   Sigmas;
  double  Thresh;     
  double  deviation;
  double  MaxDev;            /* maximum deviation from mean in image */
  double  Fact;              /* scale factor, makes max dev Sigmas+1 sigmas */
 
  if (argc !=  5) {
    printf("\n\n%s: log compress image values too far off mean.",argv[0]);
    printf("\n          -scaling so that max deviation is sigmas+1 std dev's");
    printf("\n          -use of fmed3x3 first HIGHLY recommended!");
    printf("\nusage: %s size infile sigmas outfile\n",argv[0]);
    return;
  }
  size = atoi(argv[1]);
  Sigmas = atof(argv[3]);
  x = (float *) malloc(size*size*sizeof(float));
  disk2f(x,size,size,argv[2]);

  fsstat(x,size,size,&min,&med,&max,&mean,&sigma);

  Thresh = (double)Sigmas * (double)sigma;
  MaxDev = (double)0.0;
  Fact   = (double)1.0;

 /*
  * find the max deviation 
  */
  for (k=0; k < size*size; k++) {
    deviation = fabs((double)(x[k] - mean)); 
    if (deviation > Thresh) {
      if (deviation > MaxDev) {
        MaxDev = deviation;
      }
    }
  }
  if (MaxDev > (double)0.0000001) {
    Fact = ((double)(Sigmas + (float)1.0) * (double)sigma) 
             / log(MaxDev - Thresh + (double)1.0);
  }

  for (k=0; k < size*size; k++) {
    deviation = fabs((double)(x[k] - mean)); 
    if (deviation > Thresh) {
      deviation = Thresh + Fact * log(deviation - Thresh + 1.0);
      if (x[k] < mean) {
        x[k] = mean - deviation;
      } else {
        x[k] = mean + deviation;
      }
    }
  }

  f2disk(x,size,size,argv[4]);
  return;
} 
