/*
 * fUV2spiral.c 
 *
 *   read float images for horiz & vert components of instantaneous frequency.
 *   re-order to back and forth order.  output.  for use w/ matlab
 *   plot3 function.
 *
 *   jph 8/5/97
 */

#include <stdio.h>
#include "teager2D.h"

main(argc,argv)

   int   argc;
  char *argv[];

{

  int    size;           /* row , col dim */
  char  *inUfn;          /* input filename for horiz freq data */
  char  *inVfn;          /* input filename for vert  freq data */
  char  *outUfn;         /* input filename for horiz freq data */
  char  *outVfn;         /* input filename for vert  freq data */
  char  *Outfn;          /* output filename */
  float *U;              /* horizontal component of frequency vector */
  float *V;              /* vertical   component of frequency vector */
  float *U2;             /* U data re-ordered to back and forth order */
  float *V2;             /* V data re-ordered to back and forth order */
  int    row;            /* image row counter */
  int    col;            /* image col counter */
  int    i;              /* loop counter */

 /*
  * Parse args
  */
  if (argc != 6) {
    printf("\n\n%s: Re-order U,V to back & forth for 3D plot in Matlab.",
           argv[0]);
    printf("\nusage: %s size inUfn inVfn outUfn outVfn\n",argv[0]);
    exit(0);
  }
  size = atoi(argv[1]);
  inUfn = argv[2];
  inVfn = argv[3];
  outUfn = argv[4];
  outVfn = argv[5];

 /*
  * allocate arrays
  */
  if ((U = (float*) malloc(size*size*sizeof(float))) == NULL) {
    printf("\n%s: free store exhausted.\n",argv[0]);
    exit;
  }
  if ((V = (float*) malloc(size*size*sizeof(float))) == NULL) {
    printf("\n%s: free store exhausted.\n",argv[0]);
    exit;
  }
  if ((U2 = (float*) malloc(size*size*sizeof(float))) == NULL) {
    printf("\n%s: free store exhausted.\n",argv[0]);
    exit;
  }
  if ((V2 = (float*) malloc(size*size*sizeof(float))) == NULL) {
    printf("\n%s: free store exhausted.\n",argv[0]);
    exit;
  }

 /*
  * Read input files
  */
  disk2f(U,size,size,inUfn);
  disk2f(V,size,size,inVfn);

 /*
  * Re-order U and V into back and forth order
  */
  for (i=row=0; row < size; row++) {
    for (col=0; col < size; col++,i++) {
      U2[i] = U[row*size + col];
      V2[i] = V[row*size + col];
    }
    row++;
    if (row < size) {
      for (col=size-1; col >= 0; col--,i++) {
        U2[i] = U[row*size+col];
        V2[i] = V[row*size+col];
      }
    }
  }

 /*
  * Write re-ordered U,V data 
  */
  f2disk(U2,size,size,outUfn);
  f2disk(V2,size,size,outVfn);

  return;
}
