/*
 * fUV2spiral.c 
 *
 *   read float images for horiz & vert components of instantaneous frequency.
 *   output xyz triples: pix num, U, V, for 3D plotting of the instantaneous
 *   frequency trajectory.
 *
 *   jph 8/5/97
 */

#include <stdio.h>
#include "teager2D.h"

main(argc,argv)

   int   argc;
  char *argv[];

{

  int    size;           /* row , col dim */
  char  *Ufn;            /* input filename for horiz freq data */
  char  *Vfn;            /* input filename for vert  freq data */
  char  *Outfn;          /* output filename */
  float *U;              /* horizontal component of frequency vector */
  float *V;              /* vertical   component of frequency vector */
  float *U2;             /* U data re-ordered to back and forth order */
  float *V2;             /* V data re-ordered to back and forth order */
  float *OutDat;         /* output array for (pixnum,U,V) triples */
  int    row;            /* image row counter */
  int    col;            /* image col counter */
  int    i;              /* loop counter */
  int    j;              /* cursor into OutDat array */

 /*
  * Parse args
  */
  if (argc != 5) {
    printf("\n\n%s: Compute 3D plot of (n,U,V), where n is pixel number.",
           argv[0]);
    printf("\nusage: %s size Ufile.float Vfile.float outfile.float \n",argv[0]);
    exit(0);
  }
  size = atoi(argv[1]);
  Ufn = argv[2];
  Vfn = argv[3];
  Outfn = argv[4];

 /*
  * allocate arrays
  */
  if ((U = (float*) malloc(size*size*sizeof(float))) == NULL) {
    printf("\n%s: free store exhausted.\n",argv[0]);
    exit;
  }
  if ((V = (float*) malloc(size*size*sizeof(float))) == NULL) {
    printf("\n%s: free store exhausted.\n",argv[0]);
    exit;
  }
  if ((U2 = (float*) malloc(size*size*sizeof(float))) == NULL) {
    printf("\n%s: free store exhausted.\n",argv[0]);
    exit;
  }
  if ((V2 = (float*) malloc(size*size*sizeof(float))) == NULL) {
    printf("\n%s: free store exhausted.\n",argv[0]);
    exit;
  }
  if ((OutDat = (float*) malloc(3*size*size*sizeof(float))) == NULL) {
    printf("\n%s: free store exhausted.\n",argv[0]);
    exit;
  }

 /*
  * Read input files
  */
  disk2f(U,size,size,Ufn);
  disk2f(V,size,size,Vfn);

 /*
  * Re-order U and V into back and forth order
  */
  for (i=row=0; row < size; row++) {
    for (col=0; col < size; col++,i++) {
      U2[i] = U[row*size + col];
      V2[i] = V[row*size + col];
    }
    row++;
    if (row < size) {
      for (col=size-1; col >= 0; col--,i++) {
        U2[i] = U[row*size+col];
        V2[i] = V[row*size+col];
      }
    }
  }

 /*
  * Load re-ordered U,V data and pixel numbers into OutDat
  */
  for (i=j=0; i < size*size; i++) {
    OutDat[j++] = i;
    OutDat[j++] = U2[i];
    OutDat[j++] = V2[i];
  }

 /*
  * Write OutDat array
  */
  f2disk(OutDat,1,3*size*size,Outfn);

  return;
}
