/*
 * fWriteFFTR.c
 *
 *   Make a byte image of the log magnitude spectrum of a REAL float image
 *   and save to a disk file.  The FFT of the image is 
 *   computed,  log[1 + Mag FFT] is computed, converted to a byte image, 
 *   and saved to a file.
 * 
 * 4/17/96 jph
 *
 */

#include <stdio.h>
#include <math.h>
#include <fcntl.h>
#include <string.h>

#include "teager2D.h"

main(argc,argv) 
 
  int   argc;
  char *argv[];
 
{

  int       size;              /* row/col dim of image */
  int       N;                 /* size * size */
  int       i;                 /* loop counter */
  float    *xReal;             /* Real part of image */
  float    *xImag;             /* Imag part of image */
  float    *XReal;             /* Real part of FFT */
  float    *XImag;             /* Imag part of FFT */
  float    *logMagFFT;         /* log[1 + Mag FFT] */
  char     *fname;             /* filename string */
  BYTE     *y;                 /* byte output image */

  if (argc != 4) {
    printf("\n%s: write log[1 + Mag FFT] of a REAL float image to file."
        ,argv[0]);
    printf("\nusage: %s size RealInFile OutFile\n",argv[0]);
    exit(0);
  }
  size   = atoi(argv[1]);
  fname = argv[3];
  N = size * size;

 /*
  * Allocate working arrays
  */
  if ((xReal = (float *)malloc(N * sizeof(float))) == NULL) {
    printf("\n%s: free store exhausted.\n");
    exit(-1);
  }
  if ((xImag = (float *)malloc(N * sizeof(float))) == NULL) {
    printf("\n%s: free store exhausted.\n");
    exit(-1);
  }
  if ((XReal = (float *)malloc(N * sizeof(float))) == NULL) {
    printf("\n%s: free store exhausted.\n");
    exit(-1);
  }
  if ((XImag = (float *)malloc(N * sizeof(float))) == NULL) {
    printf("\n%s: free store exhausted.\n");
    exit(-1);
  }
  if ((logMagFFT = (float *)malloc(N * sizeof(float))) == NULL) {
    printf("\n%s: free store exhausted.\n");
    exit(-1);
  }

 /*
  * Read Image
  */
  disk2f(xReal,size,size,argv[2]);
  for (i=0; i < size*size; i++) {
    xImag[i] = (float)0.0;
  }

 /*
  * Compute log[1 + Mag FFT]
  */
  fft2d(xReal,xImag,XReal,XImag,size,1);
  for (i=0; i < N; i++) {
    logMagFFT[i] = (float)log((double)1.0 + (double)fNorm(XReal[i],XImag[i]));
  }

 /*
  * Save FFT image to byte file
  */
  y = f_dbx(logMagFFT,size,size);
  byte2disk(y,size,size,fname);

  printf("\n\n");
  exit(0);

}

