/*
 * fanatlyic.c
 *
 *  This program reads a real float image, computes the imaginary part of the
 *  analytic image, and computes the amplitude of the analytic image.
 *  The amplitude and imaginary component are written out to disk files.
 *  Specifying "none" as the filename for either output file supresses
 *  the write.
 *
 *  usage: fanalytic size real_file imag_file A_file.
 *
 *  jph 7/30/97
 *
 */

#ifndef FANALYTIC_C
#define FANALYTIC_C

#include <stdio.h>
#include <math.h>
#include <fcntl.h>
#include <string.h>

#include "teager2D.h"

/*
 * Function prototypes
 */
void Initialize();

main(argc,argv)

  int   argc;
  char *argv[];

{

  int                size;           /* row/col dim of the square input image */
  char              *Rfn;            /* filename for amplitude estimates */
  char              *Ifn;            /* filename for horiz freq estimates */
  char              *Afn;            /* filename for vert  freq estimates */
  pFloatImage        X;              /* input image */
  int                n;              /* size squared */
  float             *A;              /* A estimates */
  int                i;              /* counter */

 /*
  * Check for right number of arguments and parse args
  */
  if (argc != 5) {
    printf("\n\n%s: find imag part and amplitude of float analytic image.",
           argv[0]);
    printf("\n       enter none for either output file for no write.");
    printf("\nusage: %s size real_file imag_file A_file.\n",argv[0]);
    exit(-1);
  }
  size = atoi(argv[1]);
  Rfn  = argv[2];
  Ifn  = argv[3];
  Afn  = argv[4];

#ifdef DEBUG
 /*
  * Link to PA so we can use it in the debugger 
  */
  if (!argc) {
    PA((void *)X, 0, -1, 'b');
    PRINTA((void *)X,0,-1,'b',"delete.me");
    WRITEA((void *)X,0,-1,'b',"delete.me");
    image2disk((void *)X,1,1,'*',"delete.me");
    SpecMag2disk(X,"delete.me");
    PC(X,0,0,-1,'b',0);
    PRINTC(X,0,0,-1,'b',0,"delete.me");
    WRITEC(X,0,0,-1,'b',0,"delete.me");
  }
#endif

  X = (pFloatImage)malloc(sizeof(FloatImage));
  Initialize(&size,&X->Real,&X->Imag,&X->FFTReal,&X->FFTImag,Rfn);
  n = size * size;
  X->size = X->cols = X->rows = size;
  X->square = 1; 
  if ((A = (float *)malloc(X->size*X->size*sizeof(float))) == NULL) {
    printf("\n%s: free store exhausted.\n",argv[0]);
    exit(-1);
  }

 /*
  * Compute A
  */
  for (i=0; i < n; i++) {
    A[i] = fNorm(X->Real[i],X->Imag[i]);
  }

 /*
  * Write result files 
  */
  if (strcmp(Ifn,"none")) {
    f2disk(X->Imag,X->size,X->size,Ifn);
  }
  if (strcmp(Afn,"none")) {
    f2disk(A,X->size,X->size,Afn);
  }
  printf("\n\n");
  return;

} /*-------------------------------------------- main --------------*/

/*
 * Initialize
 *
 *  Read the input image, make hardy, compute the input spectrum
 */

void Initialize(size,xReal,xImag,XReal,XImag,Rfn)

  int         *size;             /* row/col dim of the square input image */
  float      **xReal;            /* real part of input image */
  float      **xImag;            /* imag part of input image */
  float      **XReal;            /* real part of input spectrum */
  float      **XImag;            /* imag part of input spectrum */
  char        *Rfn;              /* filename for real image */
{

 /*
  * Allocate Spectra
  */
  if ((*XReal = (float *)malloc(*size**size*sizeof(float))) == NULL) {
    printf("\nInitialize: free store exhausted\n");
    exit(-1);
  }
  if ((*XImag = (float *)malloc(*size**size*sizeof(float))) == NULL) {
    printf("\nInitialize: free store exhausted\n");
    exit(-1);
  }

 /*
  * Get the input image and its spectrum
  */
  *xReal = readasf(*size,*size,Rfn,'f');

  if ((*xImag = (float *)malloc(*size**size*sizeof(float))) == NULL) {
    printf("\nInitialize: free store exhausted\n");
    exit(-1);
  }
  MakeHardy(*xReal,*xImag,*XReal,*XImag,*size);

  return;
} /*---------------------------------------------- Initialize ----------*/
#endif
