/*
 * fft1d:  read a 1D ascii input signal, take magnitude fft, write out as ascii.
 * 
 *      see book VIII p. 101 for the array indexing.
 *
 *  USAGE: fft1d size infile outfile
 *
 *  10/3/94 jph
 */

#include <stdio.h>
#include <math.h>
#include "teager2D.h"
#include "Esa1D.h"

main(argc,argv)

  int   argc;
  char *argv[];
 
{

  float *xR;      /* real part of input data */
  float *xI;      /* imag part of input data */
  float *XR;      /* real part of fft */
  float *XI;      /* imag part of fft */
  float *XM;      /* magnitude of fft */
  float f;        /* frequency */
  float fincr;    /* increment in frequency */
  FILE  *ifp;     /* input  file pointer */
  FILE  *ofp;     /* output file pointer */
  int    size;    /* number of points in the input data sequence */
  int    i;       /* loop counter */
  int    dummy;   /* dummy location to read index into from file */

 /*
  * Check for the proper number of arguments 
  */
  if (argc != 4) {
    printf(
     "\n\n%s: Read a 1D ascii signal, take mag fft, write to ascii file.",
      argv[0]);
    printf("\nusage: %s size infile outfile\n", argv[0]);
    exit(0);
  }
  size = atoi(argv[1]);

 /*
  * Allocate working arrays 
  */
  xR = (float *) malloc(size*sizeof(float));
  xI = (float *) malloc(size*sizeof(float));
  XR = (float *) malloc(size*sizeof(float));
  XI = (float *) malloc(size*sizeof(float));
  XM = (float *) malloc(size*sizeof(float));

 /*
  * Read input data 
  */
  if (!(ifp = fopen(argv[2],"r"))) {
    printf("\nCould Not Open %s\n",argv[2]);
    return;
  }
  for (i=0; i < size; i++) {
    fscanf(ifp,"%d\t%f\n",&dummy,&xR[i]);
    xI[i] = (float)0.0;
  }
  fclose(ifp);

 /*
  * Take the fft and compute the magnitude 
  */
  fft1d(xR,xI,XR,XI,size,1);
  for (i=0; i < size; i++) {
    XM[i] = fNorm(XR[i],XI[i]);
  }

 /*
  * Write the magnitude data out to disk in ascii for Xvgr
  */
  if (!(ofp = fopen(argv[3],"w"))) {
    printf("\nCould Not Open %s\n",argv[3]);
    exit(0);
  }
  fincr = (float)1.0 / (float)size;
  for (f=(float)-0.5, i=0; i < size; i++, f+=fincr) {
    fprintf(ofp,"%f\t%f\n",f,XM[i]);
  }
  fclose(ofp);

  return(0);
}
