
/*
 * fhisto.c
 *
 *  construct a 1D ascii histogram of a float image, suitable for graphing
 *  by xvgr.
 *
 */

#ifndef FHISTO_C
#define FHISTO_C
#include <stdio.h>
#include <math.h>
#include <fcntl.h>
#include "teager2D.h"

main(argc,argv)

  int   argc;
  char *argv[];

{


  int          size;         /* row , col dim */
  int          x1;           /* x1,y1 defines upper left corner of image */
  int          y1;           /* x1,y1 defines upper left corner of image */
  int          x2;           /* x2,y2 defines lower right corner of image */
  int          y2;           /* x2,y2 defines lower right corner of image */
  int          Nbins;        /* num bins in histogram */
  int          Nrows;        /* Num rows in subimage being histogrammed */
  int          Ncols;        /* Num cols in subimage being histogrammed */
  int          N;            /* Num pixels in subimage being histogrammed */
  int          k;            /* loop counter */
  int          row,col;      /* loop counter */
  int         *H;            /* histogram values */
  int          Bin;          /* Number of a bin into which s[k] falls */
  float        min;          /* min value in s */
  float        max;          /* max value in s */
  float       *x;            /* input image */
  float       *s;            /* subimage to be histogramed */
  float       *domain;       /* domain pts for histogram */
  float        BinSize;      /* width of a bin in the histogram */
  float        one_o_BinSize;/* one over the BinSize */
  FILE        *fd;           /* file descriptor */
 
  if (argc !=  9) {
    printf("\n\n%s: Make ascii histogram of float image; can use with xvgr.",
           argv[0]);
    printf("\nusage: %s infile size x1 y1 x2 y2 Nbins outfile \n",argv[0]);
    return;
  }

 /*
  * Parse command line parameters
  */
  size  = atoi(argv[2]);
  x1    = atoi(argv[3]);
  y1    = atoi(argv[4]);
  x2    = atoi(argv[5]);
  y2    = atoi(argv[6]);
  Nbins = atoi(argv[7]);

 /*
  * Derived parameters
  */
  Nrows = y2 - y1 + 1;
  Ncols = x2 - x1 + 1;
  N = Nrows * Ncols;

 /*
  * Read input and extract subimage to be histogrammed
  */
  x = (float *) malloc(size*size*sizeof(float));
  disk2f(x,size,size,argv[1]);
  s = (float *) malloc(Nrows*Ncols*sizeof(float));
  for (k=0,row=y1; row <= y2; row++) {
    for (col=x1; col <= x2; col++,k++) {
      s[k] = x[row*size+col];
    }
  }

 /*
  * Get simple statistics and set up the histogram
  */
  min = max = s[0];
  for (k=1; k < N; k++) {
    if (s[k] < min) {
      min = s[k];
    } else {
      if (s[k] > max) {
        max = s[k];
      }
    }
  }
  H = (int *)malloc(Nbins*sizeof(int));
  domain = (float *) malloc(Nbins * sizeof(float));
  BinSize = (float)((double)(max-min) / (double)Nbins);
  one_o_BinSize = (float)1.0 / BinSize;
  for (k=0; k < Nbins; k++) {
    H[k] = 0;
    domain[k] = (float)0.5 * (((k+1)*BinSize + min) + (k*BinSize + min));
  }

 /*
  * Compute the histogram
  */
  for (k=0; k < N; k++) {
    Bin = (int)floor((double)((s[k] - min) * one_o_BinSize));
    if (Bin >= Nbins) Bin = Nbins - 1;
    H[Bin]++;
  }

 /*
  * Write out the histogram 
  */
  fd = fopen(argv[8],"w");
  for (k=0; k < Nbins; k++) {
    fprintf(fd,"%f\t%d\n",domain[k],H[k]);
  }
  fclose(fd);

  return;
} 
#endif
