/*
 * fshowfilt.c
 *
 *    makes float images of filter mag, real, imag, and spectrum mag.
 *
 *  PARAMTERS:
 *
 *    size       image row dim = col dim  in pixels
 *    r0         radial center freq of first filter on each ray
 *               in CYCLES PER IMAGE
 *    R          Common Ratio of radii of adjacent filters
 *    OBW        Eta-peak octave bandwidth of filters
 *    Eta        percent BW where adjacent filters intersect
 *    m          number of the filter to be displayed
 *    zerophase  1 for a zerophase filter, 0 for a filter centered in the image
 *
 */

#include <stdio.h>
#include <math.h>
#include <fcntl.h>
#include <string.h>

#include "teager2D.h"

#define BYTE unsigned char

main(argc,argv) 
 
  int   argc;
  char *argv[];
 
{

    int       size;
    double    r0;
    float     R;
    float     OBW;
    float     Eta;
    double    u0;
    double    v0;
    int       j;
    int       m;

    float *gReal;
    float *gImag;
    float *gMag;
    float *GMag;
    int zerophase;       
    int ofpr;                     /* ptr for binary output file */
    FILE *logfile;                /* ascii output file */
    filterbank *GB;               /* gabor filter bank for lib routines */


    if (argc != 12) {
      printf(
       "\n%s: make real, imag, and mag float images of a filter and spectrum."
                    ,argv[0]);
      printf("\nusage: %s %s \n",argv[0],
        "size r0 R OBW Eta m zerophase realfile imagfile magfile spectrumfile");
      exit(0);
    }
    size   = atoi(argv[1]);
    r0 = atof(argv[2]) / (double)size;
    R  = atof(argv[3]);
    OBW = (float)atof(argv[4]);
    Eta = (float)atof(argv[5]);
    m = atoi(argv[6]); 
    zerophase = atoi(argv[7]); 
    GB = CreateFilterBank(r0,R,OBW,Eta,1,(float)1.0,(float)1.0,(float)1.0);
    fMakeFilterImage(GB->filt[m],zerophase,size,&gReal,&gImag,&gMag);
    f2disk(gReal,size,size,argv[8]);
    f2disk(gImag,size,size,argv[9]);
    f2disk(gMag ,size,size,argv[10]);
    free(gReal);
    free(gImag);
    free(gMag);

    GMag = fMakeFilterSpectrum(GB->filt[m],size);
    f2disk(GMag ,size,size,argv[11]);
    free(GMag); 

    printf("\n\n");
    return;

}

