
/*
 * graphlambda.c
 *
 * graph lambda, the spacing between rays in the filterbank, as a function of
 * R, the common ratio of the geometric progression of filter radial center
 * frequencies along the ray.
 *
 * The equation for this is on p. 82 of book VIII.
 *
 * jph 10/29/94 
 *
 */

#ifndef GRAPHLAMBDA_C
#define GRAPHLAMBDA_C
#include <stdio.h>
#include <math.h>
#include <fcntl.h>
#include "teager2D.h"

main(argc,argv)

  int   argc;
  char *argv[];

{

  double    OBW;        /* octave bandwidth of filters */
  double    gamma;      /* defined on p. 83 of book VIII */
  double    R;          /* common ratio of center freq's */
  double    Rinc;       /* increment in R each time around loop */
  double   *Lambda;     /* angular spacing between rays */
  double    XX;         /* sine of lambda / 2 */
  double    YY;         /* intermediate quantity */
  int       size;       /* number of points in graph */
  int       i;          /* loop counter */
  FILE     *fp;         /* output file pointer */
 
  if (argc !=  3) {
    printf("\n\nusage: %s OBW outfile \n",argv[0]);
    return;
  }
  size = 1000;
  OBW = atof(argv[1]);
  gamma = pow((pow((double)2.0,OBW) - (double)1.0) / 
          (pow((double)2.0,OBW) + (double)1.0),(double)2.0);
  Rinc = (double)0.003;
  Lambda = (double *)malloc(size*sizeof(double));

  for (i=0, R=(double)0.0; i < size; i++, R+=Rinc) {

    YY = (R * R + 1.0) * (gamma - 1.0) + 2.0 * R * (gamma + 1.0);
    if (YY < (double)0.0) {
      Lambda[i] = (double)0.0;
    } else {
      XX =    pow((double)4.0*R, (double)-0.5) * pow(YY, 0.5);
      if ((XX < (double)-1.0) || (XX > (double)1.0)) {
        Lambda[i] = 0.0;
      } else {
        Lambda[i] = (double)360.0 * (double)2.0*asin(XX) / ((double)2.0 * M_PI);
      }
    }
  }

  if ((fp = fopen(argv[2],"w")) == NULL) {
    printf("\n%s: cannot open file %s for output.\n",argv[0],argv[2]);
    exit(-1);
  }
  for (i=0, R=(double)0.0; i < size; i++, R+=Rinc) {
    fprintf(fp,"%lf\t%lf\n",R,Lambda[i]);
  }
  fclose(fp);
  return;
} 
#endif
