/*
 * m_needle    
 *
 * read in u,v double files and make a needle diagram.
 * usage: m_needle ufile vfile size xhost
 *
 */

#ifndef M_NEEDLE_C
#define M_NEEDLE_C
#define NEEDLE_DENSITY 16
#include <stdio.h>
#include <math.h>
#include <fcntl.h>
/*#include <malloc.h>*/
#include "teager2D.h"

main(argc,argv)

  int   argc;
  char *argv[];

{

  unsigned char * domain; /* 1 where we want to put a needle */

  float * u;             /* normalized horizontal component of w */
  float * v;             /* normalized vertical component of w */
  float * uu;            /* normalized horizontal component of w */
  float * vv;            /* normalized vertical component of w */

  double theta;          /* temporary value */
  float  gain;           /* needle gain */
  float  mag;            /* needle magnitude */

  char cmd[100];         /* user command */

  int startrow;          /* first row to make needles for */
  int stoprow;           /* last row to make needles for */
  int startcol;          /* first row to make needles for */
  int stopcol;           /* last row to make needles for */

  int row,col;           /* loop counters */
  
  int choice;            /* user menu choice */
  int row_dim;           /* row dimension of image, specified on cmd line */
  int col_dim;           /* col dimension of image, specified on cmd line */
  int i,j,ii;            /* loop counter */
  int vision;            /* 1 if Xhost is vision, 0 for sausage */
  int needle_density;
  int heads;
  int squarelen;
 

/*malloc_debug(2);*/

 
  if ((argc !=  4) || ((row_dim = atoi(argv[3])) == 0)) {
    printf("\n\nusage: m_needle ufile.float vfile.float size\n");
    return;
  }
  col_dim = row_dim;
  needle_density = NEEDLE_DENSITY;
  heads =  0;
  startrow = startcol = 0;
  stoprow  = stopcol  = row_dim - 1;

 /*
  * Allocate domain, initialize it to control the density of the needles
  */
  domain = (unsigned char *) malloc(row_dim * col_dim * sizeof(unsigned char));
  for (i=0; i<row_dim * col_dim; i++) domain[i] = 0;
  for (i=needle_density-1; i<row_dim-needle_density; i+=needle_density)
    for (j=needle_density-1; j<col_dim-needle_density; j+=needle_density)
      domain[i*col_dim + j] = 1;

 /*
  * Read the u and v files
  */
  u      = (float *) malloc(row_dim*col_dim*sizeof(float));
  v      = (float *) malloc(row_dim*col_dim*sizeof(float));
  uu     = (float *) malloc(row_dim*col_dim*sizeof(float));
  vv     = (float *) malloc(row_dim*col_dim*sizeof(float));
  disk2f(u     ,row_dim,col_dim,argv[1]);
  disk2f(v     ,row_dim,col_dim,argv[2]);

choice=1;
while (choice) {
 /*
  * Show the dominant component needle diagram
  */
  printf("\nEnter Gain (< 0.0 for unity magnitude): ");
  scanf("%f",&gain);
  if (gain > (float)0.0) {
    printf("\nNeedle proportional to: 0=wavelength 1=freq ?");
    scanf("%d",&ii);
    printf("\nsquare the length: 0=No 1=Yes ?");
    scanf("%d",&squarelen);
  }
  printf("\nNeedle Density = %d, change 0=no 1=yes ?",needle_density);
  scanf("%d",&j);
  if (j) {
    printf("\nEnter New Needle Density: ");
    scanf("%d",&needle_density);
    for (i=0; i<row_dim * col_dim; i++) domain[i] = 0;
    for (i=needle_density-1; i<row_dim-needle_density; i+=needle_density)
      for (j=needle_density-1; j<col_dim-needle_density; j+=needle_density)
        domain[i*col_dim + j] = 1;
   }
   printf("\nPrint Arrow Heads? 0=no 1=yes");
   scanf("%d",&heads);

   printf("\n\nStart Row=%d\t Stop Row=%d\nStart Col=%d\t Stop Col=%d",
           startrow,stoprow,startcol,stopcol);
   printf("\nChange ?  0=No 1=Yes ");
   scanf("%d",&j);
   if (j) {
      printf("\nStartRow=%d\tNew Value? ",startrow);
      scanf("%d",&startrow); 
      printf("\nStartCol=%d\tNew Value? ",startcol);
      scanf("%d",&startcol); 
      printf("\nStopRow=%d\tNew Value? ",stoprow);
      scanf("%d",&stoprow); 
      printf("\nStopCol=%d\tNew Value? ",stopcol);
      scanf("%d",&stopcol); 
    }
 
  for(i=0;i<row_dim*col_dim;i++) {
    if ((u[i]==(float)0.0)&&(v[i]==(float)0.0)) theta = (double)0.0;
      else theta = atan2((double)v[i],(double)u[i]);
    if (gain > (float)0.0) {
      mag = u[i]*u[i]+v[i]*v[i];
      if(!squarelen)  mag = pow(mag,0.5);
      if ((!ii) && (mag != 0.0)) mag = 1.0 / mag;
      uu[i] = gain * mag *  (float)cos(theta);
      vv[i] = gain * mag *  (float)-1.0 * sin(theta);
    }
    else {
      uu[i] =  gain*cos(theta);
      vv[i] = -gain*sin(theta);
    }
  }

  dsply_needle(uu,vv,domain,row_dim,col_dim,heads,startrow,startcol,
                                                    stoprow, stopcol);

  printf("\n\nPRINT POSTSCRIPT NEEDLE DIAGRAM? 0=No 1=Yes ");
  scanf("%d",&i);getchar();
  if (i) print_needle(uu,vv,domain,row_dim,col_dim,heads,startrow,startcol,
                                                    stoprow, stopcol);
  printf("\n\nPRINT GNUFILE NEEDLE DIAGRAM? 0=No 1=Yes ");
  scanf("%d",&i);getchar();
  if (i) printgnu_needle(uu,vv,domain,row_dim,col_dim,heads,startrow,startcol,
                                                    stoprow, stopcol);
  printf("\nContinue ? 0=Quit 1=Continue: ");
  scanf("%d",&choice);
}



return;
} 
#endif
