/*
 * overlay
 *
 * reads a 256x256 binary image, divides dynamic range in half, 
 * expands to 512x512, reads in a 512x512 needle diagram, and
 * adds the two.
 *
 */

#ifndef OVERLAY_C
#define OVERLAY_C
#include <stdio.h>
#include <stdlib.h>
#include <math.h>
#include <fcntl.h>

main(argc,argv)

  int   argc;
  char *argv[];

{


  unsigned char x[256][256];     /* image to be read */
  unsigned char xx[512][512];         /* expanded x */
  unsigned char n[512][512];  /* needle diagram */
  unsigned char y[512][512];   /* overlay */
  int fd;               /* file descriptor */
  int n_bytes;          /* number of bytes to read */
  int start,stop;       /* loop limits */
  int i,j,ii,jj;        /* loop counters */
 
  if (argc !=  4) {
    printf("\n\nusage: overlay imagefile needlefile outfile\n");
    return;
  }


 /*
  * open the file image file
  */
  if ((fd = open(argv[1], O_RDONLY))==-1) {
    printf("\noverlay.c : could not open file %s !",argv[1]);
    return;
  }

 /*
  * read image data from the file
  */
  n_bytes = 256 * 256 * sizeof(unsigned char);
  if (read(fd,x,n_bytes) != n_bytes) {
    printf("\noverlay.c : complete disk read did not succeed.");
  }

 /*
  * close file 
  */
  if (close(fd) == -1) printf("\noverlay.c : error in closing file.");

 /*
  * cut dynamic range in half, make 512x512
  */
  for (i=ii=0; i<256; i++,ii+=2)
    for (j=jj=0; j<256; j++,jj+=2) 
      xx[ii][jj] = xx[ii+1][jj] = 
      xx[ii+1][jj+1] = xx[ii][jj+1] = x[i][j] / 2.0;
 /*
  * read the needle image
  */

  if ((fd = open(argv[2], O_RDONLY))==-1) {
    printf("\noverlay.c : could not open file %s !",argv[2]);
    return;
  }
  n_bytes = 512 * 512 * sizeof(unsigned char);
  if (read(fd,n,n_bytes) != n_bytes) {
    printf("\noverlay.c : complete disk read did not succeed.");
  }
  if (close(fd) == -1) printf("\noverlay.c : error in closing file.");

 /*
  * add the two images
  */
  for (i=0; i<512; i++)
    for (j=0; j<512; j++) y[i][j] = xx[i][j] | n[i][j];

 /*
  * write y
  */
  if ((fd = open(argv[3], O_WRONLY | O_CREAT, 0644))==-1) {
    printf("\npad800.c : could not open file %s!",argv[3]);
    return;
  }

 /*
  * write image data to the file
  */
  n_bytes = 512 * 512;
  if (write(fd,y,n_bytes) != n_bytes) {
    printf("\noverlay.c : complete disk write did not succeed.");
  }

 /*
  * close file and return
  */
  if (close(fd) == -1) printf("\noverlay.c : error in closing file.");
  return;
} 
#endif
