/*
 * pad.c
 *
 *  Read a 512x512 image. Center it in a black field of size specified by
 *   the first command line parameter.  Output to file specified by 3rd
 *   command line parameter.
 *
 *   usage: pad size infile outfile
 * 
 *
 */

#ifndef PAD
#define PAD
#include <stdio.h>
#include <math.h>
#include <fcntl.h>
#include <stdlib.h>
#define INSIZE 512
#ifndef BYTE
#define BYTE unsigned char
#endif

main(argc,argv)

  int   argc;
  char *argv[];

{


 /*
  * Local variable frame
  */
  unsigned char *x;     /* input image */
  unsigned char *y;     /* output image */
  int OutSize;          /* row and col dimension of output image */
  int fdin;             /* input  file descriptor */
  int fdou;             /* output file descriptor */
  int n_bytes;          /* number of bytes to read */
  int start,stop;       /* loop limits */
  int yrow, ycol;       /* loop counters for output image */
  int xrow, xcol;       /* loop counters for input image  */
 
 /*
  * Output banner
  */
  printf("\n%s: center 512x512 image in a black field of specified size.\n"
                                                               ,argv[0]);
 /*
  * Check for correct number of arguments
  */
  if (argc !=  4) {
    printf("\n\nusage: pad size infile outfile \n");
    return;
  }

 /*
  * Check for valid output size specification
  */
  if (!(OutSize = atoi(argv[1]))) {
    printf("\n%s: invalid size %s specified.\n",argv[0],argv[1]);
  }

 /*
  * Allocate Images on the heap
  */
  if ((x = (BYTE*) malloc(INSIZE * INSIZE * sizeof(BYTE))) == NULL) {
    printf("\n%s: free store exhausted allocating input image.",argv[0]);
  }
  if ((y = (BYTE*) malloc(OutSize * OutSize * sizeof(BYTE))) == NULL) {
    printf("\n%s: free store exhausted allocating output image.",argv[0]);
  }

 /*
  * Open the input file
  */
  if ((fdin = open(argv[2], O_RDONLY))==-1) {
    printf("\n%s: could not input open file %s !",argv[0],argv[2]);
    return;
  }

 /*
  * Read image data from the input file
  */
  n_bytes = INSIZE * INSIZE * sizeof(BYTE);
  if (read(fdin,x,n_bytes) != n_bytes) {
    printf("\n%s : complete disk read of %s did not succeed.",argv[0],argv[2]);
  }

 /*
  * Close the input file 
  */
  if (close(fdin) == -1) {
    printf("\n%s : error closing file %s.",argv[0],argv[2]);
  }

 /*
  * Initialize y
  */
  for (yrow=0; yrow < OutSize; yrow++) {
    for (ycol=0 ; ycol < OutSize; ycol++) {
      y[yrow*OutSize + ycol] = 0;
    }
  }

 /*
  * Center x in y
  */
  start = (OutSize - INSIZE) >> 1;
  stop  = start + INSIZE;
  for (yrow=start, xrow=0; yrow < stop; xrow++, yrow++) {
    for (ycol=start, xcol=0; ycol < stop; ycol++, xcol++) {
      y[yrow*OutSize + ycol] = x[xrow*INSIZE + xcol];
    }
  }

 /*
  * Open output file
  */
  if ((fdou = open(argv[3], O_WRONLY | O_CREAT, 0644))==-1) {
    printf("\n%s : could not open ouptut file %s!",argv[0],argv[3]);
    return;
  }

 /*
  * Write output image data to the output file
  */
  n_bytes = OutSize * OutSize;
  if (write(fdou,y,n_bytes) != n_bytes) {
    printf("\n%s : complete disk write of %s did not succeed.",argv[0],argv[3]);
  }

 /*
  * Close output file and return
  */
  if (close(fdou) == -1) {
    printf("\n%s : error in closing output file %s.",argv[0],argv[3]);
  }
  return;
} 
#endif
