/*
 * padxy.c
 *
 *  Pad a byte image to specified square size.  Input image does not have
 *  to be square.  
 *
 *  usage: padxy inrows incols infile outsize outfile
 *
 *  11/6/99 jph
 *
 */

#ifndef PADXY
#define PADXY
#include "teager2D.h"

main(argc,argv)

  int   argc;
  char *argv[];

{


 /*
  * Local variable frame
  */
  BYTE        *x;       /* input image */
  BYTE        *y;       /* output image */
  int          InRows;  /* num rows in input image */
  int          InCols;  /* num cols in input image */
  char        *InFile;  /* input filename */
  int          OutSize; /* num rows and cols in output image */
  char        *OutFile; /* output filename */
  int          Row;     /* row counter for output image */
  int          Col;     /* col counter for output image */
  int          InP;     /* cursor for input image */
  int          OutP;    /* cursor for output image */
 
 /*
  * Pars args
  */
  if (argc !=  6) {
    printf("\n\n%s: Pad a byte image to specified square size.",argv[0]);
    printf("\n\nusage: padxy inrows incols infile outsize outfile\n");
    exit(-1);
  }
  InRows = atoi(argv[1]);
  InCols = atoi(argv[2]);
  InFile = argv[3];
  OutSize = atoi(argv[4]);
  OutFile = argv[5];

 /*
  * Check for valid output size specification
  */
  if (OutSize < InRows) {
    printf("\n%s: invalid outsize %d < inrows=%d.\n",OutSize,InRows);
    exit(-1);
  }
  if (OutSize < InCols) {
    printf("\n%s: invalid outsize %d < incols=%d.\n",OutSize,InCols);
    exit(-1);
  }

 /*
  * Allocate Images on the heap
  */
  if ((x = (BYTE*)malloc(InRows * InCols * sizeof(BYTE))) == NULL) {
    printf("\n%s: free store exhausted allocating input image.",argv[0]);
    exit(-1);
  }
  if ((y = (BYTE*) malloc(OutSize * OutSize * sizeof(BYTE))) == NULL) {
    printf("\n%s: free store exhausted allocating output image.",argv[0]);
    exit(-1);
  }

 /*
  * Read input image.
  */
  disk2byte(x,InRows,InCols,InFile);

 /*
  * Make output image
  */
  for (InP=OutP=Row=0; Row < InRows; Row++) {
    for (Col=0; Col < InCols; Col++) {
      y[OutP++] = x[InP++];
    }
    for ( ; Col < OutSize; Col++) {
      y[OutP++] = (BYTE)0;
    }
  }
  for ( ; Row < OutSize; Row++) {
    y[OutP++] = (BYTE)0;
  }

 /*
  * Write output image
  */
  byte2disk(y,OutSize,OutSize,OutFile);


  return;
} 
#endif
