/*
 * temp.c
 *
 *    temporary hack program.
 *
 *    This one will make images of the new anisotropic postfilters...
 *
 *    see book XII, p. 9.
 *
 *    This program made by modifying showfilt.c
 *
 *
 *  PARAMTERS:
 *
 *    size       image row dim = col dim  in pixels
 *    r0         radial center freq of first filter on each ray
 *               in CYCLES PER IMAGE
 *    R          Common Ratio of radii of adjacent filters
 *    OBW        Eta-peak octave bandwidth of filters
 *    Eta        percent BW where adjacent filters intersect
 *    zerophase  1 for a zerophase filter, 0 for a filter centered in the image
 *
 */

#include <stdio.h>
#include <math.h>
#include <fcntl.h>
#include <string.h>

#include "teager2D.h"

#define BYTE unsigned char

main(argc,argv) 
 
  int   argc;
  char *argv[];
 
{

    int         size;          /* size of postfilter images */
    double      r0;            /* initial radia center freq, in CPI */
    float       R;             /* geometric ratio btwn cntr freq of adj filt*/
    float       OBW;           /* filter Eta-peak octave bandwidth */
    float       Eta;           /* OBW defined in terms of Eta-peak */
    float       kappa_x;       /* postfilter horiz space cnst scale factor */
    float       kappa_y;       /* postfilter vert  space cnst scale factor */
    int         k;             /* loop counter for filterbank channels */
    BYTE       *gReal;         /* Re part of channel filter */
    BYTE       *gImag;         /* Im part of channel filter */
    BYTE       *gMag;          /* Mag of channel filter */
    BYTE       *GMag;          /* Mag spectrum of channel filter */
    filterbank *GB;            /* gabor filter bank for lib routines */
    int         zerophase;     /* 1=zero phase images (split on cornerners)*/
    pFloatImage P;             /* postfilter image */
    char        TempFn[FNSIZE];/* temp filename */
    char        TempString[FNSIZE]; /* temp string for making unix cmds */

   /*
    * Parse command line arguments 
    */
    if (argc != 9) {
      printf("\n%s: test construction of anisotropic postfilters.\n",argv[0]);
      printf("\nusage: %s %s \n",argv[0],"size r0 R OBW Eta k_x k_y zerophase");
      exit(0);
    }
    size   = atoi(argv[1]);
    r0 = atof(argv[2]) / (double)size;
    R  = atof(argv[3]);
    OBW = (float)atof(argv[4]);
    Eta = (float)atof(argv[5]);
    kappa_x = (float)atof(argv[6]);
    kappa_y = (float)atof(argv[7]);
    zerophase = atoi(argv[8]); 

   /*
    * Create filterbank
    */
    GB = CreateFilterBank(r0,R,OBW,Eta,0,(float)1.0,kappa_x,kappa_y);

   /*
    * Loop on filters
    */
    for (k=0; k < GB->TotNumFilters; k++) {
      printf("\nDoing Channel %02d... ",k);
/*      bMakeFilterImage(GB->filt[k],zerophase,size,&gReal,&gImag,&gMag);*/
/*      GMag = bMakeFilterSpectrum(GB->filt[k],size);*/
      P = MkAnisPF(GB->filt[k]->sigma,GB->filt[k]->kappa_x,
                   GB->filt[k]->kappa_y,GB->filt[k]->Theta,size,zerophase);
/*      sprintf(TempFn,"g%02dR.bin",k);
      byte2disk(gReal,size,size,TempFn);
      sprintf(TempString,"grey %d %s",size,TempFn);
      system(TempString);
      sprintf(TempString,"rm %s",TempFn);
      system(TempString);*/

/*      sprintf(TempFn,"G%02dR.bin",k);
      byte2disk(GMag ,size,size,TempFn);
      sprintf(TempString,"grey %d %s",size,TempFn);
      system(TempString);
      sprintf(TempString,"rm %s",TempFn);
      system(TempString);*/

/*      sprintf(TempFn,"p%02dR.bin",k);
      image2disk(P->Real,size,size,'f',TempFn);
      sprintf(TempString,"grey %d %s",size,TempFn);
      system(TempString);
      sprintf(TempString,"rm %s",TempFn);
      system(TempString);*/

      sprintf(TempFn,"P%02dR.bin",k);
      SpecMag2disk(P->FFTReal,P->FFTImag,size,TempFn);
      sprintf(TempString,"grey %d %s",size,TempFn);
      system(TempString);
      sprintf(TempString,"rm %s",TempFn);
      system(TempString);

/*      free(gReal);
      free(gImag);
      free(gMag);
      free(GMag); */
      P = KillFloat(P);

/*      printf("    done.  Any key ");*/
/*      getchar();*/

    }

    printf("\n\n");
    return;

}

