/*
 * showbankDC.c
 *
 *    makes a special binary image of the filterbank that is scaled so
 *    that all filters take values from 128 to 255.  This version INCLUDES
 *    the DC filter described on p. 98 of book X.
 *
 *  PARAMTERS:
 *
 *    size       image row dim = col dim  in pixels
 *    r0         radial center freq of first filter on each ray 
 *               in CYCLES PER IMAGE
 *    R          Common Ratio of radii of adjacent filters
 *    OBW        Eta-peak octave bandwidth of filters
 *    Eta        percent BW where adjacent filters intersect
 *    DCR        Eta-peak contour of DC filter is at radial center frequency
 *               of the DCR'th ring of filters in the main bank.
 *
 *    11/3/95 jph
 */

#include "teager2D.h"

#define BYTE unsigned char

main(argc,argv) 

  int   argc;
  char *argv[];

{

  int       size;
  double    r0;
  float     R;
  float     OBW;
  float     Eta;
  int       DCR;
  BYTE     *GMag;

  if (argc != 7) {
    printf(
     "\n%s: Special showbank with DC filter.  All filters go from 128 to 255."
            ,argv[0]);
    printf("\nusage: %s size r0 R OBW Eta DCR\n",
             argv[0]);
    exit(0);
  }
  size = atoi(argv[1]);
  r0 = atof(argv[2]) / (double)size;
  R  = atof(argv[3]);
  OBW = (float)atof(argv[4]);
  Eta = (float)atof(argv[5]);
  DCR = atoi(argv[6]);
  GMag = SpbMkBankSpectrumDC(size,r0,R,OBW,Eta,(float)0.0,DCR);
  printf("\n\nPutting the result image to SPbankDC.dat...\n\n");
  byte2disk(GMag ,size,size,"SPbankDC.dat");

  printf("\n\n");
  return;

}

