/*
 * spshbank.c
 *
 *    makes a special binary image of the filterbank that is scaled so
 *    that all filters take values from 128 to 255.
 *
 *  PARAMTERS:
 *
 *    size       image row dim = col dim  in pixels
 *    r0         radial center freq of first filter on each ray 
 *               in CYCLES PER IMAGE
 *    R          Common Ratio of radii of adjacent filters
 *    OBW        Eta-peak octave bandwidth of filters
 *    Eta        percent BW where adjacent filters intersect
 *
 */

#include <stdio.h>
#include <math.h>
#include <fcntl.h>
#include <string.h>

#include "teager2D.h"

#define BYTE unsigned char

main(argc,argv) 

  int   argc;
  char *argv[];

{

  int       size;
  double    r0;
  float     R;
  float     OBW;
  float     Eta;
  BYTE     *GMag;

  if (argc != 6) {
    printf("\n%s: Special showbank.  All filters go from 128 to 255."
            ,argv[0]);
    printf("\nusage: %s size r0 R OBW Eta\n",
             argv[0]);
    exit(0);
  }
  size = atoi(argv[1]);
  r0 = atof(argv[2]) / (double)size;
  R  = atof(argv[3]);
  OBW = (float)atof(argv[4]);
  Eta = (float)atof(argv[5]);
  GMag = SpbMkBankSpectrum(size,r0,R,OBW,Eta,(float)0.0);
  printf("\n\nPutting the result image to SPbank.dat...\n\n");
  byte2disk(GMag ,size,size,"SPbank.dat");

  printf("\n\n");
  return;

}

